@@ -38,6 +38,7 @@
 
 #include "nsXFormsAccessible.h"
 
+#include "States.h"
 #include "nsAccessibilityService.h"
 #include "nsAccUtils.h"
 #include "nsTextEquivUtils.h"
@@ -149,58 +150,44 @@ nsXFormsAccessible::GetValue(nsAString& aValue)
   return sXFormsService->GetValue(DOMNode, aValue);
 }
 
-nsresult
-nsXFormsAccessible::GetStateInternal(PRUint32 *aState, PRUint32 *aExtraState)
+PRUint64
+nsXFormsAccessible::NativeState()
 {
-  NS_ENSURE_ARG_POINTER(aState);
-  *aState = 0;
-
-  if (IsDefunct()) {
-    if (aExtraState)
-      *aExtraState = nsIAccessibleStates::EXT_STATE_DEFUNCT;
-
-    return NS_OK_DEFUNCT_OBJECT;
-  }
-
-  if (aExtraState)
-    *aExtraState = 0;
-
-  NS_ENSURE_TRUE(sXFormsService, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(sXFormsService, 0);
 
   nsCOMPtr<nsIDOMNode> DOMNode(do_QueryInterface(mContent));
 
   PRBool isRelevant = PR_FALSE;
   nsresult rv = sXFormsService->IsRelevant(DOMNode, &isRelevant);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, 0);
 
   PRBool isReadonly = PR_FALSE;
   rv = sXFormsService->IsReadonly(DOMNode, &isReadonly);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, 0);
 
   PRBool isRequired = PR_FALSE;
   rv = sXFormsService->IsRequired(DOMNode, &isRequired);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, 0);
 
   PRBool isValid = PR_FALSE;
   rv = sXFormsService->IsValid(DOMNode, &isValid);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, 0);
 
-  rv = nsHyperTextAccessibleWrap::GetStateInternal(aState, aExtraState);
-  NS_ENSURE_SUCCESS(rv, rv);
+  PRUint64 states = nsHyperTextAccessibleWrap::NativeState();
 
   if (!isRelevant)
-    *aState |= nsIAccessibleStates::STATE_UNAVAILABLE;
+    states |= states::UNAVAILABLE;
 
   if (isReadonly)
-    *aState |= nsIAccessibleStates::STATE_READONLY;
+    states |= states::READONLY;
 
   if (isRequired)
-    *aState |= nsIAccessibleStates::STATE_REQUIRED;
+    states |= states::REQUIRED;
 
   if (!isValid)
-    *aState |= nsIAccessibleStates::STATE_INVALID;
+    states |= states::INVALID;
 
-  return NS_OK;
+  return states;
 }
 
 nsresult
@@ -210,21 +197,15 @@ nsXFormsAccessible::GetNameInternal(nsAString& aName)
   return GetBoundChildElementValue(NS_LITERAL_STRING("label"), aName);
 }
 
-NS_IMETHODIMP
-nsXFormsAccessible::GetDescription(nsAString& aDescription)
+void
+nsXFormsAccessible::Description(nsString& aDescription)
 {
-  nsAutoString description;
-  nsresult rv = nsTextEquivUtils::
+  nsTextEquivUtils::
     GetTextEquivFromIDRefs(this, nsAccessibilityAtoms::aria_describedby,
-                           description);
+                           aDescription);
 
-  if (NS_SUCCEEDED(rv) && !description.IsEmpty()) {
-    aDescription = description;
-    return NS_OK;
-  }
-
-  // search the xforms:hint element
-  return GetBoundChildElementValue(NS_LITERAL_STRING("hint"), aDescription);
+  if (aDescription.IsEmpty())
+    GetBoundChildElementValue(NS_LITERAL_STRING("hint"), aDescription);
 }
 
 PRBool
@@ -267,45 +248,37 @@ nsXFormsEditableAccessible::
 {
 }
 
-nsresult
-nsXFormsEditableAccessible::GetStateInternal(PRUint32 *aState,
-                                             PRUint32 *aExtraState)
+PRUint64
+nsXFormsEditableAccessible::NativeState()
 {
-  NS_ENSURE_ARG_POINTER(aState);
-
-  nsresult rv = nsXFormsAccessible::GetStateInternal(aState, aExtraState);
-  NS_ENSURE_A11Y_SUCCESS(rv, rv);
-
-  if (!aExtraState)
-    return NS_OK;
+  PRUint64 state = nsXFormsAccessible::NativeState();
 
   nsCOMPtr<nsIDOMNode> DOMNode(do_QueryInterface(mContent));
 
   PRBool isReadonly = PR_FALSE;
-  rv = sXFormsService->IsReadonly(DOMNode, &isReadonly);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv = sXFormsService->IsReadonly(DOMNode, &isReadonly);
+  NS_ENSURE_SUCCESS(rv, state);
 
   if (!isReadonly) {
     PRBool isRelevant = PR_FALSE;
     rv = sXFormsService->IsRelevant(DOMNode, &isRelevant);
-    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_SUCCESS(rv, state);
     if (isRelevant) {
-      *aExtraState |= nsIAccessibleStates::EXT_STATE_EDITABLE |
-                      nsIAccessibleStates::EXT_STATE_SELECTABLE_TEXT;
+      state |= states::EDITABLE | states::SELECTABLE_TEXT;
     }
   }
 
   nsCOMPtr<nsIEditor> editor;
   GetAssociatedEditor(getter_AddRefs(editor));
-  NS_ENSURE_TRUE(editor, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(editor, state);
   PRUint32 flags;
   editor->GetFlags(&flags);
   if (flags & nsIPlaintextEditor::eEditorSingleLineMask)
-    *aExtraState |= nsIAccessibleStates::EXT_STATE_SINGLE_LINE;
+    state |= states::SINGLE_LINE;
   else
-    *aExtraState |= nsIAccessibleStates::EXT_STATE_MULTI_LINE;
+    state |= states::MULTI_LINE;
 
-  return NS_OK;
+  return state;
 }
 
 NS_IMETHODIMP
@@ -435,7 +408,6 @@ nsXFormsSelectableAccessible::RemoveItemFromSelection(PRUint32 aIndex)
   if (!itemDOMNode)
     return false;
 
-  nsresult rv;
   nsCOMPtr<nsIDOMNode> DOMNode(do_QueryInterface(mContent));
   if (mIsSelect1Element) {
     nsCOMPtr<nsIDOMNode> selItemDOMNode;
@@ -490,7 +462,6 @@ nsXFormsSelectableAccessible::IsItemSelected(PRUint32 aIndex)
   if (!itemDOMNode)
     return false;
 
-  nsresult rv;
   nsCOMPtr<nsIDOMNode> DOMNode(do_QueryInterface(mContent));
   if (mIsSelect1Element) {
     nsCOMPtr<nsIDOMNode> selItemDOMNode;
@@ -591,8 +562,8 @@ nsXFormsSelectableItemAccessible::DoAction(PRUint8 aIndex)
   return NS_OK;
 }
 
-PRBool
-nsXFormsSelectableItemAccessible::IsItemSelected()
+bool
+nsXFormsSelectableItemAccessible::IsSelected()
 {
   nsresult rv;
 
