@@ -128,10 +128,28 @@ let UI = {
   _storageBusyCount: 0,
 
   // Variable: isDOMWindowClosing
-  // Tells wether we already received the "domwindowclosed" event and the parent
-  // windows is about to close.
+  // Tells wether the parent window is about to close
   isDOMWindowClosing: false,
 
+  // Variable: _browserKeys
+  // Used to keep track of allowed browser keys.
+  _browserKeys: null,
+
+  // Variable: ignoreKeypressForSearch
+  // Used to prevent keypress being handled after quitting search mode.
+  ignoreKeypressForSearch: false,
+
+  // Variable: _lastOpenedTab
+  // Used to keep track of the last opened tab.
+  _lastOpenedTab: null,
+
+  // ----------
+  // Function: toString
+  // Prints [UI] for debug use
+  toString: function UI_toString() {
+    return "[UI]";
+  },
+
   // ----------
   // Function: init
   // Must be called after the object is created.
@@ -142,17 +160,15 @@ let UI = {
       // initialize the direction of the page
       this._initPageDirection();
 
+      // ___ thumbnail storage
+      ThumbnailStorage.init();
+
       // ___ storage
       Storage.init();
       let data = Storage.readUIData(gWindow);
       this._storageSanity(data);
       this._pageBounds = data.pageBounds;
 
-      // ___ hook into the browser
-      gWindow.addEventListener("tabviewshow", function() {
-        self.showTabView(true);
-      }, false);
-
       // ___ currentTab
       this._currentTab = gBrowser.selectedTab;
 
@@ -173,36 +189,35 @@ let UI = {
           });
         }
         if (e.originalTarget.id == "content") {
-          // Create an orphan tab on double click
-          if (Date.now() - self._lastClick <= self.DBLCLICK_INTERVAL && 
-              (self._lastClickPositions.x - self.DBLCLICK_OFFSET) <= e.clientX &&
-              (self._lastClickPositions.x + self.DBLCLICK_OFFSET) >= e.clientX &&
-              (self._lastClickPositions.y - self.DBLCLICK_OFFSET) <= e.clientY &&
-              (self._lastClickPositions.y + self.DBLCLICK_OFFSET) >= e.clientY) {
-            GroupItems.setActiveGroupItem(null);
-            TabItems.creatingNewOrphanTab = true;
-
-            let newTab = 
-              gBrowser.loadOneTab("about:blank", { inBackground: true });
-
-            let box = 
-              new Rect(e.clientX - Math.floor(TabItems.tabWidth/2),
-                       e.clientY - Math.floor(TabItems.tabHeight/2),
-                       TabItems.tabWidth, TabItems.tabHeight);
-            newTab._tabViewTabItem.setBounds(box, true);
-            newTab._tabViewTabItem.pushAway(true);
-            GroupItems.setActiveOrphanTab(newTab._tabViewTabItem);
-
-            TabItems.creatingNewOrphanTab = false;
-            newTab._tabViewTabItem.zoomIn(true);
-
+          if (!Utils.isLeftClick(e)) {
             self._lastClick = 0;
             self._lastClickPositions = null;
-            gTabView.firstUseExperienced = true;
           } else {
-            self._lastClick = Date.now();
-            self._lastClickPositions = new Point(e.clientX, e.clientY);
-            self._createGroupItemOnDrag(e);
+            // Create a group with one tab on double click
+            if (Date.now() - self._lastClick <= self.DBLCLICK_INTERVAL && 
+                (self._lastClickPositions.x - self.DBLCLICK_OFFSET) <= e.clientX &&
+                (self._lastClickPositions.x + self.DBLCLICK_OFFSET) >= e.clientX &&
+                (self._lastClickPositions.y - self.DBLCLICK_OFFSET) <= e.clientY &&
+                (self._lastClickPositions.y + self.DBLCLICK_OFFSET) >= e.clientY) {
+
+              let box =
+                new Rect(e.clientX - Math.floor(TabItems.tabWidth/2),
+                         e.clientY - Math.floor(TabItems.tabHeight/2),
+                         TabItems.tabWidth, TabItems.tabHeight);
+              box.inset(-30, -30);
+
+              let opts = {immediately: true, bounds: box};
+              let groupItem = new GroupItem([], opts);
+              groupItem.newTab();
+
+              self._lastClick = 0;
+              self._lastClickPositions = null;
+              gTabView.firstUseExperienced = true;
+            } else {
+              self._lastClick = Date.now();
+              self._lastClickPositions = new Point(e.clientX, e.clientY);
+              self._createGroupItemOnDrag(e);
+            }
           }
         }
       });
@@ -211,10 +226,6 @@ let UI = {
         self.uninit();
       });
 
-      gWindow.addEventListener("tabviewhide", function() {
-        self.exit();
-      }, false);
-
       // ___ setup key handlers
       this._setTabViewFrameKeyHandlers();
 
@@ -243,21 +254,19 @@ let UI = {
         self._resize();
       });
 
-      // ___ setup observer to save canvas images
-      function domWinClosedObserver(subject, topic, data) {
-        if (topic == "domwindowclosed" && subject == gWindow) {
-          self.isDOMWindowClosing = true;
-          if (self.isTabViewVisible())
-            GroupItems.removeHiddenGroups();
-          TabItems.saveAll(true);
-          self._save();
-        }
-      }
-      Services.obs.addObserver(
-        domWinClosedObserver, "domwindowclosed", false);
-      this._cleanupFunctions.push(function() {
-        Services.obs.removeObserver(domWinClosedObserver, "domwindowclosed");
-      });
+      // ___ setup event listener to save canvas images
+      gWindow.addEventListener("SSWindowClosing", function onWindowClosing() {
+        gWindow.removeEventListener("SSWindowClosing", onWindowClosing, false);
+
+        self.isDOMWindowClosing = true;
+
+        if (self.isTabViewVisible())
+          GroupItems.removeHiddenGroups();
+
+        Storage.saveActiveGroupName(gWindow);
+        TabItems.saveAll(true);
+        self._save();
+      }, false);
 
       // ___ Done
       this._frameInitialized = true;
@@ -267,7 +276,7 @@ let UI = {
       // initialized.
       let event = document.createEvent("Events");
       event.initEvent("tabviewframeinitialized", true, false);
-      dispatchEvent(event);      
+      dispatchEvent(event);
     } catch(e) {
       Utils.log(e);
     } finally {
@@ -275,6 +284,8 @@ let UI = {
     }
   },
 
+  // Function: uninit
+  // Should be called when window is unloaded.
   uninit: function UI_uninit() {
     // call our cleanup functions
     this._cleanupFunctions.forEach(function(func) {
@@ -286,6 +297,7 @@ let UI = {
     TabItems.uninit();
     GroupItems.uninit();
     Storage.uninit();
+    ThumbnailStorage.uninit();
 
     this._removeTabActionHandlers();
     this._currentTab = null;
@@ -338,12 +350,12 @@ let UI = {
         item.parent.remove(item);
       groupItem.add(item, {immediately: true});
     });
-    GroupItems.setActiveGroupItem(groupItem);
+    this.setActive(groupItem);
   },
 
+  // ----------
   // Function: blurAll
   // Blurs any currently focused element
-  //
   blurAll: function UI_blurAll() {
     iQ(":focus").each(function(element) {
       element.blur();
@@ -365,13 +377,12 @@ let UI = {
   // ----------
   // Function: getActiveTab
   // Returns the currently active tab as a <TabItem>
-  //
   getActiveTab: function UI_getActiveTab() {
     return this._activeTab;
   },
 
   // ----------
-  // Function: setActiveTab
+  // Function: _setActiveTab
   // Sets the currently active tab. The idea of a focused tab is useful
   // for keyboard navigation and returning to the last zoomed-in tab.
   // Hitting return/esc brings you to the focused tab, and using the
@@ -379,7 +390,7 @@ let UI = {
   //
   // Parameters:
   //  - Takes a <TabItem>
-  setActiveTab: function UI_setActiveTab(tabItem) {
+  _setActiveTab: function UI__setActiveTab(tabItem) {
     if (tabItem == this._activeTab)
       return;
 
@@ -393,18 +404,41 @@ let UI = {
       let self = this;
       this._activeTab.addSubscriber(this, "close", function(closedTabItem) {
         if (self._activeTab == closedTabItem)
-          self.setActiveTab(null);
+          self._setActiveTab(null);
       });
 
       this._activeTab.makeActive();
     }
   },
 
+  // ----------
+  // Function: setActive
+  // Sets the active tab item or group item
+  // Parameters:
+  //
+  // options
+  //  dontSetActiveTabInGroup bool for not setting active tab in group
+  setActive: function UI_setActive(item, options) {
+    Utils.assert(item, "item must be given");
+
+    if (item.isATabItem) {
+      GroupItems.setActiveGroupItem(item.parent);
+      this._setActiveTab(item);
+    } else {
+      GroupItems.setActiveGroupItem(item);
+      if (!options || !options.dontSetActiveTabInGroup) {
+        let activeTab = item.getActiveTab()
+        if (activeTab)
+          this._setActiveTab(activeTab);
+      }
+    }
+  },
+
   // ----------
   // Function: isTabViewVisible
   // Returns true if the TabView UI is currently shown.
   isTabViewVisible: function UI_isTabViewVisible() {
-    return gTabViewDeck.selectedIndex == 1;
+    return gTabViewDeck.selectedPanel == gTabViewFrame;
   },
 
   // ---------
@@ -443,7 +477,7 @@ let UI = {
     // Restore the full height when showing TabView
     gTabViewFrame.style.marginTop = "";
 #endif
-    gTabViewDeck.selectedIndex = 1;
+    gTabViewDeck.selectedPanel = gTabViewFrame;
     gWindow.TabsInTitlebar.allowedBy("tabview-open", false);
     gTabViewFrame.contentWindow.focus();
 
@@ -456,17 +490,6 @@ let UI = {
 
     Storage.saveVisibilityData(gWindow, "true");
 
-    // Close the active group if it was empty. This will happen when the
-    // user returns to Panorama after looking at an app tab, having
-    // closed all other tabs. (If the user is looking at an orphan tab, then
-    // there is no active group for the purposes of this check.)
-    let activeGroupItem = null;
-    if (!GroupItems.getActiveOrphanTab()) {
-      activeGroupItem = GroupItems.getActiveGroupItem();
-      if (activeGroupItem && activeGroupItem.closeIfEmpty())
-        activeGroupItem = null;
-    }
-
     if (zoomOut && currentTab && currentTab._tabViewTabItem) {
       item = currentTab._tabViewTabItem;
       // If there was a previous currentTab we want to animate
@@ -478,10 +501,7 @@ let UI = {
         if (!currentTab._tabViewTabItem) // if the tab's been destroyed
           item = null;
 
-        self.setActiveTab(item);
-
-        if (activeGroupItem && item.parent)
-          activeGroupItem.setTopChild(item);
+        self.setActive(item);
 
         self._resize(true);
         dispatchEvent(event);
@@ -492,7 +512,6 @@ let UI = {
         TabItems.resumePainting();
       });
     } else {
-      self.setActiveTab(null);
       dispatchEvent(event);
 
       // Flush pending updates
@@ -500,6 +519,9 @@ let UI = {
 
       TabItems.resumePainting();
     }
+
+    if (gTabView.firstUseExperienced)
+      gTabView.enableSessionRestore();
   },
 
   // ----------
@@ -525,7 +547,7 @@ let UI = {
     // as well as avoiding the flash of black as we animate out
     gTabViewFrame.style.marginTop = gBrowser.boxObject.y + "px";
 #endif
-    gTabViewDeck.selectedIndex = 0;
+    gTabViewDeck.selectedPanel = gBrowserPanel;
     gWindow.TabsInTitlebar.allowedBy("tabview-open", true);
     gBrowser.contentWindow.focus();
 
@@ -625,22 +647,22 @@ let UI = {
     // don't reenter Panorama due to all of the session restore tab
     // manipulation (which otherwise we might). When transitioning away from
     // PB, we reenter Panorama if we had been there directly before PB.
-    function pbObserver(aSubject, aTopic, aData) {
-      if (aTopic == "private-browsing") {
+    function pbObserver(subject, topic, data) {
+      if (topic == "private-browsing") {
         // We could probably do this in private-browsing-change-granted, but
         // this seems like a nicer spot, right in the middle of the process.
-        if (aData == "enter") {
+        if (data == "enter") {
           // If we are in Tab View, exit. 
           self._privateBrowsing.wasInTabView = self.isTabViewVisible();
           if (self.isTabViewVisible())
             self.goToTab(gBrowser.selectedTab);
         }
-      } else if (aTopic == "private-browsing-change-granted") {
-        if (aData == "enter" || aData == "exit") {
-          self._privateBrowsing.transitionMode = aData;
+      } else if (topic == "private-browsing-change-granted") {
+        if (data == "enter" || data == "exit") {
+          self._privateBrowsing.transitionMode = data;
           self.storageBusy();
         }
-      } else if (aTopic == "private-browsing-transition-complete") {
+      } else if (topic == "private-browsing-transition-complete") {
         // We use .transitionMode here, as aData is empty.
         if (self._privateBrowsing.transitionMode == "exit" &&
             self._privateBrowsing.wasInTabView)
@@ -669,6 +691,8 @@ let UI = {
       // if it's an app tab, add it to all the group items
       if (tab.pinned)
         GroupItems.addAppTab(tab);
+      else if (self.isTabViewVisible() && !self._storageBusyCount)
+        self._lastOpenedTab = tab;
     };
     
     // TabClose
@@ -687,15 +711,14 @@ let UI = {
       } else {
         // If we're currently in the process of entering private browsing,
         // we don't want to go to the Tab View UI. 
-        if (self._privateBrowsing.transitionMode)
-          return; 
-          
+        if (self._storageBusyCount)
+          return;
+
         // if not closing the last tab
         if (gBrowser.tabs.length > 1) {
           // Don't return to TabView if there are any app tabs
-          for (let a = 0; a < gBrowser.tabs.length; a++) {
-            let theTab = gBrowser.tabs[a]; 
-            if (theTab.pinned && gBrowser._removingTabs.indexOf(theTab) == -1) 
+          for (let a = 0; a < gBrowser._numPinnedTabs; a++) {
+            if (!gBrowser.tabs[a].closing)
               return;
           }
 
@@ -716,8 +739,9 @@ let UI = {
           // 3) When a blank tab is active while restoring a closed tab the
           // blank tab gets removed. The active group is not closed as this is
           // where the restored tab goes. So do not show the TabView.
+          let tabItem = tab && tab._tabViewTabItem;
           let closingBlankTabAfterRestore =
-            (tab && tab._tabViewTabIsRemovedAfterRestore);
+            (tabItem && tabItem.isRemovedAfterRestore);
 
           if ((closingLastOfGroup || closingUnnamedGroup) &&
               !closingBlankTabAfterRestore) {
@@ -734,9 +758,16 @@ let UI = {
       if (tab.ownerDocument.defaultView != gWindow)
         return;
 
-      let activeGroupItem = GroupItems.getActiveGroupItem();
-      if (activeGroupItem)
-        self.setReorderTabItemsOnShow(activeGroupItem);
+      if (GroupItems.groupItems.length > 0) {
+        if (tab.pinned) {
+          if (gBrowser._numPinnedTabs > 1)
+            GroupItems.arrangeAppTab(tab);
+        } else {
+          let activeGroupItem = GroupItems.getActiveGroupItem();
+          if (activeGroupItem)
+            self.setReorderTabItemsOnShow(activeGroupItem);
+        }
+      }
     };
 
     // TabSelect
@@ -797,30 +828,39 @@ let UI = {
   // Function: onTabSelect
   // Called when the user switches from one tab to another outside of the TabView UI.
   onTabSelect: function UI_onTabSelect(tab) {
-    let currentTab = this._currentTab;
     this._currentTab = tab;
 
-    // if the last visible tab has just been closed, don't show the chrome UI.
-    if (this.isTabViewVisible() &&
-        (this._closedLastVisibleTab || this._closedSelectedTabInTabView ||
-         this.restoredClosedTab)) {
-      if (this.restoredClosedTab) {
-        // when the tab view UI is being displayed, update the thumb for the 
-        // restored closed tab after the page load
-        tab.linkedBrowser.addEventListener("load", function (event) {
-          tab.linkedBrowser.removeEventListener("load", arguments.callee, true);
-          TabItems._update(tab);
-        }, true);
+    if (this.isTabViewVisible()) {
+      if (!this.restoredClosedTab && this._lastOpenedTab == tab && 
+        tab._tabViewTabItem) {
+        tab._tabViewTabItem.zoomIn(true);
+        this._lastOpenedTab = null;
+        return;
+      }
+      if (this._closedLastVisibleTab ||
+          (this._closedSelectedTabInTabView && !this.closedLastTabInTabView) ||
+          this.restoredClosedTab) {
+        if (this.restoredClosedTab) {
+          // when the tab view UI is being displayed, update the thumb for the 
+          // restored closed tab after the page load
+          tab.linkedBrowser.addEventListener("load", function (event) {
+            tab.linkedBrowser.removeEventListener("load", arguments.callee, true);
+            TabItems._update(tab);
+          }, true);
+        }
+        this._closedLastVisibleTab = false;
+        this._closedSelectedTabInTabView = false;
+        this.closedLastTabInTabView = false;
+        this.restoredClosedTab = false;
+        return;
       }
-      this._closedLastVisibleTab = false;
-      this._closedSelectedTabInTabView = false;
-      this.restoredClosedTab = false;
-      return;
     }
     // reset these vars, just in case.
     this._closedLastVisibleTab = false;
     this._closedSelectedTabInTabView = false;
+    this.closedLastTabInTabView = false;
     this.restoredClosedTab = false;
+    this._lastOpenedTab = null;
 
     // if TabView is visible but we didn't just close the last tab or
     // selected tab, show chrome.
@@ -832,12 +872,7 @@ let UI = {
     if (this._currentTab != tab)
       return;
 
-    let oldItem = null;
     let newItem = null;
-
-    if (currentTab && currentTab._tabViewTabItem)
-      oldItem = currentTab._tabViewTabItem;
-
     // update the tab bar for the new tab's group
     if (tab && tab._tabViewTabItem) {
       if (!TabItems.reconnectingPaused()) {
@@ -846,24 +881,20 @@ let UI = {
       }
     } else {
       // No tabItem; must be an app tab. Base the tab bar on the current group.
-      // If no current group or orphan tab, figure it out based on what's
-      // already in the tab bar.
-      if (!GroupItems.getActiveGroupItem() && !GroupItems.getActiveOrphanTab()) {
+      // If no current group, figure it out based on what's already in the tab
+      // bar.
+      if (!GroupItems.getActiveGroupItem()) {
         for (let a = 0; a < gBrowser.tabs.length; a++) {
-          let theTab = gBrowser.tabs[a]; 
+          let theTab = gBrowser.tabs[a];
           if (!theTab.pinned) {
-            let tabItem = theTab._tabViewTabItem; 
-            if (tabItem.parent) 
-              GroupItems.setActiveGroupItem(tabItem.parent);
-            else 
-              GroupItems.setActiveOrphanTab(tabItem); 
-              
+            let tabItem = theTab._tabViewTabItem;
+            this.setActive(tabItem.parent);
             break;
           }
         }
       }
 
-      if (GroupItems.getActiveGroupItem() || GroupItems.getActiveOrphanTab())
+      if (GroupItems.getActiveGroupItem())
         GroupItems._updateTabBar();
     }
   },
@@ -898,12 +929,11 @@ let UI = {
   
   // ----------
   updateTabButton: function UI__updateTabButton() {
-    let groupsNumber = gWindow.document.getElementById("tabviewGroupsNumber");
     let exitButton = document.getElementById("exit-button");
     let numberOfGroups = GroupItems.groupItems.length;
 
-    groupsNumber.setAttribute("groups", numberOfGroups);
     exitButton.setAttribute("groups", numberOfGroups);
+    gTabView.updateGroupNumberBroadcaster(numberOfGroups);
   },
 
   // ----------
@@ -913,7 +943,7 @@ let UI = {
     let cl = null;
     let clDist;
     TabItems.getItems().forEach(function (item) {
-      if (item.parent && item.parent.hidden)
+      if (!item.parent || item.parent.hidden)
         return;
       let testDist = tabCenter.distance(item.bounds.center());
       if (cl==null || testDist < clDist) {
@@ -924,33 +954,135 @@ let UI = {
     return cl;
   },
 
+  // ----------
+  // Function: _setupBrowserKeys
+  // Sets up the allowed browser keys using key elements.
+  _setupBrowserKeys: function UI__setupKeyWhiteList() {
+    let keys = {};
+
+    [
+#ifdef XP_UNIX
+      "quitApplication",
+#else
+      "redo",
+#endif
+#ifdef XP_MACOSX
+      "preferencesCmdMac", "minimizeWindow", "hideThisAppCmdMac",
+#endif
+      "newNavigator", "newNavigatorTab", "undo", "cut", "copy", "paste", 
+      "selectAll", "find"
+    ].forEach(function(key) {
+      let element = gWindow.document.getElementById("key_" + key);
+      keys[key] = element.getAttribute("key").toLocaleLowerCase().charCodeAt(0);
+    });
+
+    // for key combinations with shift key, the charCode of upper case letters 
+    // are different to the lower case ones so need to handle them differently.
+    [
+#ifdef XP_UNIX
+      "redo",
+#endif
+      "closeWindow", "tabview", "undoCloseTab", "undoCloseWindow",
+      "privatebrowsing"
+    ].forEach(function(key) {
+      let element = gWindow.document.getElementById("key_" + key);
+      keys[key] = element.getAttribute("key").toLocaleUpperCase().charCodeAt(0);
+    });
+
+    delete this._browserKeys;
+    this._browserKeys = keys;
+  },
+
   // ----------
   // Function: _setTabViewFrameKeyHandlers
   // Sets up the key handlers for navigating between tabs within the TabView UI.
   _setTabViewFrameKeyHandlers: function UI__setTabViewFrameKeyHandlers() {
-    var self = this;
+    let self = this;
+
+    this._setupBrowserKeys();
 
     iQ(window).keyup(function(event) {
-      if (!event.metaKey) 
+      if (!event.metaKey)
         Keys.meta = false;
     });
 
-    iQ(window).keydown(function(event) {
-      if (event.metaKey) 
+    iQ(window).keypress(function(event) {
+      if (event.metaKey)
         Keys.meta = true;
 
-      if ((iQ(":focus").length > 0 && iQ(":focus")[0].nodeName == "INPUT") || 
-          isSearchEnabled())
+      function processBrowserKeys(evt) {
+        // let any keys with alt to pass through
+        if (evt.altKey)
+          return;
+
+#ifdef XP_MACOSX
+        if (evt.metaKey) {
+#else
+        if (evt.ctrlKey) {
+#endif
+          let preventDefault = true;
+          if (evt.shiftKey) {
+            switch (evt.charCode) {
+              case self._browserKeys.tabview:
+                self.exit();
+                break;
+#ifdef XP_UNIX
+              case self._browserKeys.redo:
+#endif
+              case self._browserKeys.closeWindow:
+              case self._browserKeys.undoCloseTab:
+              case self._browserKeys.undoCloseWindow:
+              case self._browserKeys.privatebrowsing:
+                preventDefault = false;
+                break;
+            }
+          } else {
+            switch (evt.charCode) {
+              case self._browserKeys.find:
+                self.enableSearch();
+                break;
+#ifdef XP_UNIX
+              case self._browserKeys.quitApplication:
+#else
+              case self._browserKeys.redo:
+#endif
+#ifdef XP_MACOSX
+              case self._browserKeys.preferencesCmdMac:
+              case self._browserKeys.minimizeWindow:
+              case self._browserKeys.hideThisAppCmdMac:
+#endif
+              case self._browserKeys.newNavigator:
+              case self._browserKeys.newNavigatorTab:
+              case self._browserKeys.undo:
+              case self._browserKeys.cut:
+              case self._browserKeys.copy:
+              case self._browserKeys.paste:
+              case self._browserKeys.selectAll:
+                preventDefault = false;
+                break;
+            }
+          }
+          if (preventDefault) {
+            evt.stopPropagation();
+            evt.preventDefault();
+          }
+        }
+      }
+      if ((iQ(":focus").length > 0 && iQ(":focus")[0].nodeName == "INPUT") ||
+          isSearchEnabled() || self.ignoreKeypressForSearch) {
+        self.ignoreKeypressForSearch = false;
+        processBrowserKeys(event);
         return;
+      }
 
       function getClosestTabBy(norm) {
         if (!self.getActiveTab())
           return null;
-        var centers =
+        let centers =
           [[item.bounds.center(), item]
              for each(item in TabItems.getItems()) if (!item.parent || !item.parent.hidden)];
-        var myCenter = self.getActiveTab().bounds.center();
-        var matches = centers
+        let myCenter = self.getActiveTab().bounds.center();
+        let matches = centers
           .filter(function(item){return norm(item[0], myCenter)})
           .sort(function(a,b){
             return myCenter.distance(a[0]) - myCenter.distance(b[0]);
@@ -960,7 +1092,9 @@ let UI = {
         return null;
       }
 
-      var norm = null;
+      let preventDefault = true;
+      let activeTab;
+      let norm = null;
       switch (event.keyCode) {
         case KeyEvent.DOM_VK_RIGHT:
           norm = function(a, me){return a.x > me.x};
@@ -981,61 +1115,56 @@ let UI = {
         if (nextTab) {
           if (nextTab.isStacked && !nextTab.parent.expanded)
             nextTab = nextTab.parent.getChild(0);
-          self.setActiveTab(nextTab);
+          self.setActive(nextTab);
         }
-        event.stopPropagation();
-        event.preventDefault();
-      } else if (event.keyCode == KeyEvent.DOM_VK_ESCAPE) {
-        let activeGroupItem = GroupItems.getActiveGroupItem();
-        if (activeGroupItem && activeGroupItem.expanded)
-          activeGroupItem.collapse();
-        else 
-          self.exit();
+      } else {
+        switch(event.keyCode) {
+          case KeyEvent.DOM_VK_ESCAPE:
+            let activeGroupItem = GroupItems.getActiveGroupItem();
+            if (activeGroupItem && activeGroupItem.expanded)
+              activeGroupItem.collapse();
+            else
+              self.exit();
+            break;
+          case KeyEvent.DOM_VK_RETURN:
+          case KeyEvent.DOM_VK_ENTER:
+            activeTab = self.getActiveTab();
+            if (activeTab)
+              activeTab.zoomIn();
+            break;
+          case KeyEvent.DOM_VK_TAB:
+            // tab/shift + tab to go to the next tab.
+            activeTab = self.getActiveTab();
+            if (activeTab) {
+              let tabItems = (activeTab.parent ? activeTab.parent.getChildren() :
+                              [activeTab]);
+              let length = tabItems.length;
+              let currentIndex = tabItems.indexOf(activeTab);
 
-        event.stopPropagation();
-        event.preventDefault();
-      } else if (event.keyCode == KeyEvent.DOM_VK_RETURN ||
-                 event.keyCode == KeyEvent.DOM_VK_ENTER) {
-        let activeTab = self.getActiveTab();
-        if (activeTab)
-          activeTab.zoomIn();
-
-        event.stopPropagation();
-        event.preventDefault();
-      } else if (event.keyCode == KeyEvent.DOM_VK_TAB) {
-        // tab/shift + tab to go to the next tab.
-        var activeTab = self.getActiveTab();
-        if (activeTab) {
-          var tabItems = (activeTab.parent ? activeTab.parent.getChildren() :
-                          [activeTab]);
-          var length = tabItems.length;
-          var currentIndex = tabItems.indexOf(activeTab);
-
-          if (length > 1) {
-            if (event.shiftKey) {
-              if (currentIndex == 0)
-                newIndex = (length - 1);
-              else
-                newIndex = (currentIndex - 1);
-            } else {
-              if (currentIndex == (length - 1))
-                newIndex = 0;
-              else
-                newIndex = (currentIndex + 1);
+              if (length > 1) {
+                if (event.shiftKey) {
+                  if (currentIndex == 0)
+                    newIndex = (length - 1);
+                  else
+                    newIndex = (currentIndex - 1);
+                } else {
+                  if (currentIndex == (length - 1))
+                    newIndex = 0;
+                  else
+                    newIndex = (currentIndex + 1);
+                }
+                self.setActive(tabItems[newIndex]);
+              }
             }
-            self.setActiveTab(tabItems[newIndex]);
-          }
+            break;
+          default:
+            processBrowserKeys(event);
+            preventDefault = false;
+        }
+        if (preventDefault) {
+          event.stopPropagation();
+          event.preventDefault();
         }
-        event.stopPropagation();
-        event.preventDefault();
-      } else if (event.keyCode == KeyEvent.DOM_VK_SLASH) {
-        // the / event handler for find bar is defined in the findbar.xml
-        // binding.  To keep things in its own module, we handle our slash here.
-        self.enableSearch(event);
-      } else if (event.keyCode == KeyEvent.DOM_VK_BACK_SPACE) {
-        // prevent navigating backward in the selected tab's history
-        event.stopPropagation();
-        event.preventDefault();
       }
     });
   },
@@ -1043,17 +1172,10 @@ let UI = {
   // ----------
   // Function: enableSearch
   // Enables the search feature.
-  // Parameters:
-  //   event - the event triggers this action.
-  enableSearch: function UI_enableSearch(event) {
+  enableSearch: function UI_enableSearch() {
     if (!isSearchEnabled()) {
       ensureSearchShown();
       SearchEventHandler.switchToInMode();
-      
-      if (event) {
-        event.stopPropagation();
-        event.preventDefault();
-      }
     }
   },
 
@@ -1066,7 +1188,6 @@ let UI = {
     const minMinSize = 15;
 
     let lastActiveGroupItem = GroupItems.getActiveGroupItem();
-    GroupItems.setActiveGroupItem(null);
 
     var startPos = { x: e.clientX, y: e.clientY };
     var phantom = iQ("<div>")
@@ -1135,6 +1256,7 @@ let UI = {
       e.preventDefault();
     }
 
+    let self = this;
     function collapse() {
       let center = phantom.bounds().center();
       phantom.animate({
@@ -1148,7 +1270,7 @@ let UI = {
           phantom.remove();
         }
       });
-      GroupItems.setActiveGroupItem(lastActiveGroupItem);
+      self.setActive(lastActiveGroupItem);
     }
 
     function finalize(e) {
@@ -1158,19 +1280,9 @@ let UI = {
       let box = item.getBounds();
       if (box.width > minMinSize && box.height > minMinSize &&
          (box.width > minSize || box.height > minSize)) {
-        var bounds = item.getBounds();
-
-        // Add all of the orphaned tabs that are contained inside the new groupItem
-        // to that groupItem.
-        var tabs = GroupItems.getOrphanedTabs();
-        var insideTabs = [];
-        for each(let tab in tabs) {
-          if (bounds.contains(tab.bounds))
-            insideTabs.push(tab);
-        }
-
-        var groupItem = new GroupItem(insideTabs,{bounds:bounds});
-        GroupItems.setActiveGroupItem(groupItem);
+        let opts = {bounds: item.getBounds(), focusTitle: true};
+        let groupItem = new GroupItem([], opts);
+        self.setActive(groupItem);
         phantom.remove();
         dragOutInfo = null;
         gTabView.firstUseExperienced = true;
@@ -1280,7 +1392,6 @@ let UI = {
   // These computations may be done using cached values. The cache can be
   // cleared with UI.clearShouldResizeItems().
   shouldResizeItems: function UI_shouldResizeItems() {
-
     let newPageBounds = Items.getPageBounds();
     
     // If we don't have cached cached values...
@@ -1346,15 +1457,15 @@ let UI = {
       let unhiddenGroups = GroupItems.groupItems.filter(function(groupItem) {
         return (!groupItem.hidden && groupItem.getChildren().length > 0);
       });
-      // no pinned tabs, no visible groups and no orphaned tabs: open a new
-      // group. open a blank tab and return
-      if (!unhiddenGroups.length && !GroupItems.getOrphanedTabs().length) {
+      // no pinned tabs and no visible groups: open a new group. open a blank
+      // tab and return
+      if (!unhiddenGroups.length) {
         let emptyGroups = GroupItems.groupItems.filter(function (groupItem) {
           return (!groupItem.hidden && !groupItem.getChildren().length);
         });
         let group = (emptyGroups.length ? emptyGroups[0] : GroupItems.newGroup());
         if (!gBrowser._numPinnedTabs) {
-          group.newTab();
+          group.newTab(null, { closedLastTab: true });
           return;
         }
       }
@@ -1433,6 +1544,30 @@ let UI = {
     this._save();
     GroupItems.saveAll();
     TabItems.saveAll();
+  },
+
+  // ----------
+  // Function: shouldLoadFavIcon
+  // Takes a xul:browser and checks whether we should display a favicon for it.
+  shouldLoadFavIcon: function UI_shouldLoadFavIcon(browser) {
+    return !(browser.contentDocument instanceof window.ImageDocument) &&
+            (browser.currentURI.schemeIs("about") ||
+             gBrowser.shouldLoadFavIcon(browser.contentDocument.documentURIObject));
+  },
+
+  // ----------
+  // Function: getFavIconUrlForTab
+  // Gets fav icon url for the given xul:tab.
+  getFavIconUrlForTab: function UI_getFavIconUrlForTab(tab) {
+    let url;
+
+    // use the tab image if it doesn't start with http e.g. data:image/png, chrome://
+    if (tab.image && !(/^https?:/.test(tab.image)))
+      url = tab.image;
+    else
+      url = gFavIconService.getFaviconImageForPage(tab.linkedBrowser.currentURI).spec;
+
+    return url;
   }
 };
 
