@@ -13,3 +13,16 @@ registerCleanupFunction(function(){
                                      cachedLeftPaneFolderIdGetter);
   }
 });
+
+
+function openLibrary(callback) {
+  let library = window.openDialog("chrome://browser/content/places/places.xul",
+                                  "", "chrome,toolbar=yes,dialog=no,resizable");
+  waitForFocus(function () {
+    callback(library);
+  }, library);
+
+  return library;
+}
+
+Components.utils.import("resource://gre/modules/NetUtil.jsm");
