@@ -39,7 +39,6 @@
 
 // Load DownloadUtils module for convertByteUnits
 Components.utils.import("resource://gre/modules/DownloadUtils.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
 
 var gAdvancedPane = {
   _inited: false,
@@ -179,8 +178,9 @@ var gAdvancedPane = {
   updateHardwareAcceleration: function()
   {
 #ifdef XP_WIN
-    var pref = document.getElementById("layers.acceleration.disabled");
-    Services.prefs.setBoolPref("gfx.direct2d.disabled", !pref.value);
+    var fromPref = document.getElementById("layers.acceleration.disabled");
+    var toPref = document.getElementById("gfx.direct2d.disabled");
+    toPref.value = fromPref.value;
 #endif
   },
 
