@@ -117,10 +117,6 @@
 #define NS_EVENTLISTENERMANAGER_CID \
 { 0x9f689e0, 0xb4da, 0x11d2, { 0xa6, 0x8b, 0x0, 0x10, 0x4b, 0xde, 0x60, 0x48 } }
 
-// {66856477-6596-40eb-bb87-59ca2dabb6f7}
-#define NS_DOMEVENTGROUP_CID \
-{ 0x66856477, 0x6596, 0x40eb, { 0xbb, 0x87, 0x59, 0xca, 0x2d, 0xab, 0xb6, 0xf7 } }
-
 // {64F300A1-C88C-11d3-97FB-00400553EEF0}
 #define NS_XBLSERVICE_CID \
 { 0x64f300a1, 0xc88c, 0x11d3, { 0x97, 0xfb, 0x0, 0x40, 0x5, 0x53, 0xee, 0xf0 } }
@@ -214,15 +210,11 @@
 #define NS_RANGEUTILS_CID \
 { 0xa6cf9126, 0x15b3, 0x11d2, {0x93, 0x2e, 0x00, 0x80, 0x5f, 0x8a, 0xdd, 0x32 } }
 
-#ifdef MOZ_SVG
-
 #define NS_SVGDOCUMENT_CID                        \
 { /* b7f44954-1dd1-11b2-8c2e-c2feab4186bc */      \
   0xb7f44954, 0x11d1, 0x11b2,                     \
   {0x8c, 0x2e, 0xc2, 0xfe, 0xab, 0x41, 0x86, 0xbc}}
 
-#endif // MOZ_SVG
-
 #ifdef MOZ_MEDIA
 
 // {d899a152-9412-46b2-b651-2e71c5c2f05f}
@@ -232,13 +224,6 @@
 
 #endif
 
-#define NS_SYNCLOADDOMSERVICE_CID                   \
- { /* 0e4e7d00-f71a-439f-9178-1a71ff11b55f */       \
-  0x0e4e7d00, 0xf71a, 0x439f,                       \
- {0x91, 0x78, 0x1a, 0x71, 0xff, 0x11, 0xb5, 0x5f} }
-#define NS_SYNCLOADDOMSERVICE_CONTRACTID            \
-"@mozilla.org/content/syncload-dom-service;1"
-
 #define NS_EVENTLISTENERSERVICE_CID                 \
  { /* baa34652-f1f1-4185-b224-244ee82a413a */       \
    0xbaa34652, 0xf1f1, 0x4185,                      \
