@@ -45,7 +45,6 @@
 #include "nsContentList.h"
 #include "nsIContent.h"
 #include "nsIDOMNode.h"
-#include "nsIDOM3Node.h"
 #include "nsIDocument.h"
 #include "nsGenericElement.h"
 
@@ -68,7 +67,7 @@ NS_NewPreContentIterator(nsIContentIterator** aInstancePtrResult);
 #endif
 
 
-static nsContentList *gCachedContentList;
+using namespace mozilla::dom;
 
 nsBaseContentList::~nsBaseContentList()
 {
@@ -86,15 +85,17 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
     NS_INTERFACE_TABLE_ENTRY(_class, nsINodeList)                             \
     NS_INTERFACE_TABLE_ENTRY(_class, nsIDOMNodeList)
 
+DOMCI_DATA(NodeList, nsBaseContentList)
 
 // QueryInterface implementation for nsBaseContentList
 NS_INTERFACE_TABLE_HEAD(nsBaseContentList)
+  NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_NODELIST_OFFSET_AND_INTERFACE_TABLE_BEGIN(nsBaseContentList)
     NS_CONTENT_LIST_INTERFACES(nsBaseContentList)
   NS_OFFSET_AND_INTERFACE_TABLE_END
   NS_OFFSET_AND_INTERFACE_TABLE_TO_MAP_SEGUE
   NS_INTERFACE_MAP_ENTRIES_CYCLE_COLLECTION(nsBaseContentList)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(NodeList)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(NodeList)
 NS_INTERFACE_MAP_END
 
 
@@ -124,23 +125,12 @@ nsBaseContentList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
   return CallQueryInterface(tmp, aReturn);
 }
 
-nsINode*
+nsIContent*
 nsBaseContentList::GetNodeAt(PRUint32 aIndex)
 {
   return mElements.SafeObjectAt(aIndex);
 }
 
-void
-nsBaseContentList::AppendElement(nsIContent *aContent)
-{
-  mElements.AppendObject(aContent);
-}
-
-void
-nsBaseContentList::RemoveElement(nsIContent *aContent)
-{
-  mElements.RemoveObject(aContent);
-}
 
 PRInt32
 nsBaseContentList::IndexOf(nsIContent *aContent, PRBool aDoFlush)
@@ -148,39 +138,60 @@ nsBaseContentList::IndexOf(nsIContent *aContent, PRBool aDoFlush)
   return mElements.IndexOf(aContent);
 }
 
-void
-nsBaseContentList::Reset()
+PRInt32
+nsBaseContentList::IndexOf(nsIContent* aContent)
 {
-  mElements.Clear();
+  return IndexOf(aContent, PR_TRUE);
 }
 
-// static
-void
-nsBaseContentList::Shutdown()
+void nsBaseContentList::AppendElement(nsIContent *aContent) 
 {
-  NS_IF_RELEASE(gCachedContentList);
+  mElements.AppendObject(aContent);
 }
 
+void nsBaseContentList::RemoveElement(nsIContent *aContent) 
+{
+  mElements.RemoveObject(aContent);
+}
+
+void nsBaseContentList::InsertElementAt(nsIContent* aContent, PRInt32 aIndex)
+{
+  NS_ASSERTION(aContent, "Element to insert must not be null");
+  mElements.InsertObjectAt(aContent, aIndex);
+}
+
+
+NS_IMPL_CYCLE_COLLECTION_CLASS(nsSimpleContentList)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(nsSimpleContentList,
+                                                  nsBaseContentList)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mRoot)
+NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(nsSimpleContentList,
+                                                nsBaseContentList)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mRoot)
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(nsSimpleContentList)
+NS_INTERFACE_MAP_END_INHERITING(nsBaseContentList)
+
+
+NS_IMPL_ADDREF_INHERITED(nsSimpleContentList, nsBaseContentList)
+NS_IMPL_RELEASE_INHERITED(nsSimpleContentList, nsBaseContentList)
 
 // nsFormContentList
 
-nsFormContentList::nsFormContentList(nsIDOMHTMLFormElement *aForm,
+nsFormContentList::nsFormContentList(nsIContent *aForm,
                                      nsBaseContentList& aContentList)
-  : nsBaseContentList()
+  : nsSimpleContentList(aForm)
 {
 
   // move elements that belong to mForm into this content list
 
   PRUint32 i, length = 0;
-  nsCOMPtr<nsIDOMNode> item;
-
   aContentList.GetLength(&length);
 
   for (i = 0; i < length; i++) {
-    aContentList.Item(i, getter_AddRefs(item));
-
-    nsCOMPtr<nsIContent> c(do_QueryInterface(item));
-
+    nsIContent *c = aContentList.GetNodeAt(i);
     if (c && nsContentUtils::BelongsInForm(aForm, c)) {
       AppendElement(c);
     }
@@ -209,15 +220,17 @@ ContentListHashtableMatchEntry(PLDHashTable *table,
 {
   const ContentListHashEntry *e =
     static_cast<const ContentListHashEntry *>(entry);
-  const nsContentListKey* list1 = e->mContentList->GetKey();
-  const nsContentListKey* list2 = static_cast<const nsContentListKey *>(key);
+  const nsContentList* list = e->mContentList;
+  const nsContentListKey* ourKey = static_cast<const nsContentListKey *>(key);
 
-  return list1->Equals(*list2);
+  return list->MatchesKey(*ourKey);
 }
 
 already_AddRefed<nsContentList>
-NS_GetContentList(nsINode* aRootNode, nsIAtom* aMatchAtom,
-                  PRInt32 aMatchNameSpaceId)
+NS_GetContentList(nsINode* aRootNode, 
+                  PRInt32  aMatchNameSpaceId,
+                  const nsAString& aTagname)
+                  
 {
   NS_ASSERTION(aRootNode, "content list has to have a root");
 
@@ -249,15 +262,14 @@ NS_GetContentList(nsINode* aRootNode, nsIAtom* aMatchAtom,
   ContentListHashEntry *entry = nsnull;
   // First we look in our hashtable.  Then we create a content list if needed
   if (gContentListHashTable.ops) {
-    nsContentListKey hashKey(aRootNode, aMatchAtom,
-                             aMatchNameSpaceId);
+    nsContentListKey hashKey(aRootNode, aMatchNameSpaceId, aTagname);
     
     // A PL_DHASH_ADD is equivalent to a PL_DHASH_LOOKUP for cases
     // when the entry is already in the hashtable.
     entry = static_cast<ContentListHashEntry *>
                        (PL_DHashTableOperate(&gContentListHashTable,
-                                                &hashKey,
-                                                PL_DHASH_ADD));
+                                             &hashKey,
+                                             PL_DHASH_ADD));
     if (entry)
       list = entry->mContentList;
   }
@@ -265,8 +277,116 @@ NS_GetContentList(nsINode* aRootNode, nsIAtom* aMatchAtom,
   if (!list) {
     // We need to create a ContentList and add it to our new entry, if
     // we have an entry
-    list = new nsContentList(aRootNode, aMatchAtom,
-                             aMatchNameSpaceId);
+    nsCOMPtr<nsIAtom> xmlAtom = do_GetAtom(aTagname);
+    nsCOMPtr<nsIAtom> htmlAtom;
+    if (aMatchNameSpaceId == kNameSpaceID_Unknown) {
+      nsAutoString lowercaseName;
+      nsContentUtils::ASCIIToLower(aTagname, lowercaseName);
+      htmlAtom = do_GetAtom(lowercaseName);
+    } else {
+      htmlAtom = xmlAtom;
+    }
+    list = new nsContentList(aRootNode, aMatchNameSpaceId,
+                             htmlAtom, xmlAtom);
+    if (entry) {
+      entry->mContentList = list;
+    }
+  }
+
+  NS_ADDREF(list);
+
+  return list;
+}
+
+// Hashtable for storing nsCacheableFuncStringContentList
+static PLDHashTable gFuncStringContentListHashTable;
+
+struct FuncStringContentListHashEntry : public PLDHashEntryHdr
+{
+  nsCacheableFuncStringContentList* mContentList;
+};
+
+static PLDHashNumber
+FuncStringContentListHashtableHashKey(PLDHashTable *table, const void *key)
+{
+  const nsFuncStringCacheKey* funcStringKey =
+    static_cast<const nsFuncStringCacheKey *>(key);
+  return funcStringKey->GetHash();
+}
+
+static PRBool
+FuncStringContentListHashtableMatchEntry(PLDHashTable *table,
+                               const PLDHashEntryHdr *entry,
+                               const void *key)
+{
+  const FuncStringContentListHashEntry *e =
+    static_cast<const FuncStringContentListHashEntry *>(entry);
+  const nsFuncStringCacheKey* ourKey =
+    static_cast<const nsFuncStringCacheKey *>(key);
+
+  return e->mContentList->Equals(ourKey);
+}
+
+already_AddRefed<nsContentList>
+NS_GetFuncStringContentList(nsINode* aRootNode,
+                            nsContentListMatchFunc aFunc,
+                            nsContentListDestroyFunc aDestroyFunc,
+                            nsFuncStringContentListDataAllocator aDataAllocator,
+                            const nsAString& aString)
+{
+  NS_ASSERTION(aRootNode, "content list has to have a root");
+
+  nsCacheableFuncStringContentList* list = nsnull;
+
+  static PLDHashTableOps hash_table_ops =
+  {
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
+    FuncStringContentListHashtableHashKey,
+    FuncStringContentListHashtableMatchEntry,
+    PL_DHashMoveEntryStub,
+    PL_DHashClearEntryStub,
+    PL_DHashFinalizeStub
+  };
+
+  // Initialize the hashtable if needed.
+  if (!gFuncStringContentListHashTable.ops) {
+    PRBool success = PL_DHashTableInit(&gFuncStringContentListHashTable,
+                                       &hash_table_ops, nsnull,
+                                       sizeof(FuncStringContentListHashEntry),
+                                       16);
+
+    if (!success) {
+      gFuncStringContentListHashTable.ops = nsnull;
+    }
+  }
+
+  FuncStringContentListHashEntry *entry = nsnull;
+  // First we look in our hashtable.  Then we create a content list if needed
+  if (gFuncStringContentListHashTable.ops) {
+    nsFuncStringCacheKey hashKey(aRootNode, aFunc, aString);
+
+    // A PL_DHASH_ADD is equivalent to a PL_DHASH_LOOKUP for cases
+    // when the entry is already in the hashtable.
+    entry = static_cast<FuncStringContentListHashEntry *>
+                       (PL_DHashTableOperate(&gFuncStringContentListHashTable,
+                                             &hashKey,
+                                             PL_DHASH_ADD));
+    if (entry)
+      list = entry->mContentList;
+  }
+
+  if (!list) {
+    // We need to create a ContentList and add it to our new entry, if
+    // we have an entry
+    list = new nsCacheableFuncStringContentList(aRootNode, aFunc, aDestroyFunc,
+                                                aDataAllocator, aString);
+    if (list && !list->AllocatedData()) {
+      // Failed to allocate the data
+      delete list;
+      list = nsnull;
+    }
+
     if (entry) {
       if (list)
         entry->mContentList = list;
@@ -279,30 +399,23 @@ NS_GetContentList(nsINode* aRootNode, nsIAtom* aMatchAtom,
 
   NS_ADDREF(list);
 
-  // Hold on to the last requested content list to avoid having it be
-  // removed from the cache immediately when it's released. Avoid
-  // bumping the refcount on the list if the requested list is the one
-  // that's already cached.
-
-  if (gCachedContentList != list) {
-    NS_IF_RELEASE(gCachedContentList);
-
-    gCachedContentList = list;
-    NS_ADDREF(gCachedContentList);
-  }
+  // Don't cache these lists globally
 
   return list;
 }
 
-
 // nsContentList implementation
 
 nsContentList::nsContentList(nsINode* aRootNode,
-                             nsIAtom* aMatchAtom,
                              PRInt32 aMatchNameSpaceId,
+                             nsIAtom* aHTMLMatchAtom,
+                             nsIAtom* aXMLMatchAtom,
                              PRBool aDeep)
   : nsBaseContentList(),
-    nsContentListKey(aRootNode, aMatchAtom, aMatchNameSpaceId),
+    mRootNode(aRootNode),
+    mMatchNameSpaceId(aMatchNameSpaceId),
+    mHTMLMatchAtom(aHTMLMatchAtom),
+    mXMLMatchAtom(aXMLMatchAtom),
     mFunc(nsnull),
     mDestroyFunc(nsnull),
     mData(nsnull),
@@ -311,13 +424,22 @@ nsContentList::nsContentList(nsINode* aRootNode,
     mFuncMayDependOnAttr(PR_FALSE)
 {
   NS_ASSERTION(mRootNode, "Must have root");
-  if (nsGkAtoms::_asterix == mMatchAtom) {
+  if (nsGkAtoms::_asterix == mHTMLMatchAtom) {
+    NS_ASSERTION(mXMLMatchAtom == nsGkAtoms::_asterix, "HTML atom and XML atom are not both asterix?");
     mMatchAll = PR_TRUE;
   }
   else {
     mMatchAll = PR_FALSE;
   }
   mRootNode->AddMutationObserver(this);
+
+  // We only need to flush if we're in an non-HTML document, since the
+  // HTML5 parser doesn't need flushing.  Further, if we're not in a
+  // document at all right now (in the GetCurrentDoc() sense), we're
+  // not parser-created and don't need to be flushing stuff under us
+  // to get our kids right.
+  nsIDocument* doc = mRootNode->GetCurrentDoc();
+  mFlushesNeeded = doc && !doc->IsHTML();
 }
 
 nsContentList::nsContentList(nsINode* aRootNode,
@@ -329,17 +451,28 @@ nsContentList::nsContentList(nsINode* aRootNode,
                              PRInt32 aMatchNameSpaceId,
                              PRBool aFuncMayDependOnAttr)
   : nsBaseContentList(),
-    nsContentListKey(aRootNode, aMatchAtom, aMatchNameSpaceId),
+    mRootNode(aRootNode),
+    mMatchNameSpaceId(aMatchNameSpaceId),
+    mHTMLMatchAtom(aMatchAtom),
+    mXMLMatchAtom(aMatchAtom),
     mFunc(aFunc),
     mDestroyFunc(aDestroyFunc),
     mData(aData),
-    mMatchAll(PR_FALSE),
     mState(LIST_DIRTY),
+    mMatchAll(PR_FALSE),
     mDeep(aDeep),
     mFuncMayDependOnAttr(aFuncMayDependOnAttr)
 {
   NS_ASSERTION(mRootNode, "Must have root");
   mRootNode->AddMutationObserver(this);
+
+  // We only need to flush if we're in an non-HTML document, since the
+  // HTML5 parser doesn't need flushing.  Further, if we're not in a
+  // document at all right now (in the GetCurrentDoc() sense), we're
+  // not parser-created and don't need to be flushing stuff under us
+  // to get our kids right.
+  nsIDocument* doc = mRootNode->GetCurrentDoc();
+  mFlushesNeeded = doc && !doc->IsHTML();
 }
 
 nsContentList::~nsContentList()
@@ -355,10 +488,10 @@ nsContentList::~nsContentList()
   }
 }
 
+DOMCI_DATA(ContentList, nsContentList)
 
 // QueryInterface implementation for nsContentList
 NS_INTERFACE_TABLE_HEAD(nsContentList)
-  NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_NODELIST_OFFSET_AND_INTERFACE_TABLE_BEGIN(nsContentList)
     NS_CONTENT_LIST_INTERFACES(nsContentList)
     NS_INTERFACE_TABLE_ENTRY(nsContentList, nsIHTMLCollection)
@@ -366,20 +499,13 @@ NS_INTERFACE_TABLE_HEAD(nsContentList)
     NS_INTERFACE_TABLE_ENTRY(nsContentList, nsIMutationObserver)
   NS_OFFSET_AND_INTERFACE_TABLE_END
   NS_OFFSET_AND_INTERFACE_TABLE_TO_MAP_SEGUE
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(ContentList)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(ContentList)
 NS_INTERFACE_MAP_END_INHERITING(nsBaseContentList)
 
 
 NS_IMPL_ADDREF_INHERITED(nsContentList, nsBaseContentList)
 NS_IMPL_RELEASE_INHERITED(nsContentList, nsBaseContentList)
 
-
-nsISupports *
-nsContentList::GetParentObject()
-{
-  return mRootNode;
-}
-  
 PRUint32
 nsContentList::Length(PRBool aDoFlush)
 {
@@ -391,7 +517,7 @@ nsContentList::Length(PRBool aDoFlush)
 nsIContent *
 nsContentList::Item(PRUint32 aIndex, PRBool aDoFlush)
 {
-  if (mRootNode && aDoFlush) {
+  if (mRootNode && aDoFlush && mFlushesNeeded) {
     // XXX sXBL/XBL2 issue
     nsIDocument* doc = mRootNode->GetCurrentDoc();
     if (doc) {
@@ -444,12 +570,18 @@ nsContentList::IndexOf(nsIContent *aContent, PRBool aDoFlush)
   return mElements.IndexOf(aContent);
 }
 
+PRInt32
+nsContentList::IndexOf(nsIContent* aContent)
+{
+  return IndexOf(aContent, PR_TRUE);
+}
+
 void
 nsContentList::NodeWillBeDestroyed(const nsINode* aNode)
 {
   // We shouldn't do anything useful from now on
 
-  RemoveFromHashtable();
+  RemoveFromCaches();
   mRootNode = nsnull;
 
   // We will get no more updates, so we can never know we're up to
@@ -457,21 +589,6 @@ nsContentList::NodeWillBeDestroyed(const nsINode* aNode)
   SetDirty();
 }
 
-// static
-void
-nsContentList::OnDocumentDestroy(nsIDocument *aDocument)
-{
-  // If our content list cache holds a list used for a document that's
-  // now being destroyed, free the cache to prevent the list from
-  // staying around until the next use of content lists ends up
-  // replacing what's in the cache.
-
-  if (gCachedContentList && gCachedContentList->mRootNode &&
-      gCachedContentList->mRootNode->GetOwnerDoc() == aDocument) {
-    NS_RELEASE(gCachedContentList);
-  }
-}
-
 NS_IMETHODIMP
 nsContentList::GetLength(PRUint32* aLength)
 {
@@ -508,61 +625,54 @@ nsContentList::NamedItem(const nsAString& aName, nsIDOMNode** aReturn)
   return NS_OK;
 }
 
-nsINode*
+nsIContent*
 nsContentList::GetNodeAt(PRUint32 aIndex)
 {
   return Item(aIndex, PR_TRUE);
 }
 
 nsISupports*
-nsContentList::GetNodeAt(PRUint32 aIndex, nsresult* aResult)
+nsContentList::GetNamedItem(const nsAString& aName, nsWrapperCache **aCache)
 {
-  *aResult = NS_OK;
-  return Item(aIndex, PR_TRUE);
-}
-
-nsISupports*
-nsContentList::GetNamedItem(const nsAString& aName, nsresult* aResult)
-{
-  *aResult = NS_OK;
-  return NamedItem(aName, PR_TRUE);
+  nsIContent *item;
+  *aCache = item = NamedItem(aName, PR_TRUE);
+  return item;
 }
 
 void
-nsContentList::AttributeChanged(nsIDocument *aDocument, nsIContent* aContent,
+nsContentList::AttributeChanged(nsIDocument *aDocument, Element* aElement,
                                 PRInt32 aNameSpaceID, nsIAtom* aAttribute,
-                                PRInt32 aModType, PRUint32 aStateMask)
+                                PRInt32 aModType)
 {
-  NS_PRECONDITION(aContent, "Must have a content node to work with");
-  NS_PRECONDITION(aContent->IsNodeOfType(nsINode::eELEMENT),
-                  "Should be an element");
+  NS_PRECONDITION(aElement, "Must have a content node to work with");
   
   if (!mFunc || !mFuncMayDependOnAttr || mState == LIST_DIRTY ||
-      !MayContainRelevantNodes(aContent->GetNodeParent()) ||
-      !nsContentUtils::IsInSameAnonymousTree(mRootNode, aContent)) {
+      !MayContainRelevantNodes(aElement->GetNodeParent()) ||
+      !nsContentUtils::IsInSameAnonymousTree(mRootNode, aElement)) {
     // Either we're already dirty or this notification doesn't affect
-    // whether we might match aContent.
+    // whether we might match aElement.
     return;
   }
   
-  if (Match(aContent)) {
-    if (mElements.IndexOf(aContent) == -1) {
-      // We match aContent now, and it's not in our list already.  Just dirty
+  if (Match(aElement)) {
+    if (mElements.IndexOf(aElement) == -1) {
+      // We match aElement now, and it's not in our list already.  Just dirty
       // ourselves; this is simpler than trying to figure out where to insert
-      // aContent.
+      // aElement.
       SetDirty();
     }
   } else {
-    // We no longer match aContent.  Remove it from our list.  If it's
+    // We no longer match aElement.  Remove it from our list.  If it's
     // already not there, this is a no-op (though a potentially
     // expensive one).  Either way, no change of mState is required
     // here.
-    mElements.RemoveObject(aContent);
+    mElements.RemoveObject(aElement);
   }
 }
 
 void
-nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
+nsContentList::ContentAppended(nsIDocument* aDocument, nsIContent* aContainer,
+                               nsIContent* aFirstNewContent,
                                PRInt32 aNewIndexInContainer)
 {
   NS_PRECONDITION(aContainer, "Can't get at the new content if no container!");
@@ -600,26 +710,24 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
        * We want to append instead of invalidating if the first thing
        * that got appended comes after ourLastContent.
        */
-      if (nsContentUtils::PositionIsBefore(ourLastContent,
-                                           aContainer->GetChildAt(aNewIndexInContainer))) {
+      if (nsContentUtils::PositionIsBefore(ourLastContent, aFirstNewContent)) {
         appendToList = PR_TRUE;
       }
     }
     
-    PRInt32 i;
-    
+
     if (!appendToList) {
       // The new stuff is somewhere in the middle of our list; check
       // whether we need to invalidate
-      for (i = aNewIndexInContainer; i <= count-1; ++i) {
-        if (MatchSelf(aContainer->GetChildAt(i))) {
+      for (nsIContent* cur = aFirstNewContent; cur; cur = cur->GetNextSibling()) {
+        if (MatchSelf(cur)) {
           // Uh-oh.  We're gonna have to add elements into the middle
           // of our list. That's not worth the effort.
           SetDirty();
           break;
         }
       }
- 
+
       ASSERT_IN_SYNC;
       return;
     }
@@ -637,11 +745,19 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
      * We're up to date.  That means someone's actively using us; we
      * may as well grab this content....
      */
-    for (i = aNewIndexInContainer; i <= count-1; ++i) {
-      PRUint32 limit = PRUint32(-1);
-      nsIContent* newContent = aContainer->GetChildAt(i);
-      if (newContent->IsNodeOfType(nsINode::eELEMENT)) {
-        PopulateWith(newContent, limit);
+    if (mDeep) {
+      for (nsIContent* cur = aFirstNewContent;
+           cur;
+           cur = cur->GetNextNode(aContainer)) {
+        if (cur->IsElement() && Match(cur->AsElement())) {
+          mElements.AppendObject(cur);
+        }
+      }
+    } else {
+      for (nsIContent* cur = aFirstNewContent; cur; cur = cur->GetNextSibling()) {
+        if (cur->IsElement() && Match(cur->AsElement())) {
+          mElements.AppendObject(cur);
+        }
       }
     }
 
@@ -672,7 +788,8 @@ void
 nsContentList::ContentRemoved(nsIDocument *aDocument,
                               nsIContent* aContainer,
                               nsIContent* aChild,
-                              PRInt32 aIndexInContainer)
+                              PRInt32 aIndexInContainer,
+                              nsIContent* aPreviousSibling)
 {
   // Note that aContainer can be null here if we are removing from
   // the document itself; any attempted optimizations to this method
@@ -688,34 +805,42 @@ nsContentList::ContentRemoved(nsIDocument *aDocument,
 }
 
 PRBool
-nsContentList::Match(nsIContent *aContent)
+nsContentList::Match(Element *aElement)
 {
-  if (!aContent)
+  if (mFunc) {
+    return (*mFunc)(aElement, mMatchNameSpaceId, mXMLMatchAtom, mData);
+  }
+
+  if (!mXMLMatchAtom)
     return PR_FALSE;
 
-  NS_ASSERTION(aContent->IsNodeOfType(nsINode::eELEMENT),
-               "Must have element here");
+  nsINodeInfo *ni = aElement->NodeInfo();
+ 
+  PRBool unknown = mMatchNameSpaceId == kNameSpaceID_Unknown;
+  PRBool wildcard = mMatchNameSpaceId == kNameSpaceID_Wildcard;
+  PRBool toReturn = mMatchAll;
+  if (!unknown && !wildcard)
+    toReturn &= ni->NamespaceEquals(mMatchNameSpaceId);
 
-  if (mFunc) {
-    return (*mFunc)(aContent, mMatchNameSpaceId, mMatchAtom, mData);
+  if (toReturn)
+    return toReturn;
+
+  nsIDocument* doc = aElement->GetOwnerDoc();
+  PRBool matchHTML = aElement->GetNameSpaceID() == kNameSpaceID_XHTML &&
+    doc && doc->IsHTML();
+ 
+  if (unknown) {
+    return matchHTML ? ni->QualifiedNameEquals(mHTMLMatchAtom) :
+                       ni->QualifiedNameEquals(mXMLMatchAtom);
   }
-
-  if (mMatchAtom) {
-    nsINodeInfo *ni = aContent->NodeInfo();
-
-    if (mMatchNameSpaceId == kNameSpaceID_Unknown) {
-      return (mMatchAll || ni->QualifiedNameEquals(mMatchAtom));
-    }
-
-    if (mMatchNameSpaceId == kNameSpaceID_Wildcard) {
-      return (mMatchAll || ni->Equals(mMatchAtom));
-    }
-
-    return ((mMatchAll && ni->NamespaceEquals(mMatchNameSpaceId)) ||
-            ni->Equals(mMatchAtom, mMatchNameSpaceId));
+  
+  if (wildcard) {
+    return matchHTML ? ni->Equals(mHTMLMatchAtom) :
+                       ni->Equals(mXMLMatchAtom);
   }
-
-  return PR_FALSE;
+  
+  return matchHTML ? ni->Equals(mHTMLMatchAtom, mMatchNameSpaceId) :
+                     ni->Equals(mXMLMatchAtom, mMatchNameSpaceId);
 }
 
 PRBool 
@@ -725,20 +850,20 @@ nsContentList::MatchSelf(nsIContent *aContent)
   NS_PRECONDITION(mDeep || aContent->GetNodeParent() == mRootNode,
                   "MatchSelf called on a node that we can't possibly match");
 
-  if (!aContent->IsNodeOfType(nsINode::eELEMENT)) {
+  if (!aContent->IsElement()) {
     return PR_FALSE;
   }
   
-  if (Match(aContent))
+  if (Match(aContent->AsElement()))
     return PR_TRUE;
 
   if (!mDeep)
     return PR_FALSE;
 
-  PRUint32 i, count = aContent->GetChildCount();
-
-  for (i = 0; i < count; i++) {
-    if (MatchSelf(aContent->GetChildAt(i))) {
+  for (nsIContent* cur = aContent->GetFirstChild();
+       cur;
+       cur = cur->GetNextNode(aContent)) {
+    if (cur->IsElement() && Match(cur->AsElement())) {
       return PR_TRUE;
     }
   }
@@ -746,113 +871,6 @@ nsContentList::MatchSelf(nsIContent *aContent)
   return PR_FALSE;
 }
 
-void
-nsContentList::PopulateWith(nsIContent *aContent, PRUint32& aElementsToAppend)
-{
-  NS_PRECONDITION(mDeep || aContent->GetNodeParent() == mRootNode,
-                  "PopulateWith called on nodes we can't possibly match");
-  NS_PRECONDITION(aContent != mRootNode,
-                  "We should never be trying to match mRootNode");
-  NS_PRECONDITION(aContent->IsNodeOfType(nsINode::eELEMENT),
-                  "Should be an element");
-
-  if (Match(aContent)) {
-    mElements.AppendObject(aContent);
-    --aElementsToAppend;
-    if (aElementsToAppend == 0)
-      return;
-  }
-
-  // Don't recurse down if we're not doing a deep match.
-  if (!mDeep)
-    return;
-  
-#ifdef DEBUG
-  nsMutationGuard debugMutationGuard;
-#endif  
-  PRUint32 count = aContent->GetChildCount();
-  nsIContent* const* curChildPtr = aContent->GetChildArray();
-  nsIContent* const* stop = curChildPtr + count;
-  for (; curChildPtr != stop; ++curChildPtr) {
-    nsIContent* curContent = *curChildPtr;
-    if (curContent->IsNodeOfType(nsINode::eELEMENT)) {
-      PopulateWith(*curChildPtr, aElementsToAppend);
-      if (aElementsToAppend == 0)
-        break;
-    }
-  }
-#ifdef DEBUG
-  NS_ASSERTION(!debugMutationGuard.Mutated(0),
-               "Unexpected mutations happened.  Check your match function!");
-#endif  
-}
-
-void 
-nsContentList::PopulateWithStartingAfter(nsINode *aStartRoot,
-                                         nsINode *aStartChild,
-                                         PRUint32 & aElementsToAppend)
-{
-  NS_PRECONDITION(mDeep || aStartRoot == mRootNode ||
-                  (aStartRoot->GetNodeParent() == mRootNode &&
-                   aStartChild == nsnull),
-                  "Bogus aStartRoot or aStartChild");
-
-  if (mDeep || aStartRoot == mRootNode) {
-#ifdef DEBUG
-    PRUint32 invariant = aElementsToAppend + mElements.Count();
-#endif
-    PRInt32 i = 0;
-    if (aStartChild) {
-      i = aStartRoot->IndexOf(aStartChild);
-      NS_ASSERTION(i >= 0, "The start child must be a child of the start root!");
-      ++i;  // move to one past
-    }
-
-#ifdef DEBUG
-    nsMutationGuard debugMutationGuard;
-#endif  
-    PRUint32 childCount = aStartRoot->GetChildCount();
-    nsIContent* const* curChildPtr = aStartRoot->GetChildArray();
-    nsIContent* const* stop = curChildPtr + childCount;
-    // Now advance curChildPtr to the child we want to be starting with
-    NS_ASSERTION(i <= childCount, "Unexpected index");
-    curChildPtr += i;
-    for ( ; curChildPtr != stop; ++curChildPtr) {
-      nsIContent* content = *curChildPtr;
-      if (content->IsNodeOfType(nsINode::eELEMENT)) {
-        PopulateWith(content, aElementsToAppend);
-
-        NS_ASSERTION(aElementsToAppend + mElements.Count() == invariant,
-                     "Something is awry in PopulateWith!");
-        if (aElementsToAppend == 0)
-          break;
-      }
-    }
-#ifdef DEBUG
-    NS_ASSERTION(!debugMutationGuard.Mutated(0),
-                 "Unexpected mutations happened.  Check your match function!");
-#endif
-  }
-
-  if (aElementsToAppend == 0) {
-    return;
-  }
-
-  // We want to make sure we don't move up past our root node. So if
-  // we're there, don't move to the parent.
-  if (aStartRoot == mRootNode)
-    return;
-  
-  // We could call GetParent() here to avoid walking children of the
-  // document node. However they should be very few in number and we
-  // might want to walk them in the future so it's unnecessary to have
-  // this be the only thing that prevents it
-  nsINode* parent = aStartRoot->GetNodeParent();
-  
-  if (parent)
-    PopulateWithStartingAfter(parent, aStartRoot, aElementsToAppend);
-}
-
 void 
 nsContentList::PopulateSelf(PRUint32 aNeededLength)
 {
@@ -874,13 +892,33 @@ nsContentList::PopulateSelf(PRUint32 aNeededLength)
   PRUint32 invariant = elementsToAppend + mElements.Count();
 #endif
 
-  // If we already have nodes start searching at the last one, otherwise
-  // start searching at the root.
-  nsINode* startRoot = count == 0 ? mRootNode : mElements[count - 1];
+  if (mDeep) {
+    // If we already have nodes start searching at the last one, otherwise
+    // start searching at the root.
+    nsINode* cur = count ? mElements[count - 1] : mRootNode;
+    do {
+      cur = cur->GetNextNode(mRootNode);
+      if (!cur) {
+        break;
+      }
+      if (cur->IsElement() && Match(cur->AsElement())) {
+        mElements.AppendObject(cur->AsElement());
+        --elementsToAppend;
+      }
+    } while (elementsToAppend);
+  } else {
+    nsIContent* cur =
+      count ? mElements[count-1]->GetNextSibling() : mRootNode->GetFirstChild();
+    for ( ; cur && elementsToAppend; cur = cur->GetNextSibling()) {
+      if (cur->IsElement() && Match(cur->AsElement())) {
+        mElements.AppendObject(cur);
+        --elementsToAppend;
+      }
+    }
+  }
 
-  PopulateWithStartingAfter(startRoot, nsnull, elementsToAppend);
   NS_ASSERTION(elementsToAppend + mElements.Count() == invariant,
-               "Something is awry in PopulateWith!");
+               "Something is awry!");
 
   if (elementsToAppend != 0)
     mState = LIST_UP_TO_DATE;
@@ -901,8 +939,10 @@ nsContentList::RemoveFromHashtable()
   if (!gContentListHashTable.ops)
     return;
 
+  nsDependentAtomString str(mXMLMatchAtom);
+  nsContentListKey key(mRootNode, mMatchNameSpaceId, str);
   PL_DHashTableOperate(&gContentListHashTable,
-                       GetKey(),
+                       &key,
                        PL_DHASH_REMOVE);
 
   if (gContentListHashTable.entryCount == 0) {
@@ -914,7 +954,7 @@ nsContentList::RemoveFromHashtable()
 void
 nsContentList::BringSelfUpToDate(PRBool aDoFlush)
 {
-  if (mRootNode && aDoFlush) {
+  if (mRootNode && aDoFlush && mFlushesNeeded) {
     // XXX sXBL/XBL2 issue
     nsIDocument* doc = mRootNode->GetCurrentDoc();
     if (doc) {
@@ -931,6 +971,29 @@ nsContentList::BringSelfUpToDate(PRBool aDoFlush)
                "PopulateSelf dod not bring content list up to date!");
 }
 
+nsCacheableFuncStringContentList::~nsCacheableFuncStringContentList()
+{
+  RemoveFromFuncStringHashtable();
+}
+
+void
+nsCacheableFuncStringContentList::RemoveFromFuncStringHashtable()
+{
+  if (!gFuncStringContentListHashTable.ops) {
+    return;
+  }
+
+  nsFuncStringCacheKey key(mRootNode, mFunc, mString);
+  PL_DHashTableOperate(&gFuncStringContentListHashTable,
+                       &key,
+                       PL_DHASH_REMOVE);
+
+  if (gFuncStringContentListHashTable.entryCount == 0) {
+    PL_DHashTableFinish(&gFuncStringContentListHashTable);
+    gFuncStringContentListHashTable.ops = nsnull;
+  }
+}
+
 #ifdef DEBUG_CONTENT_LIST
 void
 nsContentList::AssertInSync()
@@ -949,7 +1012,7 @@ nsContentList::AssertInSync()
   // elements that are outside of the document element.
   nsIContent *root;
   if (mRootNode->IsNodeOfType(nsINode::eDOCUMENT)) {
-    root = static_cast<nsIDocument*>(mRootNode)->GetRootContent();
+    root = static_cast<nsIDocument*>(mRootNode)->GetRootElement();
   }
   else {
     root = static_cast<nsIContent*>(mRootNode);
@@ -974,7 +1037,7 @@ nsContentList::AssertInSync()
       break;
     }
 
-    if (cur->IsNodeOfType(nsINode::eELEMENT) && Match(cur)) {
+    if (cur->IsElement() && Match(cur->AsElement())) {
       NS_ASSERTION(cnt < mElements.Count() && mElements[cnt] == cur,
                    "Elements is out of sync");
       ++cnt;
