@@ -56,6 +56,9 @@
 #include "nsSVGUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIPrefService.h"
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 
 /*static*/ PRBool
 nsSVGFeatures::HaveFeature(nsISupports* aObject, const nsAString& aFeature)
@@ -103,9 +106,7 @@ nsSVGFeatures::HaveExtension(const nsAString& aExtension)
 {
 #define SVG_SUPPORTED_EXTENSION(str) if (aExtension.EqualsLiteral(str)) return PR_TRUE;
   SVG_SUPPORTED_EXTENSION("http://www.w3.org/1999/xhtml")
-#ifdef MOZ_MATHML
   SVG_SUPPORTED_EXTENSION("http://www.w3.org/1998/Math/MathML")
-#endif
 #undef SVG_SUPPORTED_EXTENSION
 
   return PR_FALSE;
@@ -239,7 +240,7 @@ nsSVGFeatures::PassesConditionalProcessingTests(nsIContent *aContent,
                         value)) {
 
     const nsAutoString acceptLangs(aAcceptLangs ? *aAcceptLangs :
-      nsContentUtils::GetLocalizedStringPref("intl.accept_languages"));
+      Preferences::GetLocalizedString("intl.accept_languages"));
 
     // Get our language preferences
     if (!acceptLangs.IsEmpty()) {
