@@ -38,9 +38,8 @@
 
 #include "nsHTMLObjectResizer.h"
 
-#include "nsIDOMEventTarget.h"
 #include "nsIDOMNSHTMLElement.h"
-#include "nsPIDOMEventTarget.h"
+#include "nsIDOMEventTarget.h"
 #include "nsIDOMText.h"
 
 #include "nsIDOMCSSValue.h"
@@ -60,14 +59,14 @@
 
 #include "nsPoint.h"
 
-#include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
 #include "nsIServiceManager.h"
+#include "mozilla/Preferences.h"
 
-#include "nsPresContext.h"
 #include "nsILookAndFeel.h"
 #include "nsWidgetsCID.h"
 
+using namespace mozilla;
+
 class nsHTMLEditUtils;
 
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
@@ -131,7 +130,7 @@ ResizerSelectionListener::NotifySelectionChanged(nsIDOMDocument *, nsISelection
 // ResizerMouseMotionListener
 // ==================================================================
 
-NS_IMPL_ISUPPORTS2(ResizerMouseMotionListener, nsIDOMEventListener, nsIDOMMouseMotionListener)
+NS_IMPL_ISUPPORTS1(ResizerMouseMotionListener, nsIDOMEventListener)
 
 ResizerMouseMotionListener::ResizerMouseMotionListener(nsIHTMLEditor * aEditor)
 {
@@ -142,9 +141,8 @@ ResizerMouseMotionListener::~ResizerMouseMotionListener()
 {
 }
 
-
 NS_IMETHODIMP
-ResizerMouseMotionListener::MouseMove(nsIDOMEvent* aMouseEvent)
+ResizerMouseMotionListener::HandleEvent(nsIDOMEvent* aMouseEvent)
 {
   nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
   if (!mouseEvent) {
@@ -163,18 +161,6 @@ ResizerMouseMotionListener::MouseMove(nsIDOMEvent* aMouseEvent)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-ResizerMouseMotionListener::HandleEvent(nsIDOMEvent* aMouseEvent)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-ResizerMouseMotionListener::DragMove(nsIDOMEvent* aMouseEvent)
-{
-  return NS_OK;
-}
-
 // ==================================================================
 // nsHTMLEditor
 // ==================================================================
@@ -188,13 +174,13 @@ nsHTMLEditor::CreateResizer(nsIDOMElement ** aReturn, PRInt16 aLocation, nsIDOMN
                                         PR_FALSE,
                                         aReturn);
 
-  if (NS_FAILED(res)) return res;
-  if (!*aReturn)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_SUCCESS(res, res);
+  NS_ENSURE_TRUE(*aReturn, NS_ERROR_FAILURE);
 
   // add the mouse listener so we can detect a click on a resizer
   nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(*aReturn));
-  evtTarget->AddEventListener(NS_LITERAL_STRING("mousedown"), mMouseListenerP, PR_TRUE);
+  evtTarget->AddEventListener(NS_LITERAL_STRING("mousedown"), mEventListener,
+                              PR_TRUE);
 
   nsAutoString locationStr;
   switch (aLocation) {
@@ -247,8 +233,7 @@ nsHTMLEditor::CreateShadow(nsIDOMElement ** aReturn, nsIDOMNode * aParentNode,
                                         PR_TRUE,
                                         aReturn);
 
-  if (!*aReturn)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(*aReturn, NS_ERROR_FAILURE);
 
   return res;
 }
@@ -263,8 +248,7 @@ nsHTMLEditor::CreateResizingInfo(nsIDOMElement ** aReturn, nsIDOMNode * aParentN
                                         PR_TRUE,
                                         aReturn);
 
-  if (!*aReturn)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(*aReturn, NS_ERROR_FAILURE);
 
   return res;
 }
@@ -272,8 +256,7 @@ nsHTMLEditor::CreateResizingInfo(nsIDOMElement ** aReturn, nsIDOMNode * aParentN
 nsresult
 nsHTMLEditor::SetAllResizersPosition()
 {
-  if (!mTopLeftHandle)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(mTopLeftHandle, NS_ERROR_FAILURE);
 
   PRInt32 x = mResizedObjectX;
   PRInt32 y = mResizedObjectY;
@@ -312,8 +295,7 @@ NS_IMETHODIMP
 nsHTMLEditor::RefreshResizers()
 {
   // nothing to do if resizers are not displayed...
-  if (!mResizedObject)
-    return NS_OK;
+  NS_ENSURE_TRUE(mResizedObject, NS_OK);
 
   nsresult res = GetPositionAndDimensions(mResizedObject,
                                           mResizedObjectX,
@@ -325,9 +307,9 @@ nsHTMLEditor::RefreshResizers()
                                           mResizedObjectMarginLeft,
                                           mResizedObjectMarginTop);
 
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = SetAllResizersPosition();
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   return SetShadowPosition(mResizingShadow, mResizedObject,
                            mResizedObjectX, mResizedObjectY);
 }
@@ -360,30 +342,30 @@ nsHTMLEditor::ShowResizersInner(nsIDOMElement *aResizedElement)
   // The resizers and the shadow will be anonymous siblings of the element.
   res = CreateResizer(getter_AddRefs(mTopLeftHandle),
                       nsIHTMLObjectResizer::eTopLeft,     parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = CreateResizer(getter_AddRefs(mTopHandle),
                       nsIHTMLObjectResizer::eTop,         parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = CreateResizer(getter_AddRefs(mTopRightHandle),
                       nsIHTMLObjectResizer::eTopRight,    parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   res = CreateResizer(getter_AddRefs(mLeftHandle),
                       nsIHTMLObjectResizer::eLeft,        parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = CreateResizer(getter_AddRefs(mRightHandle),
                       nsIHTMLObjectResizer::eRight,       parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   res = CreateResizer(getter_AddRefs(mBottomLeftHandle),
                       nsIHTMLObjectResizer::eBottomLeft,  parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = CreateResizer(getter_AddRefs(mBottomHandle),
                       nsIHTMLObjectResizer::eBottom,      parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   res = CreateResizer(getter_AddRefs(mBottomRightHandle),
                       nsIHTMLObjectResizer::eBottomRight, parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   res = GetPositionAndDimensions(aResizedElement,
                                  mResizedObjectX,
@@ -394,31 +376,31 @@ nsHTMLEditor::ShowResizersInner(nsIDOMElement *aResizedElement)
                                  mResizedObjectBorderTop,
                                  mResizedObjectMarginLeft,
                                  mResizedObjectMarginTop);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   // and let's set their absolute positions in the document
   res = SetAllResizersPosition();
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   // now, let's create the resizing shadow
   res = CreateShadow(getter_AddRefs(mResizingShadow), parentNode,
                      aResizedElement);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   // and set its position
   res = SetShadowPosition(mResizingShadow, mResizedObject,
                           mResizedObjectX, mResizedObjectY);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   // and then the resizing info tooltip
   res = CreateResizingInfo(getter_AddRefs(mResizingInfo), parentNode);
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   // and listen to the "resize" event on the window first, get the
   // window from the document...
   nsCOMPtr<nsIDOMDocument> domDoc;
   GetDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-  if (!doc) return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_TRUE(doc, NS_ERROR_NULL_POINTER);
 
   nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(doc->GetWindow());
   if (!target) { return NS_ERROR_NULL_POINTER; }
@@ -434,12 +416,13 @@ nsHTMLEditor::ShowResizersInner(nsIDOMElement *aResizedElement)
 NS_IMETHODIMP 
 nsHTMLEditor::HideResizers(void)
 {
-  if (!mResizedObject)
-    return NS_OK;
+  NS_ENSURE_TRUE(mResizedObject, NS_OK);
 
   // get the presshell's document observer interface.
-  nsCOMPtr<nsIPresShell> ps = do_QueryReferent(mPresShellWeak);
-  if (!ps) return NS_ERROR_NOT_INITIALIZED;
+  nsCOMPtr<nsIPresShell> ps = GetPresShell();
+  // We allow the pres shell to be null; when it is, we presume there
+  // are no document observers to notify, but we still want to
+  // UnbindFromTree.
 
   nsresult res;
   nsCOMPtr<nsIDOMNode> parentNode;
@@ -453,43 +436,43 @@ nsHTMLEditor::HideResizers(void)
 
   NS_NAMED_LITERAL_STRING(mousedown, "mousedown");
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mTopLeftHandle, parentContent, ps);
   mTopLeftHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mTopHandle, parentContent, ps);
   mTopHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mTopRightHandle, parentContent, ps);
   mTopRightHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mLeftHandle, parentContent, ps);
   mLeftHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mRightHandle, parentContent, ps);
   mRightHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mBottomLeftHandle, parentContent, ps);
   mBottomLeftHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mBottomHandle, parentContent, ps);
   mBottomHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mBottomRightHandle, parentContent, ps);
   mBottomRightHandle = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mResizingShadow, parentContent, ps);
   mResizingShadow = nsnull;
 
-  RemoveListenerAndDeleteRef(mousedown, mMouseListenerP, PR_TRUE,
+  RemoveListenerAndDeleteRef(mousedown, mEventListener, PR_TRUE,
                              mResizingInfo, parentContent, ps);
   mResizingInfo = nsnull;
 
@@ -500,8 +483,7 @@ nsHTMLEditor::HideResizers(void)
 
   // don't forget to remove the listeners !
 
-  nsCOMPtr<nsPIDOMEventTarget> piTarget = GetPIDOMEventTarget();
-  nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(piTarget);
+  nsCOMPtr<nsIDOMEventTarget> target = GetDOMEventTarget();
 
   if (target && mMouseMotionListenerP)
   {
@@ -558,19 +540,8 @@ nsHTMLEditor::StartResizing(nsIDOMElement *aHandle)
   mActivatedHandle->SetAttribute(NS_LITERAL_STRING("_moz_activated"), NS_LITERAL_STRING("true"));
 
   // do we want to preserve ratio or not?
-  PRBool preserveRatio = nsHTMLEditUtils::IsImage(mResizedObject);
-  nsresult result;
-  nsCOMPtr<nsIPrefBranch> prefBranch =
-    do_GetService(NS_PREFSERVICE_CONTRACTID, &result);
-  if (NS_SUCCEEDED(result) && prefBranch && preserveRatio) {
-    result = prefBranch->GetBoolPref("editor.resizing.preserve_ratio", &preserveRatio);
-    if (NS_FAILED(result)) {
-      // just in case Anvil does not update its prefs file
-      // and because it really does not make sense to me to allow free
-      // resizing on corners without a modifier key
-      preserveRatio = PR_TRUE;
-    }
-  }
+  PRBool preserveRatio = nsHTMLEditUtils::IsImage(mResizedObject) &&
+    Preferences::GetBool("editor.resizing.preserve_ratio", PR_TRUE);
 
   // the way we change the position/size of the shadow depends on
   // the handle
@@ -613,14 +584,14 @@ nsHTMLEditor::StartResizing(nsIDOMElement *aHandle)
                                       mResizedObjectHeight);
 
   // add a mouse move listener to the editor
+  nsresult result = NS_OK;
   if (!mMouseMotionListenerP) {
     mMouseMotionListenerP = new ResizerMouseMotionListener(this);
     if (!mMouseMotionListenerP) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    nsCOMPtr<nsPIDOMEventTarget> piTarget = GetPIDOMEventTarget();
-    nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(piTarget);
+    nsCOMPtr<nsIDOMEventTarget> target = GetDOMEventTarget();
     NS_ENSURE_TRUE(target, NS_ERROR_FAILURE);
 
     result = target->AddEventListener(NS_LITERAL_STRING("mousemove"),
@@ -642,7 +613,7 @@ nsHTMLEditor::MouseDown(PRInt32 aClientX, PRInt32 aClientY,
     if (anonElement) {
       nsAutoString anonclass;
       nsresult res = aTarget->GetAttribute(NS_LITERAL_STRING("_moz_anonclass"), anonclass);
-      if (NS_FAILED(res)) return res;
+      NS_ENSURE_SUCCESS(res, res);
       if (anonclass.EqualsLiteral("mozResizer")) {
         // and that element is a resizer, let's start resizing!
         aEvent->PreventDefault();
@@ -747,11 +718,11 @@ nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt3
 
   nsCOMPtr<nsIDOMNode> textInfo;
   nsresult res = mResizingInfo->GetFirstChild(getter_AddRefs(textInfo));
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   nsCOMPtr<nsIDOMNode> junk;
   if (textInfo) {
     res = mResizingInfo->RemoveChild(textInfo, getter_AddRefs(junk));
-    if (NS_FAILED(res)) return res;
+    NS_ENSURE_SUCCESS(res, res);
     textInfo = nsnull;
     junk = nsnull;
   }
@@ -775,10 +746,10 @@ nsHTMLEditor::SetResizingInfoPosition(PRInt32 aX, PRInt32 aY, PRInt32 aW, PRInt3
 
   nsCOMPtr<nsIDOMText> nodeAsText;
   res = domdoc->CreateTextNode(info, getter_AddRefs(nodeAsText));
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
   textInfo = do_QueryInterface(nodeAsText);
   res =  mResizingInfo->AppendChild(textInfo, getter_AddRefs(junk));
-  if (NS_FAILED(res)) return res;
+  NS_ENSURE_SUCCESS(res, res);
 
   PRBool hasClass = PR_FALSE;
   if (NS_SUCCEEDED(mResizingInfo->HasAttribute(NS_LITERAL_STRING("class"), &hasClass )) && hasClass)
@@ -799,9 +770,9 @@ nsHTMLEditor::SetShadowPosition(nsIDOMElement * aShadow,
     nsAutoString imageSource;
     nsresult res = aOriginalObject->GetAttribute(NS_LITERAL_STRING("src"),
                                                 imageSource);
-    if (NS_FAILED(res)) return res;
+    NS_ENSURE_SUCCESS(res, res);
     res = aShadow->SetAttribute(NS_LITERAL_STRING("src"), imageSource);
-    if (NS_FAILED(res)) return res;
+    NS_ENSURE_SUCCESS(res, res);
   }
   return NS_OK;
 }
@@ -852,7 +823,7 @@ nsHTMLEditor::GetNewResizingX(PRInt32 aX, PRInt32 aY)
   PRInt32 resized = mResizedObjectX +
                     GetNewResizingIncrement(aX, aY, kX) * mXIncrementFactor;
   PRInt32 max =   mResizedObjectX + mResizedObjectWidth;
-  return PR_MIN(resized, max);
+  return NS_MIN(resized, max);
 }
 
 PRInt32
@@ -861,7 +832,7 @@ nsHTMLEditor::GetNewResizingY(PRInt32 aX, PRInt32 aY)
   PRInt32 resized = mResizedObjectY +
                     GetNewResizingIncrement(aX, aY, kY) * mYIncrementFactor;
   PRInt32 max =   mResizedObjectY + mResizedObjectHeight;
-  return PR_MIN(resized, max);
+  return NS_MIN(resized, max);
 }
 
 PRInt32
@@ -870,7 +841,7 @@ nsHTMLEditor::GetNewResizingWidth(PRInt32 aX, PRInt32 aY)
   PRInt32 resized = mResizedObjectWidth +
                      GetNewResizingIncrement(aX, aY, kWidth) *
                          mWidthIncrementFactor;
-  return PR_MAX(resized, 1);
+  return NS_MAX(resized, 1);
 }
 
 PRInt32
@@ -879,7 +850,7 @@ nsHTMLEditor::GetNewResizingHeight(PRInt32 aX, PRInt32 aY)
   PRInt32 resized = mResizedObjectHeight +
                      GetNewResizingIncrement(aX, aY, kHeight) *
                          mHeightIncrementFactor;
-  return PR_MAX(resized, 1);
+  return NS_MAX(resized, 1);
 }
 
 
@@ -931,8 +902,8 @@ nsHTMLEditor::MouseMove(nsIDOMEvent* aMouseEvent)
     look->GetMetric(nsILookAndFeel::eMetric_DragThresholdX, xThreshold);
     look->GetMetric(nsILookAndFeel::eMetric_DragThresholdY, yThreshold);
 
-    if (PR_ABS(clientX - mOriginalX ) * 2 >= xThreshold ||
-        PR_ABS(clientY - mOriginalY ) * 2 >= yThreshold) {
+    if (NS_ABS(clientX - mOriginalX ) * 2 >= xThreshold ||
+        NS_ABS(clientY - mOriginalY ) * 2 >= yThreshold) {
       mGrabberClicked = PR_FALSE;
       StartMoving(nsnull);
     }
