@@ -115,21 +115,6 @@ public:
   void SetDefaultTarget(gfxContext* aContext, BufferMode aDoubleBuffering);
   gfxContext* GetDefaultTarget() { return mDefaultTarget; }
 
-  /**
-   * Set a target resolution for managed layers that are scalable.  It
-   * might make sense to call this outside of a transaction, but
-   * currently it's only allowed during the construction phase of
-   * transactions.
-   */
-  void SetResolution(float aXResolution, float aYResolution)
-  {
-    NS_ASSERTION(InConstruction(), "resolution must be set before drawing");
-    mXResolution = aXResolution;
-    mYResolution = aYResolution;
-  }
-  float XResolution() const { return mXResolution; }
-  float YResolution() const { return mYResolution; }
-
   nsIWidget* GetRetainerWidget() { return mWidget; }
   void ClearRetainerWidget() { mWidget = nsnull; }
 
@@ -180,6 +165,13 @@ public:
 
   void SetTransactionIncomplete() { mTransactionIncomplete = true; }
 
+  already_AddRefed<gfxContext> PushGroupForLayer(gfxContext* aContext, Layer* aLayer,
+                                                 const nsIntRegion& aRegion,
+                                                 PRBool* aNeedsClipToVisibleRegion);
+  already_AddRefed<gfxContext> PushGroupWithCachedSurface(gfxContext *aTarget,
+                                                          gfxASurface::gfxContentType aContent);
+  void PopGroupToSourceWithCachedSurface(gfxContext *aTarget, gfxContext *aPushed);
+
   virtual PRBool IsCompositingCheap() { return PR_FALSE; }
   virtual bool HasShadowManagerInternal() const { return false; }
   bool HasShadowManager() const { return HasShadowManagerInternal(); }
@@ -193,7 +185,8 @@ protected:
 #endif
 
   // Paints aLayer to mTarget.
-  void PaintLayer(Layer* aLayer,
+  void PaintLayer(gfxContext* aTarget,
+                  Layer* aLayer,
                   DrawThebesLayerCallback aCallback,
                   void* aCallbackData,
                   ReadbackProcessor* aReadback);
@@ -201,19 +194,9 @@ protected:
   // Clear the contents of a layer
   void ClearLayer(Layer* aLayer);
 
-  already_AddRefed<gfxContext> PushGroupWithCachedSurface(gfxContext *aTarget,
-                                                          gfxASurface::gfxContentType aContent,
-                                                          gfxPoint *aSavedOffset);
-  void PopGroupWithCachedSurface(gfxContext *aTarget,
-                                 const gfxPoint& aSavedOffset);
-
   bool EndTransactionInternal(DrawThebesLayerCallback aCallback,
                               void* aCallbackData);
 
-  // Target resolution for scalable content.
-  float mXResolution;
-  float mYResolution;
-
   // Widget whose surface should be used as the basis for ThebesLayer
   // buffers.
   nsIWidget* mWidget;
@@ -227,6 +210,7 @@ protected:
 
   BufferMode   mDoubleBuffering;
   PRPackedBool mUsingDefaultTarget;
+  PRPackedBool mCachedSurfaceInUse;
   bool         mTransactionIncomplete;
 };
  
