@@ -38,8 +38,6 @@
 #ifndef GFX_CANVASLAYEROGL_H
 #define GFX_CANVASLAYEROGL_H
 
-#include "mozilla/layers/PLayers.h"
-#include "mozilla/layers/ShadowLayers.h"
 
 #include "LayerManagerOGL.h"
 #include "gfxASurface.h"
@@ -110,7 +108,7 @@ public:
 
   // CanvasLayer impl
   virtual void Initialize(const Data& aData);
-  virtual void Init(const SurfaceDescriptor& aNewFront, const nsIntSize& aSize);
+  virtual void Init(const SurfaceDescriptor& aNewFront, const nsIntSize& aSize, bool needYFlip);
 
   // This isn't meaningful for shadow canvas.
   virtual void Updated(const nsIntRect&) {}
@@ -133,6 +131,7 @@ private:
   nsRefPtr<TextureImage> mTexImage;
 
   SurfaceDescriptor mDeadweight;
+  PRPackedBool mNeedsYFlip;
 };
 
 } /* layers */
