@@ -49,8 +49,8 @@
             DebugAbort(__FILE__, __LINE__, #_cond,## __VA_ARGS__);  \
     } while (0)
 
-using mozilla::MutexAutoLock;
-using mozilla::MutexAutoUnlock;
+using mozilla::MonitorAutoLock;
+using mozilla::MonitorAutoUnlock;
 
 template<>
 struct RunnableMethodTraits<mozilla::ipc::RPCChannel>
@@ -121,10 +121,10 @@ RPCChannel::Clear()
 }
 
 bool
-RPCChannel::EventOccurred()
+RPCChannel::EventOccurred() const
 {
     AssertWorkerThread();
-    mMutex.AssertCurrentThreadOwns();
+    mMonitor.AssertCurrentThreadOwns();
     RPC_ASSERT(StackDepth() > 0, "not in wait loop");
 
     return (!Connected() ||
@@ -154,7 +154,7 @@ bool
 RPCChannel::Call(Message* msg, Message* reply)
 {
     AssertWorkerThread();
-    mMutex.AssertNotCurrentThreadOwns();
+    mMonitor.AssertNotCurrentThreadOwns();
     RPC_ASSERT(!ProcessingSyncMessage(),
                "violation of sync handler invariant");
     RPC_ASSERT(msg->is_rpc(), "can only Call() RPC messages here");
@@ -166,7 +166,7 @@ RPCChannel::Call(Message* msg, Message* reply)
     Message copy = *msg;
     CxxStackFrame f(*this, OUT_MESSAGE, &copy);
 
-    MutexAutoLock lock(mMutex);
+    MonitorAutoLock lock(mMonitor);
 
     if (!Connected()) {
         ReportConnectionError("RPCChannel");
@@ -193,7 +193,7 @@ RPCChannel::Call(Message* msg, Message* reply)
 
         // now might be the time to process a message deferred because
         // of race resolution
-        MaybeProcessDeferredIncall();
+        MaybeUndeferIncall();
 
         // here we're waiting for something to happen. see long
         // comment about the queue in RPCChannel.h
@@ -238,7 +238,7 @@ RPCChannel::Call(Message* msg, Message* reply)
         }
 
         if (!recvd.is_sync() && !recvd.is_rpc()) {
-            MutexAutoUnlock unlock(mMutex);
+            MonitorAutoUnlock unlock(mMonitor);
 
             CxxStackFrame f(*this, IN_MESSAGE, &recvd);
             AsyncChannel::OnDispatchMessage(recvd);
@@ -249,7 +249,7 @@ RPCChannel::Call(Message* msg, Message* reply)
         if (recvd.is_sync()) {
             RPC_ASSERT(mPending.empty(),
                        "other side should have been blocked");
-            MutexAutoUnlock unlock(mMutex);
+            MonitorAutoUnlock unlock(mMonitor);
 
             CxxStackFrame f(*this, IN_MESSAGE, &recvd);
             SyncChannel::OnDispatchMessage(recvd);
@@ -301,10 +301,10 @@ RPCChannel::Call(Message* msg, Message* reply)
 
         // in-call.  process in a new stack frame.
 
-        // "snapshot" the current stack depth while we own the Mutex
+        // "snapshot" the current stack depth while we own the Monitor
         size_t stackDepth = StackDepth();
         {
-            MutexAutoUnlock unlock(mMutex);
+            MonitorAutoUnlock unlock(mMonitor);
             // someone called in to us from the other side.  handle the call
             CxxStackFrame f(*this, IN_MESSAGE, &recvd);
             Incall(recvd, stackDepth);
@@ -315,14 +315,14 @@ RPCChannel::Call(Message* msg, Message* reply)
     return true;
 }
 
-bool
-RPCChannel::MaybeProcessDeferredIncall()
+void
+RPCChannel::MaybeUndeferIncall()
 {
     AssertWorkerThread();
-    mMutex.AssertCurrentThreadOwns();
+    mMonitor.AssertCurrentThreadOwns();
 
     if (mDeferred.empty())
-        return false;
+        return;
 
     size_t stackDepth = StackDepth();
 
@@ -330,10 +330,10 @@ RPCChannel::MaybeProcessDeferredIncall()
     RPC_ASSERT(mDeferred.top().rpc_remote_stack_depth_guess() <= stackDepth,
                "fatal logic error");
 
-    if (mDeferred.top().rpc_remote_stack_depth_guess() < stackDepth)
-        return false;
+    if (mDeferred.top().rpc_remote_stack_depth_guess() < RemoteViewOfStackDepth(stackDepth))
+        return;
 
-    // time to process this message
+    // maybe time to process this message
     Message call = mDeferred.top();
     mDeferred.pop();
 
@@ -341,21 +341,16 @@ RPCChannel::MaybeProcessDeferredIncall()
     RPC_ASSERT(0 < mRemoteStackDepthGuess, "fatal logic error");
     --mRemoteStackDepthGuess;
 
-    MutexAutoUnlock unlock(mMutex);
-
-    if (LoggingEnabled())
-        fprintf(stderr, "  (processing deferred in-call)\n");
-
-    CxxStackFrame f(*this, IN_MESSAGE, &call);
-    Incall(call, stackDepth);
-    return true;
+    mPending.push(call);
 }
 
 void
 RPCChannel::EnqueuePendingMessages()
 {
     AssertWorkerThread();
-    mMutex.AssertCurrentThreadOwns();
+    mMonitor.AssertCurrentThreadOwns();
+
+    MaybeUndeferIncall();
 
     for (size_t i = 0; i < mDeferred.size(); ++i)
         mWorkerLoop->PostTask(
@@ -375,10 +370,10 @@ void
 RPCChannel::FlushPendingRPCQueue()
 {
     AssertWorkerThread();
-    mMutex.AssertNotCurrentThreadOwns();
+    mMonitor.AssertNotCurrentThreadOwns();
 
     {
-        MutexAutoLock lock(mMutex);
+        MonitorAutoLock lock(mMonitor);
 
         if (mDeferred.empty()) {
             if (mPending.empty())
@@ -400,11 +395,11 @@ RPCChannel::OnMaybeDequeueOne()
     // messages here
 
     AssertWorkerThread();
-    mMutex.AssertNotCurrentThreadOwns();
+    mMonitor.AssertNotCurrentThreadOwns();
 
     Message recvd;
     {
-        MutexAutoLock lock(mMutex);
+        MonitorAutoLock lock(mMonitor);
 
         if (!Connected()) {
             ReportConnectionError("RPCChannel");
@@ -412,7 +407,7 @@ RPCChannel::OnMaybeDequeueOne()
         }
 
         if (!mDeferred.empty())
-            return MaybeProcessDeferredIncall();
+            MaybeUndeferIncall();
 
         if (mPending.empty())
             return false;
@@ -440,24 +435,24 @@ RPCChannel::OnMaybeDequeueOne()
     return true;
 }
 
+size_t
+RPCChannel::RemoteViewOfStackDepth(size_t stackDepth) const
+{
+    AssertWorkerThread();
+    return stackDepth - mOutOfTurnReplies.size();
+}
+
 void
 RPCChannel::Incall(const Message& call, size_t stackDepth)
 {
     AssertWorkerThread();
-    mMutex.AssertNotCurrentThreadOwns();
+    mMonitor.AssertNotCurrentThreadOwns();
     RPC_ASSERT(call.is_rpc() && !call.is_reply(), "wrong message type");
 
     // Race detection: see the long comment near
     // mRemoteStackDepthGuess in RPCChannel.h.  "Remote" stack depth
     // means our side, and "local" means other side.
-    //
-    // We compare the remote stack depth guess against the "remote
-    // view of stack depth" because of out-of-turn replies.  When we
-    // receive one, our actual RPC stack depth doesn't decrease, but
-    // the other side (that sent the reply) thinks it has.  So, just
-    // adjust down by the number of out-of-turn replies.
-    size_t remoteViewOfStackDepth = (stackDepth - mOutOfTurnReplies.size());
-    if (call.rpc_remote_stack_depth_guess() != remoteViewOfStackDepth) {
+    if (call.rpc_remote_stack_depth_guess() != RemoteViewOfStackDepth(stackDepth)) {
         // RPC in-calls have raced.
         // the "winner", if there is one, gets to defer processing of
         // the other side's in-call
@@ -482,8 +477,8 @@ RPCChannel::Incall(const Message& call, size_t stackDepth)
         }
 
         if (LoggingEnabled()) {
-            fprintf(stderr, "  (%s won, so we're%sdeferring)\n",
-                    winner, defer ? " " : " not ");
+            fprintf(stderr, "  (%s: %s won, so we're%sdeferring)\n",
+                    mChild ? "child" : "parent", winner, defer ? " " : " not ");
         }
 
         if (defer) {
@@ -500,6 +495,10 @@ RPCChannel::Incall(const Message& call, size_t stackDepth)
         // which will make it correct again
     }
 
+#ifdef OS_WIN
+    SyncStackFrame frame(this, true);
+#endif
+
     DispatchIncall(call);
 }
 
@@ -507,7 +506,7 @@ void
 RPCChannel::DispatchIncall(const Message& call)
 {
     AssertWorkerThread();
-    mMutex.AssertNotCurrentThreadOwns();
+    mMonitor.AssertNotCurrentThreadOwns();
     RPC_ASSERT(call.is_rpc() && !call.is_reply(),
                "wrong message type");
 
@@ -528,7 +527,7 @@ RPCChannel::DispatchIncall(const Message& call)
     reply->set_seqno(call.seqno());
 
     {
-        MutexAutoLock lock(mMutex);
+        MonitorAutoLock lock(mMonitor);
         if (ChannelConnected == mChannelState)
             SendThroughTransport(reply);
     }
@@ -583,7 +582,7 @@ RPCChannel::BlockOnParent()
     if (!mChild)
         NS_RUNTIMEABORT("child tried to block parent");
 
-    MutexAutoLock lock(mMutex);
+    MonitorAutoLock lock(mMonitor);
 
     if (mBlockedOnParent || AwaitingSyncReply() || 0 < StackDepth())
         NS_RUNTIMEABORT("attempt to block child when it's already blocked");
@@ -607,7 +606,7 @@ RPCChannel::BlockOnParent()
             Message recvd = mPending.front();
             mPending.pop();
 
-            MutexAutoUnlock unlock(mMutex);
+            MonitorAutoUnlock unlock(mMonitor);
 
             CxxStackFrame f(*this, IN_MESSAGE, &recvd);
             if (recvd.is_rpc()) {
@@ -633,7 +632,7 @@ RPCChannel::UnblockFromParent()
 
     if (!mChild)
         NS_RUNTIMEABORT("child tried to block parent");
-    MutexAutoLock lock(mMutex);
+    MonitorAutoLock lock(mMonitor);
     mBlockedOnParent = false;
 }
 
@@ -642,7 +641,7 @@ RPCChannel::ExitedCxxStack()
 {
     Listener()->OnExitedCxxStack();
     if (mSawRPCOutMsg) {
-        MutexAutoLock lock(mMutex);
+        MonitorAutoLock lock(mMonitor);
         // see long comment in OnMaybeDequeueOne()
         EnqueuePendingMessages();
         mSawRPCOutMsg = false;
@@ -652,7 +651,7 @@ RPCChannel::ExitedCxxStack()
 void
 RPCChannel::DebugAbort(const char* file, int line, const char* cond,
                        const char* why,
-                       const char* type, bool reply)
+                       const char* type, bool reply) const
 {
     fprintf(stderr,
             "###!!! [RPCChannel][%s][%s:%d] "
@@ -671,19 +670,21 @@ RPCChannel::DebugAbort(const char* file, int line, const char* cond,
             mOutOfTurnReplies.size());
     fprintf(stderr, "  Pending queue size: %lu, front to back:\n",
             mPending.size());
-    while (!mPending.empty()) {
+
+    MessageQueue pending = mPending;
+    while (!pending.empty()) {
         fprintf(stderr, "    [ %s%s ]\n",
-                mPending.front().is_rpc() ? "rpc" :
-                (mPending.front().is_sync() ? "sync" : "async"),
-                mPending.front().is_reply() ? "reply" : "");
-        mPending.pop();
+                pending.front().is_rpc() ? "rpc" :
+                (pending.front().is_sync() ? "sync" : "async"),
+                pending.front().is_reply() ? "reply" : "");
+        pending.pop();
     }
 
     NS_RUNTIMEABORT(why);
 }
 
 void
-RPCChannel::DumpRPCStack(FILE* outfile, const char* const pfx)
+RPCChannel::DumpRPCStack(FILE* outfile, const char* const pfx) const
 {
     NS_WARN_IF_FALSE(MessageLoop::current() != mWorkerLoop,
                      "The worker thread had better be paused in a debugger!");
@@ -713,7 +714,7 @@ void
 RPCChannel::OnMessageReceived(const Message& msg)
 {
     AssertIOThread();
-    MutexAutoLock lock(mMutex);
+    MonitorAutoLock lock(mMonitor);
 
     if (MaybeInterceptSpecialIOMessage(msg))
         return;
@@ -743,7 +744,7 @@ RPCChannel::OnChannelError()
 {
     AssertIOThread();
 
-    MutexAutoLock lock(mMutex);
+    MonitorAutoLock lock(mMonitor);
 
     if (ChannelClosing != mChannelState)
         mChannelState = ChannelError;
