@@ -38,62 +38,54 @@
 
 
 /**
-   File Name:          10.1.5-2.js
-   ECMA Section:       10.1.5 Global Object
+   File Name:          10.2.3-2.js
+   ECMA Section:       10.2.3 Function and Anonymous Code
    Description:
-   There is a unique global object which is created before control enters
-   any execution context. Initially the global object has the following
-   properties:
 
-   Built-in objects such as Math, String, Date, parseInt, etc. These have
-   attributes { DontEnum }.
-
-   Additional host defined properties. This may include a property whose
-   value is the global object itself, for example window in HTML.
-
-   As control enters execution contexts, and as ECMAScript code is executed,
-   additional properties may be added to the global object and the initial
-   properties may be changed.
+   The scope chain is initialized to contain the activation object followed
+   by the global object. Variable instantiation is performed using the
+   activation by the global object. Variable instantiation is performed using
+   the activation object as the variable object and using property attributes
+   { DontDelete }. The caller provides the this value. If the this value
+   provided by the caller is not an object (including the case where it is
+   null), then the this value is the global object.
 
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
-var SECTION = "10.5.1-2";
+
+var SECTION = "10.2.3-2";
 var VERSION = "ECMA_1";
 startTest();
+var TITLE   = "Function and Anonymous Code";
 
-writeHeaderToLog( SECTION + " Global Object");
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( "SECTION", "Eval Code check" );
+var o = new MyObject("hello");
 
-var EVAL_STRING = 'if ( Object == null ) { gTestcases[0].reason += " Object == null" ; }' +
-  'if ( Function == null ) { gTestcases[0].reason += " Function == null"; }' +
-  'if ( String == null ) { gTestcases[0].reason += " String == null"; }'   +
-  'if ( Array == null ) { gTestcases[0].reason += " Array == null"; }'     +
-  'if ( Number == null ) { gTestcases[0].reason += " Function == null";}'  +
-  'if ( Math == null ) { gTestcases[0].reason += " Math == null"; }'       +
-  'if ( Boolean == null ) { gTestcases[0].reason += " Boolean == null"; }' +
-  'if ( Date  == null ) { gTestcases[0].reason += " Date == null"; }'      +
-  'if ( eval == null ) { gTestcases[0].reason += " eval == null"; }'       +
-  'if ( parseInt == null ) { gTestcases[0].reason += " parseInt == null"; }' ;
+new TestCase( SECTION,
+	      "MyFunction(\"PASSED!\")",
+	      "PASSED!",
+	      MyFunction("PASSED!") );
 
-eval( EVAL_STRING );
+var o = MyFunction();
 
-/*
-  if ( NaN == null ) {
-  gTestcases[0].reason += " NaN == null";
-  }
-  if ( Infinity == null ) {
-  gTestcases[0].reason += " Infinity == null";
-  }
-*/
-
-if ( gTestcases[0].reason != "" ) {
-  gTestcases[0].actual = "fail";
-} else {
-  gTestcases[0].actual = "pass";
-}
-gTestcases[0].expect = "pass";
+new TestCase( SECTION,
+	      "MyOtherFunction(true);",
+	      false,
+	      MyOtherFunction(true) );
 
 test();
 
+function MyFunction( value ) {
+  var x = value;
+  delete x;
+  return x;
+}
+function MyOtherFunction(value) {
+  var x = value;
+  return delete x;
+}
+function MyObject( value ) {
+  this.THIS = this;
+}
