@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
+/***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Mozilla Public License Version
@@ -38,38 +38,59 @@
 
 
 /**
-   File Name:          7.4.2-8-n.js
-   ECMA Section:       7.4.2
+   File Name:          7.4.3-15-n.js
+   ECMA Section:       7.4.3
 
    Description:
-   The following tokens are ECMAScript keywords and may not be used as
-   identifiers in ECMAScript programs.
+   The following words are used as keywords in proposed extensions and are
+   therefore reserved to allow for the possibility of future adoption of
+   those extensions.
 
-   Syntax
-
-   Keyword :: one of
-   break          for         new         var
-   continue       function    return      void
-   delete         if          this        while
-   else           in          typeof      with
-
-   This test verifies that the keyword cannot be used as an identifier.
-   Functioinal tests of the keyword may be found in the section corresponding
-   to the function of the keyword.
+   FutureReservedWord :: one of
+   case    debugger    export      super
+   catch   default     extends     switch
+   class   do          finally     throw
+   const   enum        import      try
 
    Author:             christine@netscape.com
    Date:               12 november 1997
-
 */
-var SECTION = "7.4.2-8";
+var SECTION = "7.4.3-15-n";
 var VERSION = "ECMA_1";
 startTest();
+var TITLE   = "Future Reserved Words";
 
-writeHeaderToLog( SECTION + " Keywords");
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var void = true";
+var actual = 'no error';
+var prefValue;
+
+print("This test requires option javascript.options.strict enabled");
+
+options('strict');
+options('werror');
+
+try
+{
+  eval("var import = true");
+}
+catch(e)
+{
+  actual = 'error';
+}
+
+DESCRIPTION = "var import = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var void = true",     "error",    eval("var void = true") );
+// force exception since this is a negative test
+if (actual == 'error')
+{
+  throw actual;
+}
+
+new TestCase( SECTION, 
+              "var import = true",    
+              "error",   
+              actual );
 
 test();
