@@ -38,22 +38,59 @@
 
 
 /**
-   File Name:          15.7.4-1.js
-   ECMA Section:       15.7.4.1 Properties of the Number Prototype Object
+   File Name:          15.7.4.3-1.js
+   ECMA Section:       15.7.4.3.1 Number.prototype.valueOf()
    Description:
+   Returns this number value.
+
+   The valueOf function is not generic; it generates a runtime error if its
+   this value is not a Number object. Therefore it cannot be transferred to
+   other kinds of objects for use as a method.
+
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-
-
-var SECTION = "15.7.4-1";
+var SECTION = "15.7.4.3-1";
 var VERSION = "ECMA_1";
 startTest();
-writeHeaderToLog( SECTION + "Properties of the Number prototype object");
 
-new TestCase(SECTION, "Number.prototype.valueOf()",      0,                  Number.prototype.valueOf() );
-new TestCase(SECTION, "typeof(Number.prototype)",        "object",           typeof(Number.prototype) );
-new TestCase(SECTION, "Number.prototype.constructor == Number",    true,     Number.prototype.constructor == Number );
-//    new TestCase(SECTION, "Number.prototype == Number.__proto__",      true,   Number.prototype == Number.__proto__ );
+
+writeHeaderToLog( SECTION + " Number.prototype.valueOf()");
+
+//  the following two line causes navigator to crash -- cmb 9/16/97
+new TestCase("SECTION",
+	     "Number.prototype.valueOf()",       
+	     0,       
+	     eval("Number.prototype.valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number(1)).valueOf()",        
+	     1,      
+	     eval("(new Number(1)).valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number(-1)).valueOf()",       
+	     -1,     
+	     eval("(new Number(-1)).valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number(0)).valueOf()",        
+	     0,      
+	     eval("(new Number(0)).valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number(Number.POSITIVE_INFINITY)).valueOf()",
+	     Number.POSITIVE_INFINITY,
+	     eval("(new Number(Number.POSITIVE_INFINITY)).valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number(Number.NaN)).valueOf()", 
+	     Number.NaN,
+	     eval("(new Number(Number.NaN)).valueOf()") );
+
+new TestCase("SECTION",
+	     "(new Number()).valueOf()",        
+	     0,      
+	     eval("(new Number()).valueOf()") );
 
 test();
