@@ -16,11 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Phil Schwartau <pschwartau@meer.net>
- *                 Bob Clary <bob@bclary.com>
+ * Contributor(s): Bertrand Le Roy
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,47 +36,43 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 99120;
-var summary = 'sort() should not be O(N^2) on sorted data';
+var BUGNUMBER = 350256;
+var summary = 'Array.apply maximum arguments: 2^19-1024';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+expectExitCode(0);
+expectExitCode(5);
 
-var data = {X:[], Y:[]};
-for (var size = 5000; size <= 15000; size += 1000)
-{ 
-  data.X.push(size);
-  data.Y.push(testSort(size));
-  gc();
-}
+//-----------------------------------------------------------------------------
+test(Math.pow(2, 19)-1024);
+//-----------------------------------------------------------------------------
 
-var order = BigO(data);
-
-var msg = '';
-for (var p = 0; p < data.X.length; p++)
+function test(length)
 {
-  msg += '(' + data.X[p] + ', ' + data.Y[p] + '); ';
-}
-printStatus(msg);
-printStatus('Order: ' + order);
-reportCompare(true, order < 2 , 'BigO ' + order + ' < 2');
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
 
-function testSort(size)
-{
-  var arry = new Array(size);
-  for (var i = 0; i < size; i++)
-  {
-    arry[i] = i;
+  var a = new Array();
+  a[length - 2] = 'length-2';
+  a[length - 1] = 'length-1';
+
+  var b = Array.apply(null, a);
+
+  expect = length + ',length-2,length-1';
+  actual = b.length + "," + b[length - 2] + "," + b[length - 1];
+  reportCompare(expect, actual, summary);
+
+  function f() {
+    return arguments.length + "," + arguments[length - 2] + "," +
+      arguments[length - 1];
   }
-  var start = new Date();
-  arry.sort(compareFn);
-  var stop = new Date();
-  return stop - start;
-}
 
-function compareFn(a, b)
-{
-  return a - b;
+  expect = length + ',length-2,length-1';
+  actual = f.apply(null, a);
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
 }
