@@ -16,11 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov  <igor@mir2.org>
- *                 Bob Clary <bob@bclary.com>
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,44 +36,25 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 280769;
-var summary = 'Do not overflow 64K length of char sequence in RegExp []';
+var BUGNUMBER = 387951;
+var summary = 'Do not assert: cg->stackDepth >= 0';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
 
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-var N = 20 * 1000; // It should be that N*6  > 64K and N < 32K
-
-var prefixes = ["000", "00", "0"];
-
-function to_4_hex(i)
+function test()
 {
-  var printed = i.toString(16);
-  if (printed.length < 4) {
-    printed= prefixes[printed.length - 1]+printed;
-  }
-  return printed;
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+ 
+  "" + (function() { if(delete(null?0:{})){[]} });
 
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
 }
-
-var a = new Array(N);
-for (var i = 0; i != N; ++i) {
-  a[i] = to_4_hex(2*i);
-}
-
-var str = '[\\u'+a.join('\\u')+']';
-// str is [\u0000\u0002\u0004...\u<printed value of 2N>]
-
-var re = new RegExp(str);
-
-var string_to_match = String.fromCharCode(2 * (N - 1));
-
-var value = re.exec(string_to_match);
-
-var expect =  string_to_match;
-var actual = value ? value[0] : value;
-
-reportCompare(expect, actual, summary);
