@@ -39,7 +39,7 @@
 
 // Interfaces
 #include "nsIDOMWindow.h"
-#include "nsIDeviceContext.h"
+#include "nsDeviceContext.h"
 #include "nsIPrintProgressParams.h"
 #include "nsIPrintOptions.h"
 #include "nsTArray.h"
@@ -92,7 +92,7 @@ public:
 
 
   ePrintDataType               mType;            // the type of data this is (Printing or Print Preview)
-  nsCOMPtr<nsIDeviceContext>   mPrintDC;
+  nsRefPtr<nsDeviceContext>   mPrintDC;
   FILE                        *mDebugFilePtr;    // a file where information can go to when printing
 
   nsPrintObject *                mPrintObject;
