@@ -47,6 +47,7 @@
 #include "nsBox.h"
 #include "nsIScrollableFrame.h"
 #include "nsSprocketLayout.h"
+#include "nsGridLayout2.h"
 #include "nsGridRow.h"
 #include "nsGridCell.h"
 
@@ -297,24 +298,21 @@ nsGrid::FindRowsAndColumns(nsIBox** aRows, nsIBox** aColumns)
        child = do_QueryFrame(scrolledFrame);
     }
 
-    nsCOMPtr<nsIBoxLayout> layout;
-    child->GetLayoutManager(getter_AddRefs(layout));
+    nsCOMPtr<nsIGridPart> monument = GetPartFromBox(child);
+    if (monument)
+    {
+      nsGridRowGroupLayout* rowGroup = monument->CastToRowGroupLayout();
+      if (rowGroup) {
+         PRBool isHorizontal = !nsSprocketLayout::IsHorizontal(child);
+         if (isHorizontal)
+           *aRows = child;
+         else
+           *aColumns = child;
 
-     nsCOMPtr<nsIGridPart> monument( do_QueryInterface(layout) );
-     if (monument)
-     {
-       nsGridRowGroupLayout* rowGroup = monument->CastToRowGroupLayout();
-       if (rowGroup) {
-          PRBool isHorizontal = !nsSprocketLayout::IsHorizontal(child);
-          if (isHorizontal)
-            *aRows = child;
-          else
-            *aColumns = child;
-         
-          if (*aRows && *aColumns)
-            return;
-       }
-     }
+         if (*aRows && *aColumns)
+           return;
+      }
+    }
 
     if (scrollFrame) {
       child = oldBox;
@@ -336,13 +334,9 @@ nsGrid::CountRowsColumns(nsIBox* aRowBox, PRInt32& aRowCount, PRInt32& aComputed
   aComputedColumnCount = 0;
   // get the rowboxes layout manager. Then ask it to do the work for us
   if (aRowBox) {
-    nsCOMPtr<nsIBoxLayout> layout;
-    aRowBox->GetLayoutManager(getter_AddRefs(layout));
-    if (layout) {
-       nsCOMPtr<nsIGridPart> monument( do_QueryInterface(layout) );
-       if (monument) 
-          monument->CountRowsColumns(aRowBox, aRowCount, aComputedColumnCount);
-    }
+    nsCOMPtr<nsIGridPart> monument = GetPartFromBox(aRowBox);
+    if (monument) 
+       monument->CountRowsColumns(aRowBox, aRowCount, aComputedColumnCount);
   }
 }
 
@@ -394,13 +388,9 @@ nsGrid::BuildRows(nsIBox* aBox, PRInt32 aRowCount, nsGridRow** aRows, PRBool aIs
   // populate it if we can. If not it will contain only dynamic columns
   if (aBox)
   {
-    nsCOMPtr<nsIBoxLayout> layout;
-    aBox->GetLayoutManager(getter_AddRefs(layout));
-    if (layout) {
-      nsCOMPtr<nsIGridPart> monument( do_QueryInterface(layout) );
-      if (monument) {
-         monument->BuildRows(aBox, row);
-      }
+    nsCOMPtr<nsIGridPart> monument = GetPartFromBox(aBox);
+    if (monument) {
+       monument->BuildRows(aBox, row);
     }
   }
 
@@ -499,32 +489,14 @@ nsGrid::DirtyRows(nsIBox* aRowBox, nsBoxLayoutState& aState)
 
   // if the box is a grid part have it recursively hand it.
   if (aRowBox) {
-    nsCOMPtr<nsIBoxLayout> layout;
-    aRowBox->GetLayoutManager(getter_AddRefs(layout));
-    if (layout) {
-       nsCOMPtr<nsIGridPart> part( do_QueryInterface(layout) );
-       if (part) 
-          part->DirtyRows(aRowBox, aState);
-    }
+    nsCOMPtr<nsIGridPart> part = GetPartFromBox(aRowBox);
+    if (part) 
+       part->DirtyRows(aRowBox, aState);
   }
 
   mMarkingDirty = PR_FALSE;
 }
 
-nsGridRow* nsGrid::GetColumns()
-{
-  RebuildIfNeeded();
-
-  return mColumns;
-}
-
-nsGridRow* nsGrid::GetRows()
-{
-  RebuildIfNeeded();
-
-  return mRows;
-}
-
 nsGridRow*
 nsGrid::GetColumnAt(PRInt32 aIndex, PRBool aIsHorizontal)
 {
@@ -617,22 +589,15 @@ nsGrid::GetMaxRowSize(nsBoxLayoutState& aState, PRInt32 aRowIndex, PRBool aIsHor
   return size;
 }
 
-void 
-nsGrid::GetPartFromBox(nsIBox* aBox, nsIGridPart** aPart)
+// static
+nsIGridPart*
+nsGrid::GetPartFromBox(nsIBox* aBox)
 {
-  *aPart = nsnull;
+  if (!aBox)
+    return nsnull;
 
-  if (aBox) {
-    nsCOMPtr<nsIBoxLayout> layout;
-    aBox->GetLayoutManager(getter_AddRefs(layout));
-    if (layout) {
-       nsCOMPtr<nsIGridPart> part( do_QueryInterface(layout) );
-       if (part) { 
-          *aPart = part.get();
-          NS_IF_ADDREF(*aPart);
-       }
-    }
-  }
+  nsIBoxLayout* layout = aBox->GetLayoutManager();
+  return layout ? layout->AsGridPart() : nsnull;
 }
 
 nsMargin
@@ -642,8 +607,7 @@ nsGrid::GetBoxTotalMargin(nsIBox* aBox, PRBool aIsHorizontal)
   // walk the boxes parent chain getting the border/padding/margin of our parent rows
   
   // first get the layour manager
-  nsCOMPtr<nsIGridPart> part;
-  GetPartFromBox(aBox, getter_AddRefs(part));
+  nsIGridPart* part = GetPartFromBox(aBox);
   if (part)
     margin = part->GetTotalMargin(aBox, aIsHorizontal);
 
@@ -893,8 +857,9 @@ nsGrid::GetPrefRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHor
   // set in CSS?
   if (box) 
   {
+    PRBool widthSet, heightSet;
     nsSize cssSize(-1, -1);
-    nsIBox::AddCSSPrefSize(aState, box, cssSize);
+    nsIBox::AddCSSPrefSize(box, cssSize, widthSet, heightSet);
 
     row->mPref = GET_HEIGHT(cssSize, aIsHorizontal);
 
@@ -917,7 +882,7 @@ nsGrid::GetPrefRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHor
      {
        size = box->GetPrefSize(aState);
        nsBox::AddMargin(box, size);
-       nsStackLayout::AddOffset(aState, box, size);
+       nsGridLayout2::AddOffset(aState, box, size);
      }
 
      row->mPref = GET_HEIGHT(size, aIsHorizontal);
@@ -968,8 +933,9 @@ nsGrid::GetMinRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHori
 
   // set in CSS?
   if (box) {
+    PRBool widthSet, heightSet;
     nsSize cssSize(-1, -1);
-    nsIBox::AddCSSMinSize(aState, box, cssSize);
+    nsIBox::AddCSSMinSize(aState, box, cssSize, widthSet, heightSet);
 
     row->mMin = GET_HEIGHT(cssSize, aIsHorizontal);
 
@@ -991,7 +957,7 @@ nsGrid::GetMinRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHori
      if (box) {
        size = box->GetPrefSize(aState);
        nsBox::AddMargin(box, size);
-       nsStackLayout::AddOffset(aState, box, size);
+       nsGridLayout2::AddOffset(aState, box, size);
      }
 
      row->mMin = GET_HEIGHT(size, aIsHorizontal) + top + bottom;
@@ -1042,11 +1008,10 @@ nsGrid::GetMaxRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHori
 
   // set in CSS?
   if (box) {
-    nsSize cssSize;
-    cssSize.width = -1;
-    cssSize.height = -1;
-    nsIBox::AddCSSMaxSize(aState, box, cssSize);
-    
+    PRBool widthSet, heightSet;
+    nsSize cssSize(-1, -1);
+    nsIBox::AddCSSMaxSize(box, cssSize, widthSet, heightSet);
+
     row->mMax = GET_HEIGHT(cssSize, aIsHorizontal);
 
     // yep do nothing.
@@ -1067,7 +1032,7 @@ nsGrid::GetMaxRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHori
      if (box) {
        size = box->GetPrefSize(aState);
        nsBox::AddMargin(box, size);
-       nsStackLayout::AddOffset(aState, box, size);
+       nsGridLayout2::AddOffset(aState, box, size);
      }
 
      row->mMax = GET_HEIGHT(size, aIsHorizontal);
@@ -1104,11 +1069,7 @@ nsGrid::GetMaxRowHeight(nsBoxLayoutState& aState, PRInt32 aIndex, PRBool aIsHori
 PRBool
 nsGrid::IsGrid(nsIBox* aBox)
 {
-  if (!aBox)
-    return PR_FALSE;
-
-  nsCOMPtr<nsIGridPart> part;
-  GetPartFromBox(aBox, getter_AddRefs(part));
+  nsIGridPart* part = GetPartFromBox(aBox);
   if (!part)
     return PR_FALSE;
 
@@ -1319,8 +1280,6 @@ nsGrid::GetScrollBox(nsIBox* aChild)
 
   // get parent
   nsIBox* parent = aChild->GetParentBox();
-  nsCOMPtr<nsIBoxLayout> layout;
-  nsCOMPtr<nsIGridPart> parentGridRow;
 
   // walk up until we find a scrollframe or a part
   // if it's a scrollframe return it.
@@ -1332,8 +1291,7 @@ nsGrid::GetScrollBox(nsIBox* aChild)
     if (scrollFrame)
       return parent;
 
-    parent->GetLayoutManager(getter_AddRefs(layout));
-    parentGridRow = do_QueryInterface(layout);
+    nsCOMPtr<nsIGridPart> parentGridRow = GetPartFromBox(parent);
     // if a part then just return the child
     if (parentGridRow) 
       break;
