@@ -50,7 +50,6 @@
 class nsGridRowGroupLayout;
 class nsGridLayout2;
 class nsBoxLayoutState;
-class nsIPresShell;
 class nsGrid;
 
 /**
@@ -69,7 +68,7 @@ public:
   virtual nsGridRowGroupLayout* CastToRowGroupLayout() { return nsnull; }
   virtual nsGridLayout2* CastToGridLayout() { return nsnull; }
   virtual nsGrid* GetGrid(nsIBox* aBox, PRInt32* aIndex, nsGridRowLayout* aRequestor=nsnull);
-  virtual void GetParentGridPart(nsIBox* aBox, nsIBox** aParentBox, nsIGridPart** aParentGridRow);
+  virtual nsIGridPart* GetParentGridPart(nsIBox* aBox, nsIBox** aParentBox);
   virtual void ChildrenInserted(nsIBox* aBox, nsBoxLayoutState& aState,
                                 nsIBox* aPrevBox,
                                 const nsFrameList::Slice& aNewChildren);
@@ -79,6 +78,8 @@ public:
   virtual void ChildrenSet(nsIBox* aBox, nsBoxLayoutState& aState, nsIBox* aChildList);
   virtual nsMargin GetTotalMargin(nsIBox* aBox, PRBool aIsHorizontal);
 
+  virtual nsIGridPart* AsGridPart() { return this; }
+
 protected:
   virtual void ChildAddedOrRemoved(nsIBox* aBox, nsBoxLayoutState& aState)=0;
 
