@@ -42,7 +42,6 @@
 #include "nsJARProtocolHandler.h"
 #include "nsMimeTypes.h"
 #include "nsNetUtil.h"
-#include "nsInt64.h"
 #include "nsEscape.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
@@ -52,7 +51,10 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
 #include "nsIFileURL.h"
-#include "nsIJAR.h"
+
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
 
 static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 
@@ -85,17 +87,22 @@ public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSIINPUTSTREAM
 
-    nsJARInputThunk(nsIFile *jarFile,
+    nsJARInputThunk(nsIZipReader *zipReader,
                     nsIURI* fullJarURI,
                     const nsACString &jarEntry,
                     nsIZipReaderCache *jarCache)
         : mJarCache(jarCache)
-        , mJarFile(jarFile)
-        , mFullJarURI(fullJarURI)
+        , mJarReader(zipReader)
         , mJarEntry(jarEntry)
         , mContentLength(-1)
     {
-        NS_ASSERTION(mJarFile, "no jar file");
+        if (fullJarURI) {
+#ifdef DEBUG
+            nsresult rv =
+#endif
+                fullJarURI->GetAsciiSpec(mJarDirSpec);
+            NS_ASSERTION(NS_SUCCEEDED(rv), "this shouldn't fail");
+        }
     }
 
     virtual ~nsJARInputThunk()
@@ -120,8 +127,7 @@ private:
 
     nsCOMPtr<nsIZipReaderCache> mJarCache;
     nsCOMPtr<nsIZipReader>      mJarReader;
-    nsCOMPtr<nsIFile>           mJarFile;
-    nsCOMPtr<nsIURI>            mFullJarURI;
+    nsCString                   mJarDirSpec;
     nsCOMPtr<nsIInputStream>    mJarStream;
     nsCString                   mJarEntry;
     PRInt32                     mContentLength;
@@ -136,26 +142,13 @@ nsJARInputThunk::EnsureJarStream()
         return NS_OK;
 
     nsresult rv;
-    if (mJarCache)
-        rv = mJarCache->GetZip(mJarFile, getter_AddRefs(mJarReader));
-    else {
-        // create an uncached jar reader
-        mJarReader = do_CreateInstance(kZipReaderCID, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = mJarReader->Open(mJarFile);
-    }
-    if (NS_FAILED(rv)) return rv;
-
     if (ENTRY_IS_DIRECTORY(mJarEntry)) {
         // A directory stream also needs the Spec of the FullJarURI
         // because is included in the stream data itself.
 
-        nsCAutoString jarDirSpec;
-        rv = mFullJarURI->GetAsciiSpec(jarDirSpec);
-        if (NS_FAILED(rv)) return rv;
+        NS_ENSURE_STATE(!mJarDirSpec.IsEmpty());
 
-        rv = mJarReader->GetInputStreamWithSpec(jarDirSpec,
+        rv = mJarReader->GetInputStreamWithSpec(mJarDirSpec,
                                                 mJarEntry.get(),
                                                 getter_AddRefs(mJarStream));
     }
@@ -172,7 +165,8 @@ nsJARInputThunk::EnsureJarStream()
     }
 
     // ask the JarStream for the content length
-    mJarStream->Available((PRUint32 *) &mContentLength);
+    rv = mJarStream->Available((PRUint32 *) &mContentLength);
+    if (NS_FAILED(rv)) return rv;
 
     return NS_OK;
 }
@@ -301,9 +295,40 @@ nsJARChannel::CreateJarInput(nsIZipReaderCache *jarCache)
     // necessarily MT-safe
     nsCOMPtr<nsIFile> clonedFile;
     nsresult rv = mJarFile->Clone(getter_AddRefs(clonedFile));
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv))
+        return rv;
 
-    mJarInput = new nsJARInputThunk(clonedFile, mJarURI, mJarEntry, jarCache);
+    nsCOMPtr<nsIZipReader> reader;
+    if (jarCache) {
+        if (mInnerJarEntry.IsEmpty())
+            rv = jarCache->GetZip(mJarFile, getter_AddRefs(reader));
+        else 
+            rv = jarCache->GetInnerZip(mJarFile, mInnerJarEntry.get(),
+                                       getter_AddRefs(reader));
+    } else {
+        // create an uncached jar reader
+        nsCOMPtr<nsIZipReader> outerReader = do_CreateInstance(kZipReaderCID, &rv);
+        if (NS_FAILED(rv))
+            return rv;
+
+        rv = outerReader->Open(mJarFile);
+        if (NS_FAILED(rv))
+            return rv;
+
+        if (mInnerJarEntry.IsEmpty())
+            reader = outerReader;
+        else {
+            reader = do_CreateInstance(kZipReaderCID, &rv);
+            if (NS_FAILED(rv))
+                return rv;
+
+            rv = reader->OpenInner(outerReader, mInnerJarEntry.get());
+        }
+    }
+    if (NS_FAILED(rv))
+        return rv;
+
+    mJarInput = new nsJARInputThunk(reader, mJarURI, mJarEntry, jarCache);
     if (!mJarInput)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(mJarInput);
@@ -336,6 +361,21 @@ nsJARChannel::EnsureJarInput(PRBool blocking)
         if (fileURL)
             fileURL->GetFile(getter_AddRefs(mJarFile));
     }
+    // try to handle a nested jar
+    if (!mJarFile) {
+        nsCOMPtr<nsIJARURI> jarURI = do_QueryInterface(mJarBaseURI);
+        if (jarURI) {
+            nsCOMPtr<nsIFileURL> fileURL;
+            nsCOMPtr<nsIURI> innerJarURI;
+            rv = jarURI->GetJARFile(getter_AddRefs(innerJarURI));
+            if (NS_SUCCEEDED(rv))
+                fileURL = do_QueryInterface(innerJarURI);
+            if (fileURL) {
+                fileURL->GetFile(getter_AddRefs(mJarFile));
+                jarURI->GetJAREntry(mInnerJarEntry);
+            }
+        }
+    }
 
     if (mJarFile) {
         mIsUnsafe = PR_FALSE;
@@ -494,14 +534,8 @@ nsJARChannel::GetOwner(nsISupports **result)
     if (!jarReader)
         return NS_ERROR_NOT_INITIALIZED;
 
-    nsCOMPtr<nsIJAR> jar = do_QueryInterface(jarReader, &rv);
-    if (NS_FAILED(rv)) {
-        NS_ERROR("nsIJAR not supported");
-        return rv;
-    }
-
     nsCOMPtr<nsIPrincipal> cert;
-    rv = jar->GetCertificatePrincipal(mJarEntry.get(), getter_AddRefs(cert));
+    rv = jarReader->GetCertificatePrincipal(mJarEntry.get(), getter_AddRefs(cert));
     if (NS_FAILED(rv)) return rv;
 
     if (cert) {
@@ -670,7 +704,8 @@ nsJARChannel::Open(nsIInputStream **stream)
 
     // force load the jar file now so GetContentLength will return a
     // meaningful value once we return.
-    mJarInput->EnsureJarStream();
+    rv = mJarInput->EnsureJarStream();
+    if (NS_FAILED(rv)) return rv;
 
     NS_ADDREF(*stream = mJarInput);
     return NS_OK;
@@ -808,18 +843,9 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
         }
     }
 
-    if (NS_SUCCEEDED(status) && mIsUnsafe) {
-        PRBool allowUnpack = PR_FALSE;
-
-        nsCOMPtr<nsIPrefBranch> prefs =
-            do_GetService(NS_PREFSERVICE_CONTRACTID);
-        if (prefs) {
-            prefs->GetBoolPref("network.jar.open-unsafe-types", &allowUnpack);
-        }
-
-        if (!allowUnpack) {
-            status = NS_ERROR_UNSAFE_CONTENT_TYPE;
-        }
+    if (NS_SUCCEEDED(status) && mIsUnsafe &&
+        !Preferences::GetBool("network.jar.open-unsafe-types", PR_FALSE)) {
+        status = NS_ERROR_UNSAFE_CONTENT_TYPE;
     }
 
     if (NS_SUCCEEDED(status)) {
@@ -911,8 +937,8 @@ nsJARChannel::OnDataAvailable(nsIRequest *req, nsISupports *ctx,
     // nsITransportEventSink implementation.
     // XXX do the 64-bit stuff for real
     if (mProgressSink && NS_SUCCEEDED(rv) && !(mLoadFlags & LOAD_BACKGROUND))
-        mProgressSink->OnProgress(this, nsnull, nsUint64(offset + count),
-                                  nsUint64(mContentLength));
+        mProgressSink->OnProgress(this, nsnull, PRUint64(offset + count),
+                                  PRUint64(mContentLength));
 
     return rv; // let the pump cancel on failure
 }
