@@ -429,7 +429,6 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
         return rv;
 
     PRBool externalProtocol = PR_FALSE;
-    PRBool listedProtocol   = PR_TRUE;
     nsCOMPtr<nsIPrefBranch2> prefBranch;
     GetPrefBranch(getter_AddRefs(prefBranch));
     if (prefBranch) {
@@ -438,7 +437,6 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
         rv = prefBranch->GetBoolPref(externalProtocolPref.get(), &externalProtocol);
         if (NS_FAILED(rv)) {
             externalProtocol = PR_FALSE;
-            listedProtocol   = PR_FALSE;
         }
     }
 
