@@ -21,6 +21,7 @@
  *  Dan Mills <thunder@mozilla.com>
  *  Anant Narayanan <anant@kix.in>
  *  Philipp von Weitershausen <philipp@weitershausen.de>
+ *  Richard Newman <rnewman@mozilla.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -45,13 +46,16 @@ const Ci = Components.interfaces;
 const Cr = Components.results;
 const Cu = Components.utils;
 
+Cu.import("resource://services-sync/async.js");
 Cu.import("resource://services-sync/constants.js");
 Cu.import("resource://services-sync/ext/Observers.js");
 Cu.import("resource://services-sync/ext/Preferences.js");
 Cu.import("resource://services-sync/log4moz.js");
 Cu.import("resource://services-sync/util.js");
 
-Utils.lazy(this, 'Auth', AuthMgr);
+XPCOMUtils.defineLazyGetter(this, "Auth", function () {
+  return new AuthMgr();
+});
 
 // XXX: the authenticator api will probably need to be changed to support
 // other methods (digest, oauth, etc)
@@ -70,7 +74,7 @@ function BrokenBasicAuthenticator(identity) {
 }
 BrokenBasicAuthenticator.prototype = {
   onRequest: function BasicAuth_onRequest(headers) {
-    headers['Authorization'] = 'Basic ' +
+    headers['authorization'] = 'Basic ' +
       btoa(this._id.username + ':' + this._id.password);
     return headers;
   }
@@ -81,7 +85,7 @@ function BasicAuthenticator(identity) {
 }
 BasicAuthenticator.prototype = {
   onRequest: function onRequest(headers) {
-    headers['Authorization'] = 'Basic ' +
+    headers['authorization'] = 'Basic ' +
       btoa(this._id.username + ':' + this._id.passwordUTF8);
     return headers;
   }
@@ -127,24 +131,43 @@ AuthMgr.prototype = {
  *   function callback(error, result) {...}
  * 
  * 'error' will be null on successful requests. Likewise, result will not be
- * passes (=undefined) when an error occurs. Note that this is independent of
+ * passed (=undefined) when an error occurs. Note that this is independent of
  * the status of the HTTP response.
  */
 function AsyncResource(uri) {
   this._log = Log4Moz.repository.getLogger(this._logName);
   this._log.level =
-    Log4Moz.Level[Utils.prefs.getCharPref("log.logger.network.resources")];
+    Log4Moz.Level[Svc.Prefs.get("log.logger.network.resources")];
   this.uri = uri;
   this._headers = {};
   this._onComplete = Utils.bind2(this, this._onComplete);
 }
 AsyncResource.prototype = {
-  _logName: "Net.Resource",
+  _logName: "Sync.AsyncResource",
 
-  // Wait 5 minutes before killing a request
+  // ** {{{ AsyncResource.serverTime }}} **
+  //
+  // Caches the latest server timestamp (X-Weave-Timestamp header).
+  serverTime: null,
+
+  // The string to use as the base User-Agent in Sync requests.
+  // These strings will look something like
+  // 
+  //   Firefox/4.0 FxSync/1.8.0.20100101.mobile
+  //   
+  // or
+  // 
+  //   Firefox Aurora/5.0a1 FxSync/1.9.0.20110409.desktop
+  //
+  _userAgent:
+    Services.appinfo.name + "/" + Services.appinfo.version +  // Product.
+    " FxSync/" + WEAVE_VERSION + "." +                        // Sync.
+    Services.appinfo.appBuildID + ".",                        // Build.
+
+  // Wait 5 minutes before killing a request.
   ABORT_TIMEOUT: 300000,
 
-  // ** {{{ Resource.authenticator }}} **
+  // ** {{{ AsyncResource.authenticator }}} **
   //
   // Getter and setter for the authenticator module
   // responsible for this particular resource. The authenticator
@@ -160,7 +183,7 @@ AsyncResource.prototype = {
     this._authenticator = value;
   },
 
-  // ** {{{ Resource.headers }}} **
+  // ** {{{ AsyncResource.headers }}} **
   //
   // Headers to be included when making a request for the resource.
   // Note: Header names should be all lower case, there's no explicit
@@ -171,15 +194,11 @@ AsyncResource.prototype = {
   set headers(value) {
     this._headers = value;
   },
-  setHeader: function Res_setHeader() {
-    if (arguments.length % 2)
-      throw "setHeader only accepts arguments in multiples of 2";
-    for (let i = 0; i < arguments.length; i += 2) {
-      this._headers[arguments[i].toLowerCase()] = arguments[i + 1];
-    }
+  setHeader: function Res_setHeader(header, value) {
+    this._headers[header.toLowerCase()] = value;
   },
 
-  // ** {{{ Resource.uri }}} **
+  // ** {{{ AsyncResource.uri }}} **
   //
   // URI representing this resource.
   get uri() {
@@ -192,7 +211,7 @@ AsyncResource.prototype = {
       this._uri = value;
   },
 
-  // ** {{{ Resource.spec }}} **
+  // ** {{{ AsyncResource.spec }}} **
   //
   // Get the string representation of the URI.
   get spec() {
@@ -201,7 +220,7 @@ AsyncResource.prototype = {
     return null;
   },
 
-  // ** {{{ Resource.data }}} **
+  // ** {{{ AsyncResource.data }}} **
   //
   // Get and set the data encapulated in the resource.
   _data: null,
@@ -210,15 +229,16 @@ AsyncResource.prototype = {
     this._data = value;
   },
 
-  // ** {{{ Resource._createRequest }}} **
+  // ** {{{ AsyncResource._createRequest }}} **
   //
   // This method returns a new IO Channel for requests to be made
   // through. It is never called directly, only {{{_doRequest}}} uses it
   // to obtain a request channel.
   //
   _createRequest: function Res__createRequest() {
-    let channel = Svc.IO.newChannel(this.spec, null, null).
-      QueryInterface(Ci.nsIRequest).QueryInterface(Ci.nsIHttpChannel);
+    let channel = Services.io.newChannel(this.spec, null, null)
+                          .QueryInterface(Ci.nsIRequest)
+                          .QueryInterface(Ci.nsIHttpChannel);
 
     // Always validate the cache:
     channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
@@ -227,10 +247,16 @@ AsyncResource.prototype = {
     // Setup a callback to handle bad HTTPS certificates.
     channel.notificationCallbacks = new BadCertListener();
 
-    // Avoid calling the authorizer more than once
+    // Compose a UA string fragment from the various available identifiers.
+    if (Svc.Prefs.get("sendVersionInfo", true)) {
+      let ua = this._userAgent + Svc.Prefs.get("client.type", "desktop");
+      channel.setRequestHeader("user-agent", ua, false);
+    }
+    
+    // Avoid calling the authorizer more than once.
     let headers = this.headers;
     for (let key in headers) {
-      if (key == 'Authorization')
+      if (key == 'authorization')
         this._log.trace("HTTP Header " + key + ": ***** (suppressed)");
       else
         this._log.trace("HTTP Header " + key + ": " + headers[key]);
@@ -242,14 +268,14 @@ AsyncResource.prototype = {
   _onProgress: function Res__onProgress(channel) {},
 
   _doRequest: function _doRequest(action, data, callback) {
+    this._log.trace("In _doRequest.");
     this._callback = callback;
     let channel = this._channel = this._createRequest();
 
     if ("undefined" != typeof(data))
       this._data = data;
 
-    // PUT and POST are trreated differently because
-    // they have payload data.
+    // PUT and POST are treated differently because they have payload data.
     if ("PUT" == action || "POST" == action) {
       // Convert non-string bodies into JSON
       if (this._data.constructor.toString() != String)
@@ -278,6 +304,8 @@ AsyncResource.prototype = {
   },
 
   _onComplete: function _onComplete(error, data) {
+    this._log.trace("In _onComplete. Error is " + error + ".");
+
     if (error) {
       this._callback(error);
       return;
@@ -287,29 +315,54 @@ AsyncResource.prototype = {
     let channel = this._channel;
     let action = channel.requestMethod;
 
-    // Set some default values in-case there's no response header
-    let headers = {};
+    this._log.trace("Channel: " + channel);
+    this._log.trace("Action: "  + action);
+
+    // Process status and success first. This way a problem with headers
+    // doesn't fail to include accurate status information.
     let status = 0;
     let success = false;
+
     try {
-      // Read out the response headers if available
+      status  = channel.responseStatus;
+      success = channel.requestSucceeded;    // HTTP status.
+
+      this._log.trace("Status: " + status);
+      this._log.trace("Success: " + success);
+
+      // Log the status of the request.
+      let mesg = [action, success ? "success" : "fail", status,
+                  channel.URI.spec].join(" ");
+      this._log.debug("mesg: " + mesg);
+
+      if (mesg.length > 200)
+        mesg = mesg.substr(0, 200) + "…";
+      this._log.debug(mesg);
+
+      // Additionally give the full response body when Trace logging.
+      if (this._log.level <= Log4Moz.Level.Trace)
+        this._log.trace(action + " body: " + data);
+
+    } catch(ex) {
+      // Got a response, but an exception occurred during processing.
+      // This shouldn't occur.
+      this._log.warn("Caught unexpected exception " + Utils.exceptionStr(ex) +
+                     " in _onComplete.");
+      this._log.debug(Utils.stackTrace(ex));
+    }
+
+    // Process headers. They can be empty, or the call can otherwise fail, so
+    // put this in its own try block.
+    let headers = {};
+    try {
+      this._log.trace("Processing response headers.");
+
+      // Read out the response headers if available.
       channel.visitResponseHeaders({
         visitHeader: function visitHeader(header, value) {
           headers[header.toLowerCase()] = value;
         }
       });
-      status = channel.responseStatus;
-      success = channel.requestSucceeded;
-
-      // Log the status of the request
-      let mesg = [action, success ? "success" : "fail", status,
-                  channel.URI.spec].join(" ");
-      if (mesg.length > 200)
-        mesg = mesg.substr(0, 200) + "…";
-      this._log.debug(mesg);
-      // Additionally give the full response body when Trace logging
-      if (this._log.level <= Log4Moz.Level.Trace)
-        this._log.trace(action + " body: " + data);
 
       // This is a server-side safety valve to allow slowing down
       // clients without hurting performance.
@@ -320,37 +373,21 @@ AsyncResource.prototype = {
       if (success && headers["x-weave-quota-remaining"])
         Observers.notify("weave:service:quota:remaining",
                          parseInt(headers["x-weave-quota-remaining"], 10));
-    }
-    // Got a response but no header; must be cached (use default values)
-    catch(ex) {
-      this._log.debug(action + " cached: " + status);
+    } catch (ex) {
+      this._log.debug("Caught exception " + Utils.exceptionStr(ex) +
+                      " visiting headers in _onComplete.");
+      this._log.debug(Utils.stackTrace(ex));
     }
 
-    let ret = new String(data);
-    ret.headers = headers;
-    ret.status = status;
+    let ret     = new String(data);
+    ret.status  = status;
     ret.success = success;
+    ret.headers = headers;
 
-    // Make a lazy getter to convert the json response into an object
-    Utils.lazy2(ret, "obj", function() JSON.parse(ret));
-
-    // Notify if we get a 401 to maybe try again with a new uri
-    if (status == 401) {
-      // Create an object to allow observers to decide if we should try again
-      let subject = {
-        newUri: "",
-        resource: this,
-        response: ret
-      }
-      Observers.notify("weave:resource:status:401", subject);
-
-      // Do the same type of request but with the new uri
-      if (subject.newUri != "") {
-        this.uri = subject.newUri;
-        this._doRequest(action, this._data, this._callback);
-        return;
-      }
-    }
+    // Make a lazy getter to convert the json response into an object.
+    // Note that this can cause a parse error to be thrown far away from the
+    // actual fetch, so be warned!
+    XPCOMUtils.defineLazyGetter(ret, "obj", function() JSON.parse(ret));
 
     this._callback(null, ret);
   },
@@ -391,10 +428,7 @@ Resource.prototype = {
 
   __proto__: AsyncResource.prototype,
 
-  // ** {{{ Resource.serverTime }}} **
-  //
-  // Caches the latest server timestamp (X-Weave-Timestamp header).
-  serverTime: null,
+  _logName: "Sync.Resource",
 
   // ** {{{ Resource._request }}} **
   //
@@ -402,7 +436,7 @@ Resource.prototype = {
   // is never called directly, but is used by the high-level
   // {{{get}}}, {{{put}}}, {{{post}}} and {{delete}} methods.
   _request: function Res__request(action, data) {
-    let cb = Utils.makeSyncCallback();
+    let cb = Async.makeSyncCallback();
     function callback(error, ret) {
       if (error)
         cb.throw(error);
@@ -412,7 +446,7 @@ Resource.prototype = {
     // The channel listener might get a failure code
     try {
       this._doRequest(action, data, callback);
-      return Utils.waitForSyncCallback(cb);
+      return Async.waitForSyncCallback(cb);
     } catch(ex) {
       // Combine the channel stack with this request stack.  Need to create
       // a new error object for that.
@@ -436,13 +470,7 @@ Resource.prototype = {
   //
   // Perform an asynchronous HTTP GET for this resource.
   get: function Res_get() {
-    let response = this._request("GET");
-    if (response.status == 0) {
-      // This must be an erroneously cached response. Try again.
-      this._log.debug("Status 0 in Resource.get: retrying once.");
-      response = this._request("GET");
-    }
-    return response;
+    return this._request("GET");
   },
 
   // ** {{{ Resource.put }}} **
@@ -481,36 +509,47 @@ function ChannelListener(onComplete, onProgress, logger, timeout) {
 ChannelListener.prototype = {
 
   onStartRequest: function Channel_onStartRequest(channel) {
+    this._log.trace("onStartRequest called for channel " + channel + ".");
     channel.QueryInterface(Ci.nsIHttpChannel);
 
-    // Save the latest server timestamp when possible
+    // Save the latest server timestamp when possible.
     try {
-      Resource.serverTime = channel.getResponseHeader("X-Weave-Timestamp") - 0;
+      AsyncResource.serverTime = channel.getResponseHeader("X-Weave-Timestamp") - 0;
     }
     catch(ex) {}
 
-    this._log.trace(channel.requestMethod + " " + channel.URI.spec);
+    this._log.trace("onStartRequest: " + channel.requestMethod + " " +
+                    channel.URI.spec);
     this._data = '';
     this.delayAbort();
   },
 
   onStopRequest: function Channel_onStopRequest(channel, context, status) {
-    // Clear the abort timer now that the channel is done
+    // Clear the abort timer now that the channel is done.
     this.abortTimer.clear();
 
+    let success = Components.isSuccessCode(status);
+    let uri = channel && channel.URI && channel.URI.spec || "<unknown>";
+    this._log.trace("Channel for " + channel.requestMethod + " " +
+                    uri + ": isSuccessCode(" + status + ")? " +
+                    success);
+
     if (this._data == '')
       this._data = null;
 
     // Throw the failure code and stop execution.  Use Components.Exception()
     // instead of Error() so the exception is QI-able and can be passed across
     // XPCOM borders while preserving the status code.
-    if (!Components.isSuccessCode(status)) {
+    if (!success) {
       let message = Components.Exception("", status).name;
       let error = Components.Exception(message, status);
       this._onComplete(error);
       return;
     }
 
+    this._log.trace("Channel: flags = " + channel.loadFlags +
+                    ", URI = " + uri +
+                    ", HTTP success? " + channel.requestSucceeded);
     this._onComplete(null, this._data);
   },
 
@@ -543,7 +582,7 @@ ChannelListener.prototype = {
    * Create or push back the abort timer that kills this request
    */
   delayAbort: function delayAbort() {
-    Utils.delay(this.abortRequest, this._timeout, this, "abortTimer");
+    Utils.namedTimer(this.abortRequest, this._timeout, this, "abortTimer");
   },
 
   abortRequest: function abortRequest() {
@@ -580,9 +619,9 @@ BadCertListener.prototype = {
 
   notifyCertProblem: function certProblem(socketInfo, sslStatus, targetHost) {
     // Silently ignore?
-    let log = Log4Moz.repository.getLogger("Service.CertListener");
+    let log = Log4Moz.repository.getLogger("Sync.CertListener");
     log.level =
-      Log4Moz.Level[Utils.prefs.getCharPref("log.logger.network.resources")];
+      Log4Moz.Level[Svc.Prefs.get("log.logger.network.resources")];
     log.debug("Invalid HTTPS certificate encountered, ignoring!");
 
     return true;
