@@ -35,7 +35,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-const EXPORTED_SYMBOLS = ['Utils', 'Svc', 'Str'];
+const EXPORTED_SYMBOLS = ["XPCOMUtils", "Services", "NetUtil", "PlacesUtils",
+                          "FileUtils", "Utils", "Svc", "Str"];
 
 const Cc = Components.classes;
 const Ci = Components.interfaces;
@@ -47,21 +48,12 @@ Cu.import("resource://services-sync/ext/Observers.js");
 Cu.import("resource://services-sync/ext/Preferences.js");
 Cu.import("resource://services-sync/ext/StringBundle.js");
 Cu.import("resource://services-sync/log4moz.js");
-
-let NetUtil;
-try {
-  let ns = {};
-  Cu.import("resource://gre/modules/NetUtil.jsm", ns);
-  NetUtil = ns.NetUtil;
-} catch (ex) {
-  // Firefox 3.5 :(
-}
-
-// Constants for makeSyncCallback, waitForSyncCallback
-const CB_READY = {};
-const CB_COMPLETE = {};
-const CB_FAIL = {};
-
+Cu.import("resource://services-sync/status.js");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/PlacesUtils.jsm");
+Cu.import("resource://gre/modules/NetUtil.jsm");
+Cu.import("resource://gre/modules/FileUtils.jsm");
 
 /*
  * Utility functions
@@ -181,29 +173,6 @@ let Utils = {
     };
   },
 
-  batchSync: function batchSync(service, engineType) {
-    return function batchedSync() {
-      let engine = this;
-      let batchEx = null;
-
-      // Try running sync in batch mode
-      Svc[service].runInBatchMode({
-        runBatched: function wrappedSync() {
-          try {
-            engineType.prototype._sync.call(engine);
-          }
-          catch(ex) {
-            batchEx = ex;
-          }
-        }
-      }, null);
-
-      // Expose the exception if something inside the batch failed
-      if (batchEx!= null)
-        throw batchEx;
-    };
-  },
-
   runInTransaction: function(db, callback, thisObj) {
     let hasTransaction = false;
     try {
@@ -220,62 +189,25 @@ let Utils = {
     }
   },
 
-  createStatement: function createStatement(db, query) {
-    // Gecko 2.0
-    if (db.createAsyncStatement)
-      return db.createAsyncStatement(query);
 
-    // Gecko <2.0
-    return db.createStatement(query);
-  },
+  /*
+   * Partition the input array into an array of arrays. Return a generator.
+   */
+  slices: function slices(arr, sliceSize) {
+    if (!sliceSize || sliceSize <= 0)
+      throw "Invalid slice size.";
 
-  // Prototype for mozIStorageCallback, used in queryAsync below.
-  // This allows us to define the handle* functions just once rather
-  // than on every queryAsync invocation.
-  _storageCallbackPrototype: {
-    results: null,
-    // These are set by queryAsync
-    names: null,
-    syncCb: null,
-
-    handleResult: function handleResult(results) {
-      if (!this.names) {
-        return;
+    if (sliceSize > arr.length) {
+      yield arr;
+    } else {
+      let offset = 0;
+      while (arr.length > offset) {
+        yield arr.slice(offset, offset + sliceSize);
+        offset += sliceSize;
       }
-      if (!this.results) {
-        this.results = [];
-      }
-      let row;
-      while ((row = results.getNextRow()) != null) {
-        let item = {};
-        for each (name in this.names) {
-          item[name] = row.getResultByName(name);
-        }
-        this.results.push(item);
-      }
-    },
-    handleError: function handleError(error) {
-      this.syncCb.throw(error);
-    },
-    handleCompletion: function handleCompletion(reason) {
-      // If we were called with column names but didn't find any results,
-      // the calling code probably still expects an array as a return value.
-      if (this.names && !this.results) {
-        this.results = [];
-      }
-      this.syncCb(this.results);
     }
   },
 
-  queryAsync: function(query, names) {
-    // Synchronously asyncExecute fetching all results by name
-    let storageCallback = {names: names,
-                           syncCb: Utils.makeSyncCallback()};
-    storageCallback.__proto__ = Utils._storageCallbackPrototype;
-    query.executeAsync(storageCallback);
-    return Utils.waitForSyncCallback(storageCallback.syncCb);
-  },
-
   byteArrayToString: function byteArrayToString(bytes) {
     return [String.fromCharCode(byte) for each (byte in bytes)].join("");
   },
@@ -324,32 +256,12 @@ let Utils = {
     window.addEventListener("unload", function() windows[url] = null, false);
   },
 
-  // Returns a nsILocalFile representing a file relative to the
-  // current user's profile directory.  If the argument is a string,
-  // it should be a string with unix-style slashes for directory names
-  // (these slashes are automatically converted to platform-specific
-  // path separators).
-  //
-  // Alternatively, if the argument is an object, it should contain
-  // the following attributes:
-  //
-  //   path: the path to the file, relative to the current user's
-  //   profile dir.
-  //
-  //   autoCreate: whether or not the file should be created if it
-  //   doesn't already exist.
-  getProfileFile: function getProfileFile(arg) {
-    if (typeof arg == "string")
-      arg = {path: arg};
-
-    let pathParts = arg.path.split("/");
-    let file = Svc.Directory.get("ProfD", Ci.nsIFile);
-    file.QueryInterface(Ci.nsILocalFile);
-    for (let i = 0; i < pathParts.length; i++)
-      file.append(pathParts[i]);
-    if (arg.autoCreate && !file.exists())
-      file.create(file.NORMAL_FILE_TYPE, PERMS_FILE);
-    return file;
+  // Returns a nsILocalFile representing a file relative to the current
+  // user's profile directory.  The argument should be a string with
+  // unix-style slashes for directory names (these slashes are automatically
+  // converted to platform-specific path separators).
+  getProfileFile: function getProfileFile(path) {
+    return FileUtils.getFile("ProfD", path.split("/"), true);
   },
 
   /**
@@ -394,55 +306,6 @@ let Utils = {
   isArray: function Utils_isArray(val) val != null && typeof val == "object" &&
     val.constructor.name == "Array",
 
-  // lazy load objects from a constructor on first access.  It will
-  // work with the global object ('this' in the global context).
-  lazy: function Weave_lazy(dest, prop, ctr) {
-    delete dest[prop];
-    dest.__defineGetter__(prop, Utils.lazyCb(dest, prop, ctr));
-  },
-  lazyCb: function Weave_lazyCb(dest, prop, ctr) {
-    return function() {
-      delete dest[prop];
-      dest[prop] = new ctr();
-      return dest[prop];
-    };
-  },
-
-  // like lazy, but rather than new'ing the 3rd arg we use its return value
-  lazy2: function Weave_lazy2(dest, prop, fn) {
-    delete dest[prop];
-    dest.__defineGetter__(prop, Utils.lazyCb2(dest, prop, fn));
-  },
-  lazyCb2: function Weave_lazyCb2(dest, prop, fn) {
-    return function() {
-      delete dest[prop];
-      return dest[prop] = fn();
-    };
-  },
-
-  lazySvc: function Weave_lazySvc(dest, prop, cid, iface) {
-    let getter = function() {
-      delete dest[prop];
-      let svc = null;
-
-      // Use the platform's service if it exists
-      if (cid in Cc && iface in Ci)
-        svc = Cc[cid].getService(Ci[iface]);
-      else {
-        svc = FakeSvc[cid];
-
-        let log = Log4Moz.repository.getLogger("Service.Util");
-        if (svc == null)
-          log.warn("Component " + cid + " doesn't exist on this platform.");
-        else
-          log.debug("Using a fake svc object for " + cid);
-      }
-
-      return dest[prop] = svc;
-    };
-    dest.__defineGetter__(prop, getter);
-  },
-
   lazyStrings: function Weave_lazyStrings(name) {
     let bundle = "chrome://weave/locale/services/" + name + ".properties";
     return function() new StringBundle(bundle);
@@ -554,32 +417,6 @@ let Utils = {
     return ex && ex.indexOf && (ex.indexOf(hmacFail) == 0);
   },
   
-  checkStatus: function Weave_checkStatus(code, msg, ranges) {
-    if (!ranges)
-      ranges = [[200,300]];
-
-    for (let i = 0; i < ranges.length; i++) {
-      var rng = ranges[i];
-      if (typeof(rng) == "object" && code >= rng[0] && code < rng[1])
-        return true;
-      else if (typeof(rng) == "number" && code == rng) {
-        return true;
-      }
-    }
-
-    if (msg) {
-      let log = Log4Moz.repository.getLogger("Service.Util");
-      log.error(msg + " Error code: " + code);
-    }
-
-    return false;
-  },
-
-  ensureStatus: function Weave_ensureStatus(args) {
-    if (!Utils.checkStatus.apply(Utils, arguments))
-      throw 'checkStatus failed';
-  },
-
   /**
    * UTF8-encode a message and hash it with the given hasher. Returns a
    * string containing bytes. The hasher is reset if it's an HMAC hasher.
@@ -1010,9 +847,9 @@ let Utils = {
     if (!URIString)
       return null;
     try {
-      return Svc.IO.newURI(URIString, null, null);
+      return Services.io.newURI(URIString, null, null);
     } catch (e) {
-      let log = Log4Moz.repository.getLogger("Service.Util");
+      let log = Log4Moz.repository.getLogger("Sync.Utils");
       log.debug("Could not create URI: " + Utils.exceptionStr(e));
       return null;
     }
@@ -1024,30 +861,6 @@ let Utils = {
     return url;
   },
 
-  xpath: function Weave_xpath(xmlDoc, xpathString) {
-    let root = xmlDoc.ownerDocument == null ?
-      xmlDoc.documentElement : xmlDoc.ownerDocument.documentElement;
-    let nsResolver = xmlDoc.createNSResolver(root);
-
-    return xmlDoc.evaluate(xpathString, xmlDoc, nsResolver,
-                           Ci.nsIDOMXPathResult.ANY_TYPE, null);
-  },
-
-  getTmp: function Weave_getTmp(name) {
-    let tmp = Svc.Directory.get("ProfD", Ci.nsIFile);
-    tmp.QueryInterface(Ci.nsILocalFile);
-
-    tmp.append("weave");
-    tmp.append("tmp");
-    if (!tmp.exists())
-      tmp.create(tmp.DIRECTORY_TYPE, PERMS_DIRECTORY);
-
-    if (name)
-      tmp.append(name);
-
-    return tmp;
-  },
-
   /**
    * Load a json object from disk
    *
@@ -1069,21 +882,6 @@ let Utils = {
       return;
     }
 
-    // Gecko < 2.0
-    if (!NetUtil || !NetUtil.newChannel) {
-      let json;
-      try {
-        let [is] = Utils.open(file, "<");
-        json = JSON.parse(Utils.readStream(is));
-        is.close();
-      } catch (ex) {
-        if (that._log)
-          that._log.debug("Failed to load json: " + Utils.exceptionStr(ex));
-      }
-      callback.call(that, json);
-      return;
-    }
-
     let channel = NetUtil.newChannel(file);
     channel.contentType = "application/json";
 
@@ -1123,24 +921,11 @@ let Utils = {
     if (that._log)
       that._log.trace("Saving json to disk: " + filePath);
 
-    let file = Utils.getProfileFile({ autoCreate: true, path: filePath });
+    let file = Utils.getProfileFile(filePath);
     let json = typeof obj == "function" ? obj.call(that) : obj;
     let out = JSON.stringify(json);
 
-    // Firefox 3.5
-    if (!NetUtil) {
-      let [fos] = Utils.open(file, ">");
-      fos.writeString(out);
-      fos.close();
-      if (typeof callback == "function") {
-        callback.call(that);
-      }
-      return;
-    }
-
-    let fos = Cc["@mozilla.org/network/safe-file-output-stream;1"]
-                .createInstance(Ci.nsIFileOutputStream);
-    fos.init(file, MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE, PERMS_FILE, 0);
+    let fos = FileUtils.openSafeFileOutputStream(file);
     let is = this._utf8Converter.convertToInputStream(out);
     NetUtil.asyncCopy(is, fos, function (result) {
       if (typeof callback == "function") {
@@ -1149,17 +934,30 @@ let Utils = {
     });
   },
 
+  /**
+   * Execute a function on the next event loop tick.
+   * 
+   * @param callback
+   *        Function to invoke.
+   * @param thisObj [optional]
+   *        Object to bind the callback to.
+   */
+  nextTick: function nextTick(callback, thisObj) {
+    if (thisObj) {
+      callback = callback.bind(thisObj);
+    }
+    Services.tm.currentThread.dispatch(callback, Ci.nsIThread.DISPATCH_NORMAL);
+  },
+
   /**
    * Return a timer that is scheduled to call the callback after waiting the
    * provided time or as soon as possible. The timer will be set as a property
    * of the provided object with the given timer name.
    */
-  delay: function delay(callback, wait, thisObj, name) {
-    // Default to running right away
-    wait = wait || 0;
-
-    // Use a dummy object if one wasn't provided
-    thisObj = thisObj || {};
+  namedTimer: function delay(callback, wait, thisObj, name) {
+    if (!thisObj || !name) {
+      throw "You must provide both an object and a property name for the timer!";
+    }
 
     // Delay an existing timer if it exists
     if (name in thisObj && thisObj[name] instanceof Ci.nsITimer) {
@@ -1189,68 +987,15 @@ let Utils = {
     return thisObj[name] = timer;
   },
 
-  // Gecko <2.0
-  open: function open(pathOrFile, mode, perms) {
-    let stream, file;
-
-    if (pathOrFile instanceof Ci.nsIFile) {
-      file = pathOrFile;
-    } else {
-      file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
-      dump("PATH IS" + pathOrFile + "\n");
-      file.initWithPath(pathOrFile);
-    }
-
-    if (!perms)
-      perms = PERMS_FILE;
-
-    switch(mode) {
-    case "<": {
-      if (!file.exists())
-        throw "Cannot open file for reading, file does not exist";
-      let fis = Cc["@mozilla.org/network/file-input-stream;1"].
-        createInstance(Ci.nsIFileInputStream);
-      fis.init(file, MODE_RDONLY, perms, 0);
-      stream = Cc["@mozilla.org/intl/converter-input-stream;1"].
-        createInstance(Ci.nsIConverterInputStream);
-      stream.init(fis, "UTF-8", 4096,
-                  Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
-    } break;
-
-    case ">": {
-      let fos = Cc["@mozilla.org/network/file-output-stream;1"].
-        createInstance(Ci.nsIFileOutputStream);
-      fos.init(file, MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE, perms, 0);
-      stream = Cc["@mozilla.org/intl/converter-output-stream;1"]
-        .createInstance(Ci.nsIConverterOutputStream);
-      stream.init(fos, "UTF-8", 4096, 0x0000);
-    } break;
-
-    case ">>": {
-      let fos = Cc["@mozilla.org/network/file-output-stream;1"].
-        createInstance(Ci.nsIFileOutputStream);
-      fos.init(file, MODE_WRONLY | MODE_CREATE | MODE_APPEND, perms, 0);
-      stream = Cc["@mozilla.org/intl/converter-output-stream;1"]
-        .createInstance(Ci.nsIConverterOutputStream);
-      stream.init(fos, "UTF-8", 4096, 0x0000);
-    } break;
-
-    default:
-      throw "Illegal mode to open(): " + mode;
-    }
-
-    return [stream, file];
-  },
-
   getIcon: function(iconUri, defaultIcon) {
     try {
       let iconURI = Utils.makeURI(iconUri);
-      return Svc.Favicon.getFaviconLinkForIcon(iconURI).spec;
+      return PlacesUtils.favicons.getFaviconLinkForIcon(iconURI).spec;
     }
     catch(ex) {}
 
     // Just give the provided default icon or the system's default
-    return defaultIcon || Svc.Favicon.defaultFavicon.spec;
+    return defaultIcon || PlacesUtils.favicons.defaultFavicon.spec;
   },
 
   getErrorString: function Utils_getErrorString(error, args) {
@@ -1262,16 +1007,6 @@ let Utils = {
     return Str.errors.get("error.reason.unknown");
   },
 
-  // Gecko <2.0
-  // assumes an nsIConverterInputStream
-  readStream: function Weave_readStream(is) {
-    let ret = "", str = {};
-    while (is.readString(4096, str) != 0) {
-      ret += str.value;
-    }
-    return ret;
-  },
-
   encodeUTF8: function(str) {
     try {
       str = this._utf8Converter.ConvertFromUnicode(str);
@@ -1399,49 +1134,24 @@ let Utils = {
     return over ? atob(b64.substr(0, len - over)) : atob(b64);
   },
 
-  /*
-   * Calculate the strength of a passphrase provided by the user
-   * according to the NIST algorithm (NIST 800-63 Appendix A.1).
-   */
-  passphraseStrength: function passphraseStrength(value) {
-    let bits = 0;
-
-    // The entropy of the first character is taken to be 4 bits.
-    if (value.length)
-      bits = 4;
-
-    // The entropy of the next 7 characters are 2 bits per character.
-    if (value.length > 1)
-      bits += Math.min(value.length - 1, 7) * 2;
-
-    // For the 9th through the 20th character the entropy is taken to
-    // be 1.5 bits per character.
-    if (value.length > 8)
-      bits += Math.min(value.length - 8, 12) * 1.5;
-
-    // For characters 21 and above the entropy is taken to be 1 bit per character.
-    if (value.length > 20)
-      bits += value.length - 20;
-
-    // Bonus of 6 bits if we find non-alphabetic characters
-    if ([char.charCodeAt() for each (char in value.toLowerCase())]
-        .some(function(chr) chr < 97 || chr > 122))
-      bits += 6;
-      
-    return bits;
-  },
-
   /**
-   * Create an array like the first but without elements of the second
+   * Create an array like the first but without elements of the second. Reuse
+   * arrays if possible.
    */
   arraySub: function arraySub(minuend, subtrahend) {
+    if (!minuend.length || !subtrahend.length)
+      return minuend;
     return minuend.filter(function(i) subtrahend.indexOf(i) == -1);
   },
 
   /**
-   * Build the union of two arrays.
+   * Build the union of two arrays. Reuse arrays if possible.
    */
   arrayUnion: function arrayUnion(foo, bar) {
+    if (!foo.length)
+      return bar;
+    if (!bar.length)
+      return foo;
     return foo.concat(Utils.arraySub(bar, foo));
   },
 
@@ -1479,23 +1189,6 @@ let Utils = {
     return false;
   },
   
-  __prefs: null,
-  get prefs() {
-    if (!this.__prefs) {
-      this.__prefs = Cc["@mozilla.org/preferences-service;1"]
-        .getService(Ci.nsIPrefService);
-      this.__prefs = this.__prefs.getBranch(PREFS_BRANCH);
-      this.__prefs.QueryInterface(Ci.nsIPrefBranch2);
-    }
-    return this.__prefs;
-  },
-
-  /**
-   * Helpers for making asynchronous calls within a synchronous API possible.
-   * 
-   * If you value your sanity, do not look closely at the following functions.
-   */
-
   /**
    * Check if the app is ready (not quitting)
    */
@@ -1511,120 +1204,19 @@ let Utils = {
   },
 
   /**
-   * Create a sync callback that remembers state like whether it's been called
+   * Return a value for a backoff interval.  Maximum is eight hours, unless
+   * Status.backoffInterval is higher.
+   *
    */
-  makeSyncCallback: function makeSyncCallback() {
-    // The main callback remembers the value it's passed and that it got data
-    let onComplete = function onComplete(data) {
-      onComplete.state = CB_COMPLETE;
-      onComplete.value = data;
-    };
-
-    // Initialize private callback data to prepare to be called
-    onComplete.state = CB_READY;
-    onComplete.value = null;
-
-    // Allow an alternate callback to trigger an exception to be thrown
-    onComplete.throw = function onComplete_throw(data) {
-      onComplete.state = CB_FAIL;
-      onComplete.value = data;
-
-      // Cause the caller to get an exception and stop execution
-      throw data;
-    };
-
-    return onComplete;
-  },
-
-  /**
-   * Wait for a sync callback to finish
-   */
-  waitForSyncCallback: function waitForSyncCallback(callback) {
-    // Grab the current thread so we can make it give up priority
-    let thread = Cc["@mozilla.org/thread-manager;1"].getService().currentThread;
-
-    // Keep waiting until our callback is triggered unless the app is quitting
-    while (Utils.checkAppReady() && callback.state == CB_READY) {
-      thread.processNextEvent(true);
-    }
-
-    // Reset the state of the callback to prepare for another call
-    let state = callback.state;
-    callback.state = CB_READY;
-
-    // Throw the value the callback decided to fail with
-    if (state == CB_FAIL) {
-      throw callback.value;
-    }
-
-    // Return the value passed to the callback
-    return callback.value;
+  calculateBackoff: function calculateBackoff(attempts, base_interval) {
+    let backoffInterval = attempts *
+                          (Math.floor(Math.random() * base_interval) +
+                           base_interval);
+    return Math.max(Math.min(backoffInterval, MAXIMUM_BACKOFF_INTERVAL), Status.backoffInterval);
   }
 };
 
-let FakeSvc = {
-  // Private Browsing
-  "@mozilla.org/privatebrowsing;1": {
-    autoStarted: false,
-    privateBrowsingEnabled: false
-  },
-  // Session Restore
-  "@mozilla.org/browser/sessionstore;1": {
-    setTabValue: function(tab, key, value) {
-      if (!tab.__SS_extdata)
-        tab.__SS_extdata = {};
-      tab.__SS_extData[key] = value;
-    },
-    getBrowserState: function() {
-      // Fennec should have only one window. Not more, not less.
-      let state = { windows: [{ tabs: [] }] };
-      let window = Svc.WinMediator.getMostRecentWindow("navigator:browser");
-
-      // Extract various pieces of tab data
-      window.Browser._tabs.forEach(function(tab) {
-        let tabState = { entries: [{}] };
-        let browser = tab.browser;
-
-        // Cases when we want to skip the tab. Could come up if we get
-        // state as a tab is opening or closing.
-        if (!browser || !browser.currentURI || !browser.sessionHistory)
-          return;
-
-        let history = browser.sessionHistory;
-        if (history.count > 0) {
-          // We're only grabbing the current history entry for now.
-          let entry = history.getEntryAtIndex(history.index, false);
-          tabState.entries[0].url = entry.URI.spec;
-          // Like SessionStore really does it...
-          if (entry.title && entry.title != entry.url)
-            tabState.entries[0].title = entry.title;
-        }
-        // index is 1-based
-        tabState.index = 1;
-
-        // Get the image for the tab. Fennec doesn't quite work the same
-        // way as Firefox, so we'll just get this from the browser object.
-        tabState.attributes = { image: browser.mIconURL };
-
-        // Collect the extdata
-        if (tab.__SS_extdata) {
-          tabState.extData = {};
-          for (let key in tab.__SS_extdata)
-            tabState.extData[key] = tab.__SS_extdata[key];
-        }
-
-        // Add the tab to the window
-        state.windows[0].tabs.push(tabState);
-      });
-      return JSON.stringify(state);
-    }
-  },
-  // A fake service only used for testing
-  "@labs.mozilla.com/Fake/Thing;1": {
-    isFake: true
-  }
-};
-Utils.lazy2(Utils, "_utf8Converter", function() {
+XPCOMUtils.defineLazyGetter(Utils, "_utf8Converter", function() {
   let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
                     .createInstance(Ci.nsIScriptableUnicodeConverter);
   converter.charset = "UTF-8";
@@ -1639,54 +1231,41 @@ Svc.Prefs = new Preferences(PREFS_BRANCH);
 Svc.DefaultPrefs = new Preferences({branch: PREFS_BRANCH, defaultBranch: true});
 Svc.Obs = Observers;
 
-this.__defineGetter__("_sessionCID", function() {
-  //sets session CID based on browser type
-  let appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
-  return appInfo.ID == SEAMONKEY_ID ? "@mozilla.org/suite/sessionstore;1"
-                                    : "@mozilla.org/browser/sessionstore;1";
-});
-[["Annos", "@mozilla.org/browser/annotation-service;1", "nsIAnnotationService"],
- ["AppInfo", "@mozilla.org/xre/app-info;1", "nsIXULAppInfo"],
- ["Bookmark", "@mozilla.org/browser/nav-bookmarks-service;1", "nsINavBookmarksService"],
- ["Directory", "@mozilla.org/file/directory_service;1", "nsIProperties"],
- ["Env", "@mozilla.org/process/environment;1", "nsIEnvironment"],
- ["Favicon", "@mozilla.org/browser/favicon-service;1", "nsIFaviconService"],
- ["Form", "@mozilla.org/satchel/form-history;1", "nsIFormHistory2"],
- ["History", "@mozilla.org/browser/nav-history-service;1", "nsPIPlacesDatabase"],
+let _sessionCID = Services.appinfo.ID == SEAMONKEY_ID ?
+  "@mozilla.org/suite/sessionstore;1" :
+  "@mozilla.org/browser/sessionstore;1";
+
+[["Form", "@mozilla.org/satchel/form-history;1", "nsIFormHistory2"],
  ["Idle", "@mozilla.org/widget/idleservice;1", "nsIIdleService"],
- ["IO", "@mozilla.org/network/io-service;1", "nsIIOService"],
  ["KeyFactory", "@mozilla.org/security/keyobjectfactory;1", "nsIKeyObjectFactory"],
- ["Login", "@mozilla.org/login-manager;1", "nsILoginManager"],
- ["Memory", "@mozilla.org/xpcom/memory-service;1", "nsIMemory"],
- ["Private", "@mozilla.org/privatebrowsing;1", "nsIPrivateBrowsingService"],
- ["Profiles", "@mozilla.org/toolkit/profile-service;1", "nsIToolkitProfileService"],
- ["Prompt", "@mozilla.org/embedcomp/prompt-service;1", "nsIPromptService"],
- ["Script", "@mozilla.org/moz/jssubscript-loader;1", "mozIJSSubScriptLoader"],
- ["SysInfo", "@mozilla.org/system-info;1", "nsIPropertyBag2"],
- ["Version", "@mozilla.org/xpcom/version-comparator;1", "nsIVersionComparator"],
- ["WinMediator", "@mozilla.org/appshell/window-mediator;1", "nsIWindowMediator"],
- ["WinWatcher", "@mozilla.org/embedcomp/window-watcher;1", "nsIWindowWatcher"],
- ["Session", this._sessionCID, "nsISessionStore"],
-].forEach(function(lazy) Utils.lazySvc(Svc, lazy[0], lazy[1], lazy[2]));
+ ["Session", _sessionCID, "nsISessionStore"]
+].forEach(function([name, contract, iface]) {
+  XPCOMUtils.defineLazyServiceGetter(Svc, name, contract, iface);
+});
+
+// nsIPrivateBrowsingService is not implemented in mobile Firefox.
+// Svc.Private should just return undefined in this case instead of throwing.
+XPCOMUtils.defineLazyGetter(Svc, "Private", function() {
+  try {
+    return Cc["@mozilla.org/privatebrowsing;1"].getService(Ci["nsIPrivateBrowsingService"]);
+  } catch (e) {
+    return undefined;
+  }
+});
 
 Svc.__defineGetter__("Crypto", function() {
   let cryptoSvc;
-  try {
-    let ns = {};
-    Cu.import("resource://services-crypto/WeaveCrypto.js", ns);
-    cryptoSvc = new ns.WeaveCrypto();
-  } catch (ex) {
-    // Fallback to binary WeaveCrypto
-    cryptoSvc = Cc["@labs.mozilla.com/Weave/Crypto;1"].
-                getService(Ci.IWeaveCrypto);
-  }
+  let ns = {};
+  Cu.import("resource://services-crypto/WeaveCrypto.js", ns);
+  cryptoSvc = new ns.WeaveCrypto();
   delete Svc.Crypto;
   return Svc.Crypto = cryptoSvc;
 });
 
 let Str = {};
-["errors", "sync"]
-  .forEach(function(lazy) Utils.lazy2(Str, lazy, Utils.lazyStrings(lazy)));
+["errors", "sync"].forEach(function(lazy) {
+  XPCOMUtils.defineLazyGetter(Str, lazy, Utils.lazyStrings(lazy));
+});
 
 Svc.Obs.add("xpcom-shutdown", function () {
   for (let name in Svc)
