@@ -18,16 +18,24 @@ function test() {
 function download_progress(addon, value, maxValue) {
   try {
     Services.io.manageOfflineStatus = false;
-    Services.prefs.setBoolPref("browser.offline", true);
     Services.io.offline = true;
   } catch (ex) {
   }
 }
 
 function finish_test(count) {
+  function wait_for_online() {
+    info("Checking if the browser is still offline...");
+    var request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
+                  createInstance(Ci.nsIXMLHttpRequest);
+    request.open("GET", TESTROOT + "empty.xpi", true);
+    request.onerror = wait_for_online;
+    request.onload = Harness.finish;
+    request.send(null);
+  }
+
   is(count, 0, "No add-ons should have been installed");
   try {
-    Services.prefs.setBoolPref("browser.offline", false);
     Services.io.offline = false;
   } catch (ex) {
   }
@@ -35,5 +43,5 @@ function finish_test(count) {
   Services.perms.remove("example.com", "install");
 
   gBrowser.removeCurrentTab();
-  Harness.finish();
+  wait_for_online();
 }
