@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Stan Shebs <shebs@mozilla.com>
+ *   Thomas K. Dyas <tom.dyas@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -47,13 +48,9 @@
 #import <Carbon/Carbon.h>
 
 #include "nsCOMPtr.h"
-#include "nsIBaseWindow.h"
 #include "nsINativeAppSupport.h"
-#include "nsIWidget.h"
-#include "nsIWindowMediator.h"
 #include "nsAppRunner.h"
 #include "nsComponentManagerUtils.h"
-#include "nsCommandLineServiceMac.h"
 #include "nsIServiceManager.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIAppStartup.h"
@@ -63,6 +60,11 @@
 #include "nsIFile.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsICommandLineRunner.h"
+#include "nsIMacDockSupport.h"
+#include "nsIStandaloneNativeMenu.h"
+#include "nsILocalFileMac.h"
+#include "nsString.h"
+#include "nsCommandLineServiceMac.h"
 
 class AutoAutoreleasePool {
 public:
@@ -84,7 +86,11 @@ private:
 
 @end
 
-// Something to call from non-objective code.
+static PRBool sProcessedGetURLEvent = PR_FALSE;
+
+@class GeckoNSApplication;
+
+// Methods that can be called from non-Objective-C code.
 
 // This is needed, on relaunch, to force the OS to use the "Cocoa Dock API"
 // instead of the "Carbon Dock API".  For more info see bmo bug 377166.
@@ -93,7 +99,7 @@ EnsureUseCocoaDockAPI()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
-  [NSApplication sharedApplication];
+  [GeckoNSApplication sharedApplication];
 
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
@@ -107,6 +113,9 @@ SetupMacApplicationDelegate()
   // needs an autorelease pool to avoid cocoa object leakage (bug 559075)
   AutoAutoreleasePool pool;
 
+  // Ensure that ProcessPendingGetURLAppleEvents() doesn't regress bug 377166.
+  [GeckoNSApplication sharedApplication];
+
   // This call makes it so that application:openFile: doesn't get bogus calls
   // from Cocoa doing its own parsing of the argument string. And yes, we need
   // to use a string with a boolean value in it. That's just how it works.
@@ -115,11 +124,35 @@ SetupMacApplicationDelegate()
 
   // Create the delegate. This should be around for the lifetime of the app.
   MacApplicationDelegate *delegate = [[MacApplicationDelegate alloc] init];
-  [[NSApplication sharedApplication] setDelegate:delegate];
+  [NSApp setDelegate:delegate];
 
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
+// Indirectly make the OS process any pending GetURL Apple events.  This is
+// done via _DPSNextEvent() (an undocumented AppKit function called from
+// [NSApplication nextEventMatchingMask:untilDate:inMode:dequeue:]).  Apple
+// events are only processed if 'dequeue' is 'YES' -- so we need to call
+// [NSApplication sendEvent:] on any event that gets returned.  'event' will
+// never itself be an Apple event, and it may be 'nil' even when Apple events
+// are processed.
+void
+ProcessPendingGetURLAppleEvents()
+{
+  AutoAutoreleasePool pool;
+  PRBool keepSpinning = PR_TRUE;
+  while (keepSpinning) {
+    sProcessedGetURLEvent = PR_FALSE;
+    NSEvent *event = [NSApp nextEventMatchingMask:NSAnyEventMask
+                                        untilDate:nil
+                                           inMode:NSDefaultRunLoopMode
+                                          dequeue:YES];
+    if (event)
+      [NSApp sendEvent:event];
+    keepSpinning = sProcessedGetURLEvent;
+  }
+}
+
 @implementation MacApplicationDelegate
 
 - (id)init
@@ -143,6 +176,14 @@ SetupMacApplicationDelegate()
                andSelector:@selector(handleAppleEvent:withReplyEvent:)
              forEventClass:kCoreEventClass
                 andEventID:kAEOpenDocuments];
+
+    if (![NSApp windowsMenu]) {
+      // If the application has a windows menu, it will keep it up to date and
+      // prepend the window list to the Dock menu automatically.
+      NSMenu* windowsMenu = [[NSMenu alloc] initWithTitle:@"Window"];
+      [NSApp setWindowsMenu:windowsMenu];
+      [windowsMenu release];
+    }
   }
   return self;
 
@@ -162,14 +203,9 @@ SetupMacApplicationDelegate()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-// Opening the application is handled specially elsewhere,
-// don't define applicationOpenUntitledFile: .
-
 // The method that NSApplication calls upon a request to reopen, such as when
-// the Dock icon is clicked and no windows are open.
-
-// A "visible" window may be miniaturized, so we can't skip
-// nsCocoaNativeReOpen() if 'flag' is 'true'.
+// the Dock icon is clicked and no windows are open. A "visible" window may be
+// miniaturized, so we can't skip nsCocoaNativeReOpen() if 'flag' is 'true'.
 - (BOOL)applicationShouldHandleReopen:(NSApplication*)theApp hasVisibleWindows:(BOOL)flag
 {
   nsCOMPtr<nsINativeAppSupport> nas = do_CreateInstance(NS_NATIVEAPPSUPPORT_CONTRACTID);
@@ -185,7 +221,6 @@ SetupMacApplicationDelegate()
 
 // The method that NSApplication calls when documents are requested to be opened.
 // It will be called once for each selected document.
-
 - (BOOL)application:(NSApplication*)theApplication openFile:(NSString*)filename
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
@@ -198,17 +233,40 @@ SetupMacApplicationDelegate()
   if (!urlString)
     return NO;
 
-  nsMacCommandLine& cmdLine = nsMacCommandLine::GetMacCommandLine();
-
   // Add the URL to any command line we're currently setting up.
-  if (cmdLine.AddURLToCurrentCommandLine([urlString UTF8String]))
+  if (CommandLineServiceMac::AddURLToCurrentCommandLine([urlString UTF8String]))
     return YES;
 
-  // We don't actually care about Mac filetypes in this context, just pass a placeholder.
-  
-  cmdLine.HandleOpenOneDoc((CFURLRef)url, 'abcd');
+  nsCOMPtr<nsILocalFileMac> inFile;
+  nsresult rv = NS_NewLocalFileWithCFURL((CFURLRef)url, PR_TRUE, getter_AddRefs(inFile));
+  if (NS_FAILED(rv))
+    return NO;
 
-  return YES;
+  nsCOMPtr<nsICommandLineRunner> cmdLine(do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+  if (!cmdLine) {
+    NS_ERROR("Couldn't create command line!");
+    return NO;
+  }
+
+  nsCString filePath;
+  rv = inFile->GetNativePath(filePath);
+  if (NS_FAILED(rv))
+    return NO;
+
+  nsCOMPtr<nsIFile> workingDir;
+  rv = NS_GetSpecialDirectory(NS_OS_CURRENT_WORKING_DIR, getter_AddRefs(workingDir));
+  if (NS_FAILED(rv))
+    return NO;
+
+  const char *argv[3] = {nsnull, "-file", filePath.get()};
+  rv = cmdLine->Init(3, const_cast<char**>(argv), workingDir, nsICommandLine::STATE_REMOTE_EXPLICIT);
+  if (NS_FAILED(rv))
+    return NO;
+
+  if (NS_SUCCEEDED(cmdLine->Run()))
+    return YES;
+
+  return NO;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(NO);
 }
@@ -216,118 +274,64 @@ SetupMacApplicationDelegate()
 // The method that NSApplication calls when documents are requested to be printed
 // from the Finder (under the "File" menu).
 // It will be called once for each selected document.
-
 - (BOOL)application:(NSApplication*)theApplication printFile:(NSString*)filename
 {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
-
-  // Take advantage of the existing "command line" code for Macs.
-  nsMacCommandLine& cmdLine = nsMacCommandLine::GetMacCommandLine();
-  // We don't actually care about Mac filetypes in this context, just pass a placeholder.
-  cmdLine.HandlePrintOneDoc((CFURLRef)[NSURL URLWithString:filename], 'abcd');
-
-  return YES;
-
-  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(NO);
-}
-
-// Drill down from nsIXULWindow and get an NSWindow. We get passed nsISupports
-// because that's what nsISimpleEnumerator returns.
-
-static NSWindow* GetCocoaWindowForXULWindow(nsISupports *aXULWindow)
-{
-  nsresult rv;
-  nsCOMPtr<nsIBaseWindow> baseWindow = do_QueryInterface(aXULWindow, &rv);
-  NS_ENSURE_SUCCESS(rv, nil);
-  nsCOMPtr<nsIWidget> widget;
-  rv = baseWindow->GetMainWidget(getter_AddRefs(widget));
-  NS_ENSURE_SUCCESS(rv, nil);
-  // If it fails, we return nil anyway, no biggie
-  return (NSWindow *)widget->GetNativeData(NS_NATIVE_WINDOW);
+  return NO;
 }
 
 // Create the menu that shows up in the Dock.
-
 - (NSMenu*)applicationDockMenu:(NSApplication*)sender
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
 
-  // Why we're not just using Cocoa to enumerate our windows:
-  // The Dock thinks we're a Carbon app, probably because we don't have a
-  // blessed Window menu, so we get none of the automatic handling for dock
-  // menus that Cocoa apps get. Add in Cocoa being a bit braindead when you hide
-  // the app, and we end up having to get our list of windows via XPCOM. Ugh.
-
-  // Get the window mediator to do all our lookups.
-  nsresult rv;
-  nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, nil);
-
-  // Get the frontmost window
-  nsCOMPtr<nsISimpleEnumerator> orderedWindowList;
-  rv = wm->GetZOrderXULWindowEnumerator(nsnull, PR_TRUE,
-                                        getter_AddRefs(orderedWindowList));
-  NS_ENSURE_SUCCESS(rv, nil);
-  PRBool anyWindows = false;
-  rv = orderedWindowList->HasMoreElements(&anyWindows);
-  NS_ENSURE_SUCCESS(rv, nil);
-  nsCOMPtr<nsISupports> frontWindow;
-  rv = orderedWindowList->GetNext(getter_AddRefs(frontWindow));
-  NS_ENSURE_SUCCESS(rv, nil);
-
-  // Get our list of windows and prepare to iterate. We use this list, ordered
-  // by window creation date, instead of the z-ordered list because that's what
-  // native apps do.
-  nsCOMPtr<nsISimpleEnumerator> windowList;
-  rv = wm->GetXULWindowEnumerator(nsnull, getter_AddRefs(windowList));
-  NS_ENSURE_SUCCESS(rv, nil);
-
-  // Iterate through our list of windows to create our menu
+  // Create the NSMenu that will contain the dock menu items.
   NSMenu *menu = [[[NSMenu alloc] initWithTitle:@""] autorelease];
-  PRBool more;
-  while (NS_SUCCEEDED(windowList->HasMoreElements(&more)) && more) {
-    // Get our native window
-    nsCOMPtr<nsISupports> xulWindow;
-    rv = windowList->GetNext(getter_AddRefs(xulWindow));
-    NS_ENSURE_SUCCESS(rv, nil);
-    NSWindow *cocoaWindow = GetCocoaWindowForXULWindow(xulWindow);
-    if (!cocoaWindow) continue;
-    
-    NSString *windowTitle = [cocoaWindow title];
-    if (!windowTitle) continue;
-    
-    // Now, create a menu item, and add it to the menu
-    NSMenuItem *menuItem = [[NSMenuItem alloc]
-                              initWithTitle:windowTitle
-                                     action:@selector(dockMenuItemSelected:)
-                              keyEquivalent:@""];
-    [menuItem setTarget:self];
-    [menuItem setRepresentedObject:cocoaWindow];
+  [menu setAutoenablesItems:NO];
 
-    // If this is the foreground window, put a checkmark next to it
-    if (SameCOMIdentity(xulWindow, frontWindow))
-      [menuItem setState:NSOnState];
+  // Add application-specific dock menu items. On error, do not insert the
+  // dock menu items.
+  nsresult rv;
+  nsCOMPtr<nsIMacDockSupport> dockSupport = do_GetService("@mozilla.org/widget/macdocksupport;1", &rv);
+  if (NS_FAILED(rv) || !dockSupport)
+    return menu;
 
-    [menu addItem:menuItem];
-    [menuItem release];
+  nsCOMPtr<nsIStandaloneNativeMenu> dockMenu;
+  rv = dockSupport->GetDockMenu(getter_AddRefs(dockMenu));
+  if (NS_FAILED(rv) || !dockMenu)
+    return menu;
+
+  // Determine if the dock menu items should be displayed. This also gives
+  // the menu the opportunity to update itself before display.
+  PRBool shouldShowItems;
+  rv = dockMenu->MenuWillOpen(&shouldShowItems);
+  if (NS_FAILED(rv) || !shouldShowItems)
+    return menu;
+
+  // Obtain a copy of the native menu.
+  NSMenu * nativeDockMenu;
+  rv = dockMenu->GetNativeMenu(reinterpret_cast<void **>(&nativeDockMenu));
+  if (NS_FAILED(rv) || !nativeDockMenu)
+    return menu;
+
+  // Loop through the application-specific dock menu and insert its
+  // contents into the dock menu that we are building for Cocoa.
+  int numDockMenuItems = [nativeDockMenu numberOfItems];
+  if (numDockMenuItems > 0) {
+    if ([menu numberOfItems] > 0)
+      [menu addItem:[NSMenuItem separatorItem]];
+
+    for (int i = 0; i < numDockMenuItems; i++) {
+      NSMenuItem * itemCopy = [[nativeDockMenu itemAtIndex:i] copy];
+      [menu addItem:itemCopy];
+      [itemCopy release];
+    }
   }
+
   return menu;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
 
-// One of our dock menu items was selected
-- (void)dockMenuItemSelected:(id)sender
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
-
-  // Our represented object is an NSWindow
-  [[sender representedObject] makeKeyAndOrderFront:nil];
-  [NSApp activateIgnoringOtherApps:YES];
-
-  NS_OBJC_END_TRY_ABORT_BLOCK;
-}
-
 // If we don't handle applicationShouldTerminate:, a call to [NSApp terminate:]
 // (from the browser or from the OS) can result in an unclean shutdown.
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
@@ -365,7 +369,12 @@ static NSWindow* GetCocoaWindowForXULWindow(nsISupports *aXULWindow)
 
   AutoAutoreleasePool pool;
 
-  if (([event eventClass] == kInternetEventClass && [event eventID] == kAEGetURL) ||
+  PRBool isGetURLEvent =
+    ([event eventClass] == kInternetEventClass && [event eventID] == kAEGetURL);
+  if (isGetURLEvent)
+    sProcessedGetURLEvent = PR_TRUE;
+
+  if (isGetURLEvent ||
       ([event eventClass] == 'WWW!' && [event eventID] == 'OURL')) {
     NSString* urlString = [[event paramDescriptorForKeyword:keyDirectObject] stringValue];
 
@@ -379,8 +388,7 @@ static NSWindow* GetCocoaWindowForXULWindow(nsISupports *aXULWindow)
     }
 
     // Add the URL to any command line we're currently setting up.
-    nsMacCommandLine& macCmdLine = nsMacCommandLine::GetMacCommandLine();
-    if (macCmdLine.AddURLToCurrentCommandLine([urlString UTF8String]))
+    if (CommandLineServiceMac::AddURLToCurrentCommandLine([urlString UTF8String]))
       return;
 
     nsCOMPtr<nsICommandLineRunner> cmdLine(do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
@@ -404,8 +412,8 @@ static NSWindow* GetCocoaWindowForXULWindow(nsISupports *aXULWindow)
       return;
 
     // Descriptor list indexing is one-based...
-    int numberOfFiles = [fileListDescriptor numberOfItems];
-    for (int i = 1; i <= numberOfFiles; i++) {
+    NSInteger numberOfFiles = [fileListDescriptor numberOfItems];
+    for (NSInteger i = 1; i <= numberOfFiles; i++) {
       NSString* urlString = [[fileListDescriptor descriptorAtIndex:i] stringValue];
       if (!urlString)
         continue;
