@@ -102,9 +102,8 @@ using namespace QtMobility;
 #include "nsWidgetsCID.h"
 #include "nsQtKeyUtils.h"
 #include "mozilla/Services.h"
+#include "mozilla/Preferences.h"
 
-#include "nsIPrefService.h"
-#include "nsIPrefBranch.h"
 #include "nsIStringBundle.h"
 #include "nsGfxCIID.h"
 
@@ -310,8 +309,8 @@ UpdateOffScreenBuffers(int aDepth, QSize aSize, QWidget* aWidget = nsnull)
             return true;
     }
 
-    gBufferMaxSize.width = PR_MAX(gBufferMaxSize.width, size.width);
-    gBufferMaxSize.height = PR_MAX(gBufferMaxSize.height, size.height);
+    gBufferMaxSize.width = NS_MAX(gBufferMaxSize.width, size.width);
+    gBufferMaxSize.height = NS_MAX(gBufferMaxSize.height, size.height);
 
     // Check if system depth has related gfxImage format
     gfxASurface::gfxImageFormat format =
@@ -465,14 +464,8 @@ nsWindow::SetParent(nsIWidget *aNewParent)
     if (parent) {
         parent->RemoveChild(this);
     }
-    if (aNewParent) {
-        ReparentNativeWidget(aNewParent);
-        aNewParent->AddChild(this);
-        return NS_OK;
-    }
-    if (mWidget) {
-        mWidget->setParentItem(0);
-    }
+    ReparentNativeWidget(aNewParent);
+    aNewParent->AddChild(this);
     return NS_OK;
 }
 
@@ -2503,15 +2496,9 @@ nsresult
 initialize_prefs(void)
 {
     // check to see if we should set our raise pref
-    nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-    if (!prefs)
-        return NS_OK;
-
-    PRBool val = PR_TRUE;
-    nsresult rv;
-    rv = prefs->GetBoolPref("mozilla.widget.disable-native-theme", &val);
-    if (NS_SUCCEEDED(rv))
-        gDisableNativeTheme = val;
+    gDisableNativeTheme =
+        Preferences::GetBool("mozilla.widget.disable-native-theme",
+                             gDisableNativeTheme);
 
     return NS_OK;
 }
@@ -2609,7 +2596,12 @@ nsWindow::createQWidget(MozQWidget *parent, nsWidgetInitData *aInitData)
     // create a QGraphicsView if this is a new toplevel window
 
     if (mIsTopLevel) {
-        QGraphicsView* newView = new MozQGraphicsView(widget, parentWidget);
+        QGraphicsView* newView = nsnull;
+#if defined MOZ_ENABLE_MEEGOTOUCH
+        newView = new MozMGraphicsView(widget, parentWidget);
+#else
+        newView = new MozQGraphicsView(widget, parentWidget);
+#endif
 
         if (!newView) {
             delete widget;
@@ -3053,11 +3045,8 @@ nsWindow::SetInputMode(const IMEContext& aContext)
         case nsIWidget::IME_STATUS_ENABLED:
         case nsIWidget::IME_STATUS_PASSWORD:
             {
-                PRInt32 openDelay = 200;
-                nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-                if (prefs)
-                  prefs->GetIntPref("ui.vkb.open.delay", &openDelay);
-
+                PRInt32 openDelay =
+                    Preferences::GetInt("ui.vkb.open.delay", 200);
                 mWidget->requestVKB(openDelay);
             }
             break;
