@@ -53,10 +53,8 @@
  **************************************************************
  **************************************************************/
 
-#ifdef MOZ_IPC
 #include "mozilla/plugins/PluginInstanceParent.h"
 using mozilla::plugins::PluginInstanceParent;
-#endif
 
 #include "nsWindowGfx.h"
 #include <windows.h>
@@ -66,8 +64,7 @@ using mozilla::plugins::PluginInstanceParent;
 #include "gfxWindowsPlatform.h"
 #include "nsGfxCIID.h"
 #include "gfxContext.h"
-#include "nsIRenderingContext.h"
-#include "nsIDeviceContext.h"
+#include "nsRenderingContext.h"
 #include "prmem.h"
 
 #include "LayerManagerOGL.h"
@@ -79,10 +76,8 @@ using mozilla::plugins::PluginInstanceParent;
 #include "LayerManagerD3D10.h"
 #endif
 
-#ifndef WINCE
 #include "nsUXThemeData.h"
 #include "nsUXThemeConstants.h"
-#endif
 
 extern "C" {
 #include "pixman.h"
@@ -109,15 +104,6 @@ using namespace mozilla::layers;
 static nsAutoPtr<PRUint8>  sSharedSurfaceData;
 static gfxIntSize          sSharedSurfaceSize;
 
-/**************************************************************
- *
- * SECTION: global variables.
- *
- **************************************************************/
-
-static NS_DEFINE_CID(kRegionCID,                  NS_REGION_CID);
-static NS_DEFINE_IID(kRenderingContextCID,        NS_RENDERING_CONTEXT_CID);
-
 /**************************************************************
  **************************************************************
  **
@@ -174,25 +160,7 @@ nsWindowGfx::ConvertHRGNToRegion(HRGN aRgn)
  **************************************************************
  **************************************************************/
 
-void nsWindowGfx::OnSettingsChangeGfx(WPARAM wParam)
-{
-#if defined(WINCE_WINDOWS_MOBILE)
-  if (wParam == SETTINGCHANGE_RESET) {
-    if (glpDDSecondary) {
-      glpDDSecondary->Release();
-      glpDDSecondary = NULL;
-    }
-
-    if(glpDD)
-      glpDD->RestoreAllSurfaces();
-  }
-#endif
-}
-
 // GetRegionToPaint returns the invalidated region that needs to be painted
-// it's abstracted out because Windows XP/Vista/7 handles this for us, but
-// we need to keep track of it our selves for Windows CE and Windows Mobile
-
 nsIntRegion nsWindow::GetRegionToPaint(PRBool aForceFullRepaint,
                                        PAINTSTRUCT ps, HDC aDC)
 {
@@ -202,14 +170,9 @@ nsIntRegion nsWindow::GetRegionToPaint(PRBool aForceFullRepaint,
     return nsIntRegion(nsWindowGfx::ToIntRect(paintRect));
   }
 
-#if defined(WINCE_WINDOWS_MOBILE) || !defined(WINCE)
   HRGN paintRgn = ::CreateRectRgn(0, 0, 0, 0);
   if (paintRgn != NULL) {
-# ifdef WINCE
-    int result = GetUpdateRgn(mWnd, paintRgn, FALSE);
-# else
     int result = GetRandomRgn(aDC, paintRgn, SYSRGN);
-# endif
     if (result == 1) {
       POINT pt = {0,0};
       ::MapWindowPoints(NULL, mWnd, &pt, 1);
@@ -217,12 +180,8 @@ nsIntRegion nsWindow::GetRegionToPaint(PRBool aForceFullRepaint,
     }
     nsIntRegion rgn(nsWindowGfx::ConvertHRGNToRegion(paintRgn));
     ::DeleteObject(paintRgn);
-# ifdef WINCE
-    if (!rgn.IsEmpty())
-# endif
-      return rgn;
+    return rgn;
   }
-#endif
   return nsIntRegion(nsWindowGfx::ToIntRect(ps.rcPaint));
 }
 
@@ -251,7 +210,6 @@ EnsureSharedSurfaceSize(gfxIntSize size)
 
 PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
 {
-#ifdef MOZ_IPC
   // We never have reentrant paint events, except when we're running our RPC
   // windows event spin loop. If we don't trap for this, we'll try to paint,
   // but view manager will refuse to paint the surface, resulting is black
@@ -285,16 +243,6 @@ PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
       return PR_TRUE;
     }
   }
-#endif
-
-#ifdef MOZ_IPC
-  // We never have reentrant paint events, except when we're running our RPC
-  // windows event spin loop. If we don't trap for this, we'll try to paint,
-  // but view manager will refuse to paint the surface, resulting is black
-  // flashes on the plugin rendering surface.
-  if (mozilla::ipc::RPCChannel::IsSpinLoopActive() && mPainting)
-    return PR_FALSE;
-#endif
 
   nsPaintEvent willPaintEvent(PR_TRUE, NS_WILL_PAINT, this);
   willPaintEvent.willSendDidPaint = PR_TRUE;
@@ -448,15 +396,12 @@ PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
             thebesContext->Paint();
             thebesContext->SetOperator(gfxContext::OPERATOR_OVER);
           }
-#ifdef WINCE
-          thebesContext->SetFlag(gfxContext::FLAG_SIMPLIFY_OPERATORS);
-#endif
 
           // don't need to double buffer with anything but GDI
           BasicLayerManager::BufferMode doubleBuffering =
             BasicLayerManager::BUFFER_NONE;
           if (IsRenderMode(gfxWindowsPlatform::RENDER_GDI)) {
-# if defined(MOZ_XUL) && !defined(WINCE)
+#ifdef MOZ_XUL
             switch (mTransparencyMode) {
               case eTransparencyGlass:
               case eTransparencyBorderlessGlass:
@@ -501,11 +446,8 @@ PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
           }
 #endif
           if (result) {
-            if (IsRenderMode(gfxWindowsPlatform::RENDER_DDRAW) ||
-                       IsRenderMode(gfxWindowsPlatform::RENDER_DDRAW_GL))
-            {
-            } else if (IsRenderMode(gfxWindowsPlatform::RENDER_IMAGE_STRETCH24) ||
-                       IsRenderMode(gfxWindowsPlatform::RENDER_IMAGE_STRETCH32)) 
+            if (IsRenderMode(gfxWindowsPlatform::RENDER_IMAGE_STRETCH24) ||
+                IsRenderMode(gfxWindowsPlatform::RENDER_IMAGE_STRETCH32))
             {
               gfxIntSize surfaceSize = targetSurfaceImage->GetSize();
 
@@ -643,7 +585,7 @@ PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
 
   mPaintDC = nsnull;
 
-#if defined(WIDGET_DEBUG_OUTPUT) && !defined(WINCE)
+#if defined(WIDGET_DEBUG_OUTPUT)
   if (debug_WantPaintFlashing())
   {
     // Only flash paint events which have not ignored the paint message.
@@ -658,7 +600,7 @@ PRBool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
     ::ReleaseDC(mWnd, debugPaintFlashDC);
     ::DeleteObject(debugPaintFlashRegion);
   }
-#endif // WIDGET_DEBUG_OUTPUT && !WINCE
+#endif // WIDGET_DEBUG_OUTPUT
 
   mPainting = PR_FALSE;
 
@@ -751,9 +693,6 @@ PRUint8* nsWindowGfx::Data32BitTo1Bit(PRUint8* aImageData,
 
 PRBool nsWindowGfx::IsCursorTranslucencySupported()
 {
-#ifdef WINCE
-  return PR_FALSE;
-#else
   static PRBool didCheck = PR_FALSE;
   static PRBool isSupported = PR_FALSE;
   if (!didCheck) {
@@ -763,7 +702,6 @@ PRBool nsWindowGfx::IsCursorTranslucencySupported()
   }
 
   return isSupported;
-#endif
 }
 
 /**
@@ -786,7 +724,6 @@ HBITMAP nsWindowGfx::DataToBitmap(PRUint8* aImageData,
                                   PRUint32 aHeight,
                                   PRUint32 aDepth)
 {
-#ifndef WINCE
   HDC dc = ::GetDC(NULL);
 
   if (aDepth == 32 && IsCursorTranslucencySupported()) {
@@ -847,7 +784,4 @@ HBITMAP nsWindowGfx::DataToBitmap(PRUint8* aImageData,
   HBITMAP bmp = ::CreateDIBitmap(dc, &head, CBM_INIT, aImageData, &bi, DIB_RGB_COLORS);
   ::ReleaseDC(NULL, dc);
   return bmp;
-#else
-  return nsnull;
-#endif
 }
