@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:expandtab:shiftwidth=4:tabstop=4:
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim:expandtab:shiftwidth=2:tabstop=8:
  */
 /* vim:set ts=8 sw=2 et cindent: */
 /* ***** BEGIN LICENSE BLOCK *****
@@ -53,10 +53,8 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <limits.h>
 #include <X11/Xatom.h>
-#ifdef POLL_WITH_XCONNECTIONNUMBER
-#include <poll.h>
-#endif
 
 #define MOZILLA_VERSION_PROP   "_MOZILLA_VERSION"
 #define MOZILLA_LOCK_PROP      "_MOZILLA_LOCK"
@@ -76,13 +74,20 @@
 #endif
     
 #ifndef MAX_PATH
+#ifdef PATH_MAX
+#define MAX_PATH PATH_MAX
+#else
 #define MAX_PATH 1024
 #endif
+#endif
 
 #define ARRAY_LENGTH(array_) (sizeof(array_)/sizeof(array_[0]))
 
 static PRLogModuleInfo *sRemoteLm = NULL;
 
+static int (*sOldHandler)(Display *, XErrorEvent *);
+static PRBool sGotBadWindow;
+
 XRemoteClient::XRemoteClient()
 {
   mDisplay = 0;
@@ -178,40 +183,10 @@ XRemoteClient::SendCommand (const char *aProgram, const char *aUsername,
 {
   PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("XRemoteClient::SendCommand"));
 
-  *aWindowFound = PR_FALSE;
-
-  Window w = FindBestWindow(aProgram, aUsername, aProfile, PR_FALSE);
-
-  nsresult rv = NS_OK;
-
-  if (w) {
-    // ok, let the caller know that we at least found a window.
-    *aWindowFound = PR_TRUE;
-
-    // make sure we get the right events on that window
-    XSelectInput(mDisplay, w,
-                 (PropertyChangeMask|StructureNotifyMask));
-        
-    PRBool destroyed = PR_FALSE;
-
-    // get the lock on the window
-    rv = GetLock(w, &destroyed);
-
-    if (NS_SUCCEEDED(rv)) {
-      // send our command
-      rv = DoSendCommand(w, aCommand, aDesktopStartupID, aResponse, &destroyed);
-
-      // if the window was destroyed, don't bother trying to free the
-      // lock.
-      if (!destroyed)
-          FreeLock(w); // doesn't really matter what this returns
-
-    }
-  }
-
-  PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("SendCommand returning 0x%x\n", rv));
-
-  return rv;
+  return SendCommandInternal(aProgram, aUsername, aProfile,
+                             aCommand, 0, nsnull,
+                             aDesktopStartupID,
+                             aResponse, aWindowFound);
 }
 
 nsresult
@@ -223,9 +198,39 @@ XRemoteClient::SendCommandLine (const char *aProgram, const char *aUsername,
 {
   PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("XRemoteClient::SendCommandLine"));
 
-  *aWindowFound = PR_FALSE;
+  return SendCommandInternal(aProgram, aUsername, aProfile,
+                             nsnull, argc, argv,
+                             aDesktopStartupID,
+                             aResponse, aWindowFound);
+}
 
-  Window w = FindBestWindow(aProgram, aUsername, aProfile, PR_TRUE);
+static int
+HandleBadWindow(Display *display, XErrorEvent *event)
+{
+  if (event->error_code == BadWindow) {
+    sGotBadWindow = PR_TRUE;
+    return 0; // ignored
+  }
+  else {
+    return (*sOldHandler)(display, event);
+  }
+}
+
+nsresult
+XRemoteClient::SendCommandInternal(const char *aProgram, const char *aUsername,
+                                   const char *aProfile, const char *aCommand,
+                                   PRInt32 argc, char **argv,
+                                   const char* aDesktopStartupID,
+                                   char **aResponse, PRBool *aWindowFound)
+{
+  *aWindowFound = PR_FALSE;
+  PRBool isCommandLine = !aCommand;
+
+  // FindBestWindow() iterates down the window hierarchy, so catch X errors
+  // when windows get destroyed before being accessed.
+  sOldHandler = XSetErrorHandler(HandleBadWindow);
+
+  Window w = FindBestWindow(aProgram, aUsername, aProfile, isCommandLine);
 
   nsresult rv = NS_OK;
 
@@ -233,10 +238,15 @@ XRemoteClient::SendCommandLine (const char *aProgram, const char *aUsername,
     // ok, let the caller know that we at least found a window.
     *aWindowFound = PR_TRUE;
 
+    // Ignore BadWindow errors up to this point.  The last request from
+    // FindBestWindow() was a synchronous XGetWindowProperty(), so no need to
+    // Sync.  Leave the error handler installed to detect if w gets destroyed.
+    sGotBadWindow = PR_FALSE;
+
     // make sure we get the right events on that window
     XSelectInput(mDisplay, w,
                  (PropertyChangeMask|StructureNotifyMask));
-        
+
     PRBool destroyed = PR_FALSE;
 
     // get the lock on the window
@@ -244,7 +254,14 @@ XRemoteClient::SendCommandLine (const char *aProgram, const char *aUsername,
 
     if (NS_SUCCEEDED(rv)) {
       // send our command
-      rv = DoSendCommandLine(w, argc, argv, aDesktopStartupID, aResponse, &destroyed);
+      if (isCommandLine) {
+        rv = DoSendCommandLine(w, argc, argv, aDesktopStartupID, aResponse,
+                               &destroyed);
+      }
+      else {
+        rv = DoSendCommand(w, aCommand, aDesktopStartupID, aResponse,
+                           &destroyed);
+      }
 
       // if the window was destroyed, don't bother trying to free the
       // lock.
@@ -254,7 +271,9 @@ XRemoteClient::SendCommandLine (const char *aProgram, const char *aUsername,
     }
   }
 
-  PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("SendCommandLine returning 0x%x\n", rv));
+  XSetErrorHandler(sOldHandler);
+
+  PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("SendCommandInternal returning 0x%x\n", rv));
 
   return rv;
 }
@@ -333,6 +352,8 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
   PRBool waited = PR_FALSE;
   *aDestroyed = PR_FALSE;
 
+  nsresult rv = NS_OK;
+
   if (!mLockData) {
     
     char pidstr[32];
@@ -372,7 +393,16 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
 				 &actual_type, &actual_format,
 				 &nitems, &bytes_after,
 				 &data);
-    if (result != Success || actual_type == None) {
+
+    // aWindow may have been destroyed before XSelectInput was processed, in
+    // which case there may not be any DestroyNotify event in the queue to
+    // tell us.  XGetWindowProperty() was synchronous so error responses have
+    // now been processed, setting sGotBadWindow.
+    if (sGotBadWindow) {
+      *aDestroyed = PR_TRUE;
+      rv = NS_ERROR_FAILURE;
+    }
+    else if (result != Success || actual_type == None) {
       /* It's not now locked - lock it. */
       XChangeProperty (mDisplay, aWindow, mMozLockAtom, XA_STRING, 8,
 		       PropModeReplace,
@@ -382,9 +412,9 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
     }
 
     XUngrabServer(mDisplay);
-    XSync(mDisplay, False);
+    XFlush(mDisplay); // ungrab now!
 
-    if (!locked) {
+    if (!locked && !NS_FAILED(rv)) {
       /* We tried to grab the lock this time, and failed because someone
 	 else is holding it already.  So, wait for a PropertyDelete event
 	 to come in, and try again. */
@@ -395,12 +425,6 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
       while (1) {
 	XEvent event;
 	int select_retval;
-#ifdef POLL_WITH_XCONNECTIONNUMBER
-       struct pollfd fds[1];
-       fds[0].fd = XConnectionNumber(mDisplay);
-       fds[0].events = POLLIN;
-       select_retval = poll(fds,1,10*1000);
-#else
 	fd_set select_set;
 	struct timeval delay;
 	delay.tv_sec = 10;
@@ -411,21 +435,19 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
 	FD_SET(ConnectionNumber(mDisplay), &select_set);
 	select_retval = select(ConnectionNumber(mDisplay) + 1,
 			       &select_set, NULL, NULL, &delay);
-#endif
 	// did we time out?
 	if (select_retval == 0) {
 	  PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("timed out waiting for window\n"));
-	  return NS_ERROR_FAILURE;
+          rv = NS_ERROR_FAILURE;
+          break;
 	}
 	PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("xevent...\n"));
 	XNextEvent (mDisplay, &event);
 	if (event.xany.type == DestroyNotify &&
 	    event.xdestroywindow.window == aWindow) {
-	  PR_LOG(sRemoteLm, PR_LOG_DEBUG,
-		 ("window 0x%x unexpectedly destroyed.\n",
-		  (unsigned int) aWindow));
 	  *aDestroyed = PR_TRUE;
-	  return NS_ERROR_FAILURE;
+          rv = NS_ERROR_FAILURE;
+          break;
 	}
 	else if (event.xany.type == PropertyNotify &&
 		 event.xproperty.state == PropertyDelete &&
@@ -442,13 +464,17 @@ XRemoteClient::GetLock(Window aWindow, PRBool *aDestroyed)
     }
     if (data)
       XFree(data);
-  } while (!locked);
+  } while (!locked && !NS_FAILED(rv));
 
-  if (waited) {
+  if (waited && locked) {
     PR_LOG(sRemoteLm, PR_LOG_DEBUG, ("obtained lock.\n"));
+  } else if (*aDestroyed) {
+    PR_LOG(sRemoteLm, PR_LOG_DEBUG,
+           ("window 0x%x unexpectedly destroyed.\n",
+            (unsigned int) aWindow));
   }
 
-  return NS_OK;
+  return rv;
 }
 
 Window
@@ -460,7 +486,6 @@ XRemoteClient::FindBestWindow(const char *aProgram, const char *aUsername,
   Window bestWindow = 0;
   Window root2, parent, *kids;
   unsigned int nkids;
-  int i;
 
   // Get a list of the children of the root window, walk the list
   // looking for the best window that fits the criteria.
@@ -478,7 +503,7 @@ XRemoteClient::FindBestWindow(const char *aProgram, const char *aUsername,
   // We'll walk the list of windows looking for a window that best
   // fits the criteria here.
 
-  for (i=nkids-1; i >= 0; i--) {
+  for (unsigned int i = 0; i < nkids; i++) {
     Atom type;
     int format;
     unsigned long nitems, bytesafter;
@@ -591,9 +616,10 @@ XRemoteClient::FindBestWindow(const char *aProgram, const char *aUsername,
     // Check to see if the window supports the new command-line passing
     // protocol, if that is requested.
 
-    // If we got this far, this is the best window so far.  It passed
+    // If we got this far, this is the best window.  It passed
     // all the tests.
     bestWindow = w;
+    break;
   }
 
   if (kids)
@@ -699,8 +725,6 @@ XRemoteClient::DoSendCommandLine(Window aWindow, PRInt32 argc, char **argv,
                                  const char* aDesktopStartupID,
                                  char **aResponse, PRBool *aDestroyed)
 {
-  int i;
-
   *aDestroyed = PR_FALSE;
 
   char cwdbuf[MAX_PATH];
@@ -716,7 +740,7 @@ XRemoteClient::DoSendCommandLine(Window aWindow, PRInt32 argc, char **argv,
   static char desktopStartupPrefix[] = " DESKTOP_STARTUP_ID=";
 
   PRInt32 argvlen = strlen(cwdbuf);
-  for (i = 0; i < argc; ++i) {
+  for (int i = 0; i < argc; ++i) {
     PRInt32 len = strlen(argv[i]);
     if (i == 0 && aDesktopStartupID) {
       len += sizeof(desktopStartupPrefix) - 1 + strlen(aDesktopStartupID);
